@@ -111,21 +111,26 @@ class DistUpgradeQuirks(object):
         logging.debug("running Quirks.PreCacheOpen")
 
     # individual quirks handler that run *after* the cache is opened
-    def impishPostInitialUpdate(self):
+    def jammyPostInitialUpdate(self):
         # PreCacheOpen would be better but controller.abort fails terribly
         """ run after the apt cache is opened the first time """
-        logging.debug("running Quirks.impishPostInitialUpdate")
+        logging.debug("running Quirks.jammyPostInitialUpdate")
         self._get_from_and_to_version()
         self._test_and_fail_on_i386()
         self._test_and_fail_on_aufs()
+        self._test_and_fail_on_power8()
+        self._test_and_fail_on_pam_tally()
 
         cache = self.controller.cache
         self._test_and_warn_if_ros_installed(cache)
+        self._fix_missing_nvidia_metapackage()
 
         if 'snapd' not in cache:
             logging.debug("package required for Quirk not in cache")
             return
-        if cache['snapd'].is_installed:
+        if cache['snapd'].is_installed and \
+                (os.path.exists('/run/snapd.socket') or
+                 os.path.exists('/run/snapd-snap.socket')):
             self._checkStoreConnectivity()
         # If the snap store is accessible, at the same time calculate the
         # extra size needed by to-be-installed snaps.  This also prepares
@@ -133,8 +138,8 @@ class DistUpgradeQuirks(object):
         if self._snapstore_reachable:
             self._calculateSnapSizeRequirements()
 
-    def impishPostUpgrade(self):
-        logging.debug("running Quirks.impishPostUpgrade")
+    def jammyPostUpgrade(self):
+        logging.debug("running Quirks.jammyPostUpgrade")
         cache = self.controller.cache
         if 'snapd' not in cache:
             logging.debug("package required for Quirk not in cache")
@@ -148,6 +153,7 @@ class DistUpgradeQuirks(object):
         if 'linux-firmware-raspi2' in cache:
             if cache['linux-firmware-raspi2'].is_installed:
                 self._remove_uboot_on_rpi()
+                self._remove_netplan_match_rpi()
 
     # individual quirks handler when the dpkg run is finished ---------
     def PostCleanup(self):
@@ -161,7 +167,6 @@ class DistUpgradeQuirks(object):
         self._removeOldApportCrashes()
         self._killUpdateNotifier()
         self._killKBluetooth()
-        self._killScreensaver()
         self._pokeScreensaver()
         self._stopDocvertConverter()
 
@@ -369,6 +374,73 @@ class DistUpgradeQuirks(object):
                 self._view.error(summary, msg)
                 self.controller.abort()
 
+    def _test_and_fail_on_pam_tally(self):
+        """
+        Test and fail if pam_tally or pam_tally2 appears in the system's
+        PAM config for any service.  These modules have been dropped as
+        obsolete, so references to them in the config will break services
+        on upgrade.  The libpam-modules package has a preinst that will also
+        enforce this, but we want to enforce at the start of the upgrade
+        before we get halfway through an apt transaction.  LP: #1977493
+        """
+
+        for f in os.listdir('/etc/pam.d'):
+            if f in ('common-account', 'common-auth', 'common-password',
+                     'common-session', 'common-session-noninteractive'):
+                # managed by pam-auth-updates, and any references to
+                # pam_tally* will be handled by libpam-modules on upgrade
+                # without breaking
+                continue
+            with open(os.path.join('/etc/pam.d', f)) as f:
+                content = f.read()
+                if re.search('^[^#]*pam_tally', content, re.MULTILINE):
+                    logging.error("pam_tally* in use")
+                    # from libpam-modules.templates
+                    summary = _("You are using pam_tally or pam_tally2 in "
+                                "your configuration.")
+                    msg = _("The pam_tally and pam_tally2 modules have been "
+                            "removed from PAM. You are using one of these "
+                            "modules in your PAM configuration in /etc/pam.d. "
+                            "You must remove the uses of these modules before "
+                            "PAM can be upgraded; including these modules in "
+                            "your PAM configuration after the upgrade will "
+                            "stop users from being able to log into the "
+                            "system.") + "\n\n" + \
+                        _("Consider the pam_faillock module as a "
+                          "replacement for pam_tally.")
+                    self._view.error(summary, msg)
+                    self.controller.abort()
+                    return
+
+    def _test_and_fail_on_power8(self):
+        """
+        Test and fail if running on a POWER8 processor. Starting with Ubuntu
+        22.04, ppc64el is compiled for POWER9 processors. Prior to 22.04 it
+        was/is compiled for POWER8 processors. Hence POWER8 processors are not
+        able to run, install or upgrade to jammy.
+        """
+        if self.arch != "ppc64el":
+            return
+
+        updates_end = ""
+        if self._from_version == "20.04":
+            updates_end = "April 2025"
+        elif self._from_version == "21.10":
+            updates_end = "July 2022"
+
+        with open("/proc/cpuinfo") as f:
+            cpuinfo = f.read()
+
+            if re.search("^cpu\s*:\s*POWER8", cpuinfo, re.MULTILINE):
+                logging.error("POWER8 processor detected")
+                summary = _("Sorry, no more upgrades for this system")
+                msg = _("There will not be any further Ubuntu releases "
+                        "for this system's POWER8 processor.\n\n"
+                        "Updates for Ubuntu %s will continue until %s." %
+                        (self._from_version, updates_end))
+                self._view.error(summary, msg)
+                self.controller.abort()
+
     def _test_and_warn_if_vserver(self):
         """
         upstart and vserver environments are not a good match, warn
@@ -491,12 +563,6 @@ class DistUpgradeQuirks(object):
             logging.debug("killing kblueplugd kbluetooth4")
             subprocess.call(["killall", "-q", "kblueplugd", "kbluetooth4"])
 
-    def _killScreensaver(self):
-        """killall gnome-screensaver """
-        if os.path.exists("/usr/bin/killall"):
-            logging.debug("killing gnome-screensaver")
-            subprocess.call(["killall", "-q", "gnome-screensaver"])
-
     def _pokeScreensaver(self):
         if (os.path.exists("/usr/bin/xdg-screensaver") and
                 os.environ.get('DISPLAY')):
@@ -1017,6 +1083,25 @@ class DistUpgradeQuirks(object):
                 logging.warning(msg)
                 apt.apt_pkg.config.set("Apt::Install-Recommends", "1")
 
+    def _is_deb2snap_metapkg_installed(self, deb2snap_entry):
+        """ Helper function that checks if the given deb2snap entry
+            has at least one metapkg which is installed on the system.
+        """
+        metapkg_list = deb2snap_entry.get("metapkg", None)
+        if not isinstance(metapkg_list, list):
+            metapkg_list = [metapkg_list]
+
+        for metapkg in metapkg_list:
+            if metapkg not in self.controller.cache:
+                continue
+            if metapkg and \
+                    self.controller.cache[metapkg].is_installed is False:
+                continue
+
+            return True
+
+        return False
+
     def _prepare_snap_replacement_data(self):
         """ Helper function fetching all required info for the deb-to-snap
             migration: version strings for upgrade (from and to) and the list
@@ -1037,11 +1122,7 @@ class DistUpgradeQuirks(object):
 
             for snap in d2s["seeded"]:
                 seed = d2s["seeded"][snap]
-                metapkg = seed.get("metapkg", None)
-                if metapkg not in self.controller.cache:
-                    continue
-                if metapkg and \
-                        self.controller.cache[metapkg].is_installed is False:
+                if not self._is_deb2snap_metapkg_installed(seed):
                     continue
                 deb = seed.get("deb", None)
                 from_chan = seed.get("from_channel", from_channel)
@@ -1051,11 +1132,7 @@ class DistUpgradeQuirks(object):
             for snap in d2s["unseeded"]:
                 unseed = d2s["unseeded"][snap]
                 deb = unseed.get("deb", None)
-                metapkg = unseed.get("metapkg", None)
-                if metapkg not in self.controller.cache:
-                    continue
-                if metapkg and \
-                        self.controller.cache[metapkg].is_installed is False:
+                if not self._is_deb2snap_metapkg_installed(unseed):
                     continue
                 from_chan = unseed.get("from_channel", from_channel)
                 unseeded_snaps[snap] = (deb, from_chan)
@@ -1341,3 +1418,116 @@ class DistUpgradeQuirks(object):
         except IOError as exc:
             logging.error("unable to write new boot config to %s: %s; %s",
                           boot_config_filename, exc, failure_action)
+
+    def _remove_netplan_match_rpi(self, netplan_dir='/etc/netplan'):
+        failure_action = (
+            "You may need to manually edit your netplan config "
+            "and remove the line \"driver: bcmgenet smsc95xx lan78xx\". "
+            "See LP: #1970761 for further details")
+
+        try:
+            import yaml
+        except ImportError:
+            logging.error("failed to import yaml library. %s", failure_action)
+            return
+
+        for netplan_config_file in glob.glob("%s/*.yaml" % netplan_dir):
+            try:
+                netplan_config_filename = os.path.join(netplan_dir,
+                                                       netplan_config_file)
+                with open(netplan_config_filename, 'r', encoding='utf-8') as f:
+                    orig_content = f.read()
+                    netplan_yaml = yaml.load(orig_content,
+                                             Loader=yaml.SafeLoader)
+
+            except FileNotFoundError:
+                logging.error("failed to open netplan configuration in %s; %s",
+                              netplan_config_filename, failure_action)
+                return
+
+            file_modified = False
+            try:
+                for eth in netplan_yaml["network"]["ethernets"].values():
+                    if eth["match"]["driver"] == "bcmgenet smsc95xx lan78xx":
+                        file_modified = True
+                        del eth["match"]
+                        if "set-name" in eth:
+                            del eth["set-name"]
+            except KeyError:
+                continue
+
+            if not file_modified:
+                continue
+
+            try:
+                netplan_backup_filename = (netplan_config_filename +
+                                           '.distUpgrade')
+                with open(netplan_backup_filename, 'w', encoding='utf-8') as f:
+                    f.write(orig_content)
+            except IOError as exc:
+                logging.error("unable to write netplan config backup to "
+                              "%s: %s; %s",
+                              netplan_backup_filename, exc, failure_action)
+                return
+            try:
+                with open(netplan_config_filename, 'w', encoding='utf-8') as f:
+                    yaml.dump(netplan_yaml, f, default_flow_style=False)
+            except IOError as exc:
+                logging.error("unable to write new netplan config to "
+                              "%s: %s; %s",
+                              netplan_config_filename, exc, failure_action)
+                return
+
+    def _fix_missing_nvidia_metapackage(self):
+        # LP: #1955047
+        # Desktop installs for 20.04.3 on systems with nvidia graphics don't
+        # have the driver metapackage installed
+        logging.debug(
+            "running Quirks._test_and_fix_missing_nvidia_metapackage")
+        try:
+            from NvidiaDetector.nvidiadetector import NvidiaDetection
+        except (ImportError) as e:
+            logging.error("NvidiaDetection can not be imported %s" % e)
+            return
+        nv = NvidiaDetection()
+        versions = nv.drivers.keys()
+        cache = self.controller.cache
+        # Early exit if there's a driver metapackage installed
+        for version in versions:
+            driver = 'nvidia-driver-{}'.format(version)
+            if driver in cache and cache[driver].is_installed:
+                logging.debug("package {} detected, exit.".format(driver))
+                return
+        version_to_fix = None
+        # If we find an autoremovable libnvidia-gl package, this probably
+        # means the user hit LP: #1955047
+        for version in versions:
+            nvidia_gl = 'libnvidia-gl-{}'.format(version)
+            if nvidia_gl in cache and cache[nvidia_gl].is_installed \
+               and cache[nvidia_gl].is_auto_removable:
+                version_to_fix = version
+                logging.debug("Found autoremovable package {}, assuming "
+                              "we're missing nvidia-driver-{}"
+                              .format(nvidia_gl, version))
+                break
+        if version_to_fix is None:
+            logging.debug("No autoremovable libnvidia-gl found, exiting.")
+            return
+        missing_metapackage = "nvidia-driver-{}".format(version_to_fix)
+        if missing_metapackage not in cache:
+            logging.error("Package {} not available in the APT cache!"
+                          .format(missing_metapackage))
+            summary = _("NVIDIA driver metapackage unavailable")
+            msg = _("Your system has the package libnvidia-gl-%s installed "
+                    "but the corresponding metapackage nvidia-driver-%s is "
+                    "not installed and is not available to be installed. "
+                    "The upgrade process was not able to automatically fix "
+                    "this issue and consequently the upgrade will now exit." %
+                    (version_to_fix, version_to_fix))
+            self._view.error(summary, msg)
+            self.controller.abort()
+            return
+        logging.debug("Installing missing nvidia driver metapackage {}"
+                      .format(missing_metapackage))
+        cache[missing_metapackage].mark_install()
+        cache.commit(None, None)
