@@ -7,6 +7,7 @@
 from __future__ import absolute_import
 
 import gi
+gi.require_version("Gtk", "3.0")
 gi.require_version('Snapd', '1')
 from gi.repository import GLib, Gtk, Snapd
 
@@ -26,7 +27,8 @@ from UpdateManager.Dialogs import Dialog
 
 class InstallBackend(Dialog):
     ACTION_UPDATE = 0
-    ACTION_INSTALL = 1
+    ACTION_PRE_INSTALL = 1
+    ACTION_INSTALL = 2
 
     def __init__(self, window_main, action):
         Dialog.__init__(self, window_main)
@@ -39,7 +41,24 @@ class InstallBackend(Dialog):
         # Do not suspend during the update process
         self.sleep_cookie = inhibit_sleep()
 
-        if self.action == self.ACTION_INSTALL:
+        if self.action == self.ACTION_PRE_INSTALL:
+            unfresh_cache = self.window_main.cache
+            fresh_cache = Cache(rootdir=self.window_main.cache.rootdir)
+            # Install OEM packages, update, then do ACTION_INSTALL
+            pkgs_install_oem = []
+            pkgs_upgrade_oem = []
+            for pkg in self.window_main.oem_metapackages:
+                unfresh_pkg = unfresh_cache[pkg]
+                fresh_pkg = fresh_cache[pkg]
+
+                if (unfresh_pkg.marked_install
+                        and not fresh_pkg.is_installed):
+                    pkgs_install_oem.append(pkg)
+                elif (unfresh_pkg.marked_upgrade
+                      and fresh_pkg.is_upgradable):
+                    pkgs_upgrade_oem.append(pkg)
+            self.commit_oem(pkgs_install_oem, pkgs_upgrade_oem)
+        elif self.action == self.ACTION_INSTALL:
             # Get the packages which should be installed and update
             pkgs_install = []
             pkgs_upgrade = []
@@ -72,6 +91,13 @@ class InstallBackend(Dialog):
         """Run a update to refresh the package list"""
         raise NotImplementedError
 
+    def commit_oem(self, pkgs_install_oem, pkgs_upgrade_oem):
+        """ Install these OEM packages """
+        self._action_done(self.ACTION_PRE_INSTALL,
+                          authorized=True, success=True,
+                          error_string=None, error_desc=None,
+                          trans_failed=None)
+
     def commit(self, pkgs_install, pkgs_upgrade, pkgs_remove):
         """Commit the cache changes """
         raise NotImplementedError
@@ -81,6 +107,7 @@ class InstallBackend(Dialog):
         unseeded_snaps = {}
 
         curr_channel = "stable/ubuntu-" + get_dist_version()
+        cache = self.window_main.cache
 
         try:
             d2s_file = open(
@@ -92,11 +119,21 @@ class InstallBackend(Dialog):
                 seed = d2s["seeded"][snap]
                 deb = seed.get("deb", None)
                 to_channel = seed.get("to_channel", curr_channel)
+                metapkg = seed.get("metapkg", None)
+                if metapkg not in cache:
+                    continue
+                if metapkg and cache[metapkg].is_installed is False:
+                    continue
                 seeded_snaps[snap] = (deb, to_channel)
 
             for snap in d2s["unseeded"]:
                 unseed = d2s["unseeded"][snap]
                 from_channel = unseed.get("from_channel", curr_channel)
+                metapkg = seed.get("metapkg", None)
+                if metapkg not in cache:
+                    continue
+                if metapkg and cache[metapkg].is_installed is False:
+                    continue
                 unseeded_snaps[snap] = (from_channel)
         except Exception as e:
             logging.debug("error reading deb2snap.json file (%s)" % e)
@@ -302,7 +339,11 @@ class InstallBackend(Dialog):
         # FIXME: confirm with mpt whether this should still be a setting
         #close_after_install = False
 
-        if action == self.ACTION_INSTALL:
+        if action == self.ACTION_PRE_INSTALL and success:
+            # Now do the regular updates
+            self.action = self.ACTION_INSTALL
+            self.start()
+        elif action == self.ACTION_INSTALL:
             if (success and os.path.exists("/usr/bin/snap")
                and hasattr(self, 'pane_update_progress')):
                 Thread(target=self.update_snaps).start()
