@@ -58,7 +58,7 @@ def allowed_to_report():
         return True
 
     try:
-        return subprocess.call(["/bin/systemctl", "-q", "is-enabled", "whoopsie.service"]) == 0
+        return subprocess.call(["/bin/systemctl", "-q", "is-enabled", "whoopsie.path"]) == 0
     except OSError:
         return False
 
@@ -492,6 +492,27 @@ def get_boot_id():
     return boot_id
 
 
+def get_process_environ(proc_pid_fd: int) -> dict[str, str]:
+    """Get the process environ from a proc directory file descriptor.
+
+    Raises an OSError in case the environ file could not been read.
+    """
+
+    def opener(path, flags: int) -> int:
+        return os.open(path, flags, dir_fd=proc_pid_fd)
+
+    with open(
+        "environ", encoding="utf-8", errors="replace", opener=opener
+    ) as environ_fd:
+        environ = environ_fd.read().rstrip("\0 ")
+
+    if not environ:
+        return {}
+    return dict(
+        [entry.split("=", 1) for entry in environ.split("\0") if "=" in entry]
+    )
+
+
 def get_process_path(proc_pid_fd=None):
     '''Gets the process path from a proc directory file descriptor'''
 
