@@ -22,6 +22,7 @@ import re
 import stat
 import base64
 import tempfile
+import select
 import shutil
 import locale
 import json
@@ -32,6 +33,7 @@ import apport
 import apport.fileutils
 
 _invalid_key_chars_re = re.compile(r'[^0-9a-zA-Z_.-]')
+_AGENT = None
 
 
 def path_to_key(path):
@@ -60,6 +62,11 @@ def attach_file_if_exists(report, path, key=None, overwrite=True, force_unicode=
     If the contents is valid UTF-8, or force_unicode is True, then the value
     will be a string, otherwise it will be bytes.
     '''
+    # Prevent directory traversal. Do it here too so it won't disclose if
+    # a file exists or not.
+    if "../" in path:
+        return
+
     if not key:
         key = path_to_key(path)
 
@@ -77,8 +84,23 @@ def read_file(path, force_unicode=False):
     instead of failing.
     '''
     try:
-        with open(path, 'rb') as f:
-            contents = f.read().strip()
+        # Prevent directory traversal
+        if "../" in path:
+            return 'Error: invalid path.'
+        fd = os.open(path, os.O_NOFOLLOW | os.O_RDONLY | os.O_NONBLOCK)
+        st = os.fstat(fd)
+        # make sure there are no symlinks in the full path
+        real_path = os.path.realpath(path)
+        if st.st_ino != os.stat(real_path).st_ino or path != real_path:
+            os.close(fd)
+            return 'Error: path contained symlinks.'
+        # make sure the file isn't a FIFO or symlink
+        if stat.S_ISREG(st.st_mode):
+            with os.fdopen(fd, 'rb') as f:
+                contents = f.read().strip()
+        else:
+            os.close(fd)
+            return 'Error: path was not a regular file.'
         if force_unicode:
             return contents.decode('UTF-8', errors='replace')
         try:
@@ -202,14 +224,14 @@ def attach_dmesg(report):
     This will not overwrite already existing information.
     '''
     if not report.get('CurrentDmesg', '').strip():
-        report['CurrentDmesg'] = command_output(['dmesg'])
+        report['CurrentDmesg'] = root_command_output(['dmesg'])
 
 
 def attach_dmi(report):
     dmi_dir = '/sys/class/dmi/id'
     if os.path.isdir(dmi_dir):
         for f in os.listdir(dmi_dir):
-            p = '%s/%s' % (dmi_dir, f)
+            p = os.path.realpath('%s/%s' % (dmi_dir, f))
             st = os.stat(p)
             # ignore the root-only ones, since they have serial numbers
             if not stat.S_ISREG(st.st_mode) or (st.st_mode & 4 == 0):
@@ -257,6 +279,8 @@ def attach_hardware(report):
     report['Lsusb-t'] = command_output(['lsusb', '-t'])
     report['ProcModules'] = command_output(['sort', '/proc/modules'])
     report['UdevDb'] = command_output(['udevadm', 'info', '--export-db'])
+    report['acpidump'] = \
+        root_command_output(["/usr/share/apport/dump_acpi_tables.py"])
 
     # anonymize partition labels
     labels = report['UdevDb']
@@ -335,7 +359,9 @@ def attach_alsa_old(report):
 def attach_alsa(report):
     '''Attach ALSA subsystem information to the report.
     '''
-    if os.path.exists('/usr/share/alsa-base/alsa-info.sh'):
+    if os.path.exists('/usr/sbin/alsa-info'):
+        report['AlsaInfo'] = command_output(['/usr/sbin/alsa-info', '--stdout', '--no-upload'])
+    elif os.path.exists('/usr/share/alsa-base/alsa-info.sh'):
         report['AlsaInfo'] = command_output(['/usr/share/alsa-base/alsa-info.sh', '--stdout', '--no-upload'])
     else:
         attach_alsa_old(report)
@@ -346,6 +372,9 @@ def attach_alsa(report):
     if os.path.exists('/usr/bin/pacmd'):
         report['PulseList'] = command_output(['pacmd', 'list'])
 
+    if os.path.exists('/usr/bin/pa-info'):
+        report['PaInfo'] = command_output(['/usr/bin/pa-info'])
+
     attach_dmi(report)
     attach_dmesg(report)
 
@@ -397,10 +426,57 @@ def command_output(command, input=None, stderr=subprocess.STDOUT,
     return res
 
 
+def _spawn_pkttyagent():
+    global _AGENT
+
+    if _AGENT is not None:
+        return
+    if os.geteuid() == 0:
+        return
+    if not sys.stdin.isatty():
+        return
+    if not os.path.exists('/usr/bin/pkttyagent'):
+        return
+
+    try:
+        (r, w) = os.pipe2(0)
+    except OSError:
+        return
+
+    _AGENT = subprocess.Popen(['pkttyagent', '--notify-fd', str(w), '--fallback'],
+                              close_fds=False,
+                              stdin=subprocess.PIPE,
+                              stdout=subprocess.PIPE)
+
+    os.close(w)
+
+    with select.epoll() as epoll:
+        while True:
+            epoll.register(r, select.EPOLLIN)
+            events = epoll.poll()
+            for fd, event_type in events:
+                if event_type & select.EPOLLHUP:
+                    os.close(r)
+                    return
+    return
+
+
+def kill_pkttyagent():
+    global _AGENT
+
+    if _AGENT is None:
+        return
+
+    _AGENT.terminate()
+    _AGENT.wait()
+    _AGENT = None
+
+
 def _root_command_prefix():
     if os.getuid() == 0:
         return []
     elif os.path.exists('/usr/bin/pkexec'):
+        _spawn_pkttyagent()
         return ['pkexec']
     # the package hook won't have everything it wanted but that's okay
     else:
@@ -418,8 +494,9 @@ def root_command_output(command, input=None, stderr=subprocess.STDOUT, decode_ut
     otherwise left as bytes.
     '''
     assert isinstance(command, list), 'command must be a list'
-    return command_output(_root_command_prefix() + command, input, stderr,
-                          keep_locale=True, decode_utf8=decode_utf8)
+    output = command_output(_root_command_prefix() + command, input, stderr,
+                            keep_locale=True, decode_utf8=decode_utf8)
+    return output
 
 
 def attach_root_command_outputs(report, command_map):
@@ -723,8 +800,8 @@ def attach_mac_events(report, profiles=None):
     aa_re = re.compile(aa_regex, re.IGNORECASE)
 
     if 'KernLog' not in report:
-        report['KernLog'] = __filter_re_process(
-            mac_re, subprocess.Popen(['dmesg'], stdout=subprocess.PIPE))
+        report['KernLog'] = '\n'.join(re.findall(
+            mac_re, root_command_output(['dmesg'])))
 
     if 'AuditLog' not in report and os.path.exists('/var/run/auditd.pid'):
         attach_root_command_outputs(report, {'AuditLog': 'egrep "' + mac_regex + '" /var/log/audit/audit.log'})
@@ -932,16 +1009,17 @@ def attach_default_grub(report, key=None):
 
 def attach_casper_md5check(report, location):
     '''attach the results of the casper md5check of install media'''
-    result = 'skip'
+    result = 'unknown'
     mismatches = []
     if os.path.exists(location):
-        with open(location) as json_file:
-            check = json.load(json_file)
-            result = check['result']
-            mismatches = check['checksum_missmatch']
+        attach_root_command_outputs(report, {'CasperMD5json': "cat '%s'" % location})
+        check = json.loads(report['CasperMD5json'])
+        result = check['result']
+        mismatches = check['checksum_missmatch']
     report['CasperMD5CheckResult'] = result
     if mismatches:
         report['CasperMD5CheckMismatches'] = ' '.join(mismatches)
+    report.pop('CasperMD5json', None)
 
 
 # backwards compatible API
