@@ -9,8 +9,8 @@
 # option) any later version.  See http://www.gnu.org/copyleft/gpl.html for
 # the full text of the license.
 
-import subprocess, tempfile, os.path, re, pwd, grp, os, time, io
-import fnmatch, glob, traceback, errno, sys, atexit, locale, imp
+import subprocess, tempfile, os.path, re, pwd, grp, os, io
+import fnmatch, glob, traceback, errno, sys, atexit, imp, stat
 
 import xml.dom, xml.dom.minidom
 from xml.parsers.expat import ExpatError
@@ -31,6 +31,7 @@ import problem_report
 import apport
 import apport.fileutils
 from apport.packaging_impl import impl as packaging
+from apport.hookutils import kill_pkttyagent
 
 _data_dir = os.environ.get('APPORT_DATA_DIR', '/usr/share/apport')
 _hook_dir = '%s/package-hooks/' % (_data_dir)
@@ -386,6 +387,29 @@ class Report(problem_report.ProblemReport):
         '''
         self['Snap'] = '%s %s (%s)' % (snap.get("name"), snap.get("version"),
                                        snap.get("channel", "unknown"))
+        snapname = snap.get("name")
+        self['Snap.Changes'] = \
+            _command_output(['snap', 'changes', '--abs-time', snapname])
+        self['Snap.Connections'] = \
+            _command_output(['snap', 'connections', snapname])
+        self['Snap.Info.%s' % snapname] = \
+            _command_output(['snap', 'info', '--abs-time', snapname])
+        import yaml
+        with open('/snap/%s/current/meta/snap.yaml' % snapname) as f:
+            snap_meta = yaml.safe_load(f)
+        if 'base' in snap_meta:
+            base = snap_meta['base']
+            self['Snap.Info.%s' % base] = \
+                _command_output(['snap', 'info', '--abs-time', base])
+        providers = []
+        if 'plugs' in snap_meta:
+            for plug in snap_meta['plugs']:
+                dp = snap_meta['plugs'][plug].get('default-provider')
+                if dp and dp not in providers:
+                    providers.append(dp)
+        for provider in providers:
+            self['Snap.Info.%s' % provider] = \
+                _command_output(['snap', 'info', '--abs-time', provider])
         # Automatically handle snaps which have a Launchpad contact defined
         if snap.get("contact"):
             # Parse Launchpad project (e.g. 'subiquity') or source package string
@@ -421,7 +445,8 @@ class Report(problem_report.ProblemReport):
         This adds:
         - UserGroups: system groups the user is in
         '''
-        user = pwd.getpwuid(os.getuid())[0]
+        # Use effective uid in case privileges were dropped
+        user = pwd.getpwuid(os.geteuid())[0]
         groups = [name for name, p, gid, memb in grp.getgrall()
                   if user in memb and gid < 1000]
         groups.sort()
@@ -802,11 +827,13 @@ class Report(problem_report.ProblemReport):
 
         # check for truncated stack trace
         if 'is truncated: expected core file size' in out or \
-                'is not a core dump: file truncated' in out:
+                'is not a core dump' in out:
             if 'warning:' in out:
-                warnings = '\n'.join([l for l in out.splitlines() if 'warning:' in l])
+                warnings = '\n'.join([line for line in out.splitlines() if
+                                      'warning:' in line])
             elif 'Warning:' in out:
-                warnings = '\n'.join([l for l in out.splitlines() if 'Warning:' in l])
+                warnings = '\n'.join([line for line in out.splitlines() if
+                                      'Warning:' in line])
             else:
                 warnings = out.splitlines()[0]
             reason = 'Invalid core dump: ' + warnings.strip()
@@ -928,6 +955,11 @@ class Report(problem_report.ProblemReport):
         return True if the hook requested to stop the report filing process,
         False otherwise.
         '''
+        ret = self._add_hooks_info(ui, package, srcpackage)
+        kill_pkttyagent()
+        return ret
+
+    def _add_hooks_info(self, ui, package, srcpackage):
         # determine package names, unless already given as arguments
         # avoid path traversal
         if not package:
@@ -1036,31 +1068,37 @@ class Report(problem_report.ProblemReport):
 
         Raises ValueError if the file exists but is invalid XML.
         '''
-        orig_home = os.getenv('HOME')
-        if orig_home is not None:
-            del os.environ['HOME']
-        ifpath = os.path.expanduser(_ignore_file)
-        if orig_home is not None:
-            os.environ['HOME'] = orig_home
-        euid = os.geteuid()
+        # Properly handle dropped privileges
+        homedir = pwd.getpwuid(os.geteuid())[5]
+        ifpath = _ignore_file.replace("~", homedir)
+
+        contents = ''
+        fd = None
+        f = None
+
         try:
-            # drop permissions temporarily to try open users ignore file
-            os.seteuid(os.getuid())
-            fp = open(ifpath, 'r')
+            fd = os.open(ifpath, os.O_NOFOLLOW | os.O_RDONLY)
+            st = os.fstat(fd)
+            if stat.S_ISREG(st.st_mode):
+                f = os.fdopen(fd, 'r')
+                # Limit size to prevent DoS
+                contents = f.read(50000)
         except (IOError, OSError):
-            fp = None
+            pass
         finally:
-            os.seteuid(euid)
-        if fp is None or os.fstat(fp.fileno()).st_size == 0:
+            if f is not None:
+                f.close()
+            elif fd is not None:
+                os.close(fd)
+
+        if contents == '':
             # create a document from scratch
             dom = xml.dom.getDOMImplementation().createDocument(None, 'apport', None)
         else:
             try:
-                dom = xml.dom.minidom.parse(fp)
+                dom = xml.dom.minidom.parseString(contents)
             except ExpatError as e:
                 raise ValueError('%s has invalid format: %s' % (_ignore_file, str(e)))
-        if fp is not None:
-            fp.close()
         # remove whitespace so that writing back the XML does not accumulate
         # whitespace
         dom.documentElement.normalize()
@@ -1080,6 +1118,8 @@ class Report(problem_report.ProblemReport):
 
         This requires the ExecutablePath attribute. Throws a ValueError if the
         file has an invalid format.
+
+        Privileges may need to be dropped before calling this.
         '''
         assert 'ExecutablePath' in self
 
@@ -1126,10 +1166,13 @@ class Report(problem_report.ProblemReport):
             return False
 
         # search for existing entry and update it
-        for ignore in dom.getElementsByTagName('ignore'):
-            if ignore.getAttribute('program') == self['ExecutablePath']:
-                if float(ignore.getAttribute('mtime')) >= cur_mtime:
-                    return True
+        try:
+            for ignore in dom.getElementsByTagName('ignore'):
+                if ignore.getAttribute('program') == self['ExecutablePath']:
+                    if float(ignore.getAttribute('mtime')) >= cur_mtime:
+                        return True
+        except (ValueError, KeyError):
+            pass
 
         return False
 
@@ -1142,6 +1185,8 @@ class Report(problem_report.ProblemReport):
 
         Throws a ValueError if the file already exists and has an invalid
         format.
+
+        Privileges may need to be dropped before calling this.
         '''
         assert 'ExecutablePath' in self
 
@@ -1167,14 +1212,10 @@ class Report(problem_report.ProblemReport):
             e.setAttribute('mtime', mtime)
             dom.documentElement.appendChild(e)
 
-        # write back file; temporarily unset $HOME, as this gets the wrong home
-        # dir for e. g. sudo
-        orig_home = os.getenv('HOME')
-        if orig_home is not None:
-            del os.environ['HOME']
-        ignore_file_path = os.path.expanduser(_ignore_file)
-        if orig_home is not None:
-            os.environ['HOME'] = orig_home
+        # Write back file
+        # Properly handle dropped privileges
+        homedir = pwd.getpwuid(os.geteuid())[5]
+        ignore_file_path = _ignore_file.replace("~", homedir)
 
         with open(ignore_file_path, 'w') as fd:
             dom.writexml(fd, addindent='  ', newl='\n')
@@ -1571,9 +1612,10 @@ class Report(problem_report.ProblemReport):
         removes the ProcCwd attribute completely.
         '''
         replacements = []
+        # Do not replace "root"
         if (os.getuid() > 0):
-            # do not replace "root"
-            p = pwd.getpwuid(os.getuid())
+            # Use effective uid in case privileges were dropped
+            p = pwd.getpwuid(os.geteuid())
             if len(p[0]) >= 2:
                 replacements.append((re.compile(r'\b%s\b' % re.escape(p[0])), 'username'))
             replacements.append((re.compile(r'\b%s\b' % re.escape(p[5])), '/home/username'))
@@ -1775,21 +1817,3 @@ class Report(problem_report.ProblemReport):
             return None
 
         return (my_session, session_start_time)
-
-    def get_timestamp(self):
-        '''Get timestamp (seconds since epoch) from Date field
-
-        Return None if it is not present.
-        '''
-        # report time is from asctime(), not in locale representation
-        orig_ctime = locale.getlocale(locale.LC_TIME)
-        try:
-            try:
-                locale.setlocale(locale.LC_TIME, 'C')
-                return time.mktime(time.strptime(self['Date']))
-            except KeyError:
-                return None
-            finally:
-                locale.setlocale(locale.LC_TIME, orig_ctime)
-        except locale.Error:
-            return None
