@@ -141,7 +141,10 @@ def thread_collect_info(report, reportfile, package, ui, symptom_script=None,
                         not apport.packaging.is_native_origin_package(report['Package'].split()[0]):
                     # TRANS: %s is the name of the operating system
                     report['UnreportableReason'] = _(
-                        'This is not an official %s package. Please remove any third party package and try again.') % report['DistroRelease'].split()[0]
+                        'This does not seem to be an official %s package. \
+Please retry after updating the indexes of available packages, \
+if that does not work then remove related third party packages \
+and try again.') % report['DistroRelease'].split()[0]
         except ValueError:
             # this happens if we are collecting information on an uninstalled
             # package
@@ -395,6 +398,12 @@ class UserInterface:
         mark the report for uploading.
         '''
         self.report = apport.Report('Hang')
+
+        if not self.options.pid:
+            self.ui_error_message(_('No PID specified'),
+                                  _('You need to specify a PID. See --help for more information.'))
+            return False
+
         try:
             self.report.add_proc_info(pid)
         except ValueError as e:
@@ -701,7 +710,7 @@ class UserInterface:
         if self.options.symptom:
             self.run_symptom()
             return True
-        elif hasattr(self.options, 'pid') and self.options.hanging:
+        elif self.options.hanging:
             self.run_hang(self.options.pid)
             return True
         elif self.options.filebug:
@@ -718,6 +727,19 @@ class UserInterface:
                 self.ui_error_message(_('Invalid problem report'), excstr(e))
             return True
         elif self.options.window:
+            if os.getenv('XDG_SESSION_TYPE') == 'wayland':
+                self.ui_error_message(_('Cannot create report'),
+                                      _('The window option cannot be used on '
+                                        'Wayland.\n\nPlease find the window\'s '
+                                        'process ID and then run \'ubuntu-bug '
+                                        '<process ID>\'.'
+                                        '\n\nThe process ID can be found '
+                                        'by running the System Monitor application. '
+                                        'In the Processes tab, scroll until you '
+                                        'find the correct application. The process '
+                                        'ID is the number listed in the ID column.'))
+                return True
+
             self.ui_info_message('', _('After closing this message '
                                        'please click on an application window to report a problem about it.'))
             xprop = subprocess.Popen(['xprop', '_NET_WM_PID'],
@@ -763,6 +785,7 @@ class UserInterface:
         self.options.filebug = False
         self.options.crash_file = None
         self.options.version = None
+        self.options.hanging = False
         self.args = []
 
     def parse_argv(self):
@@ -1250,8 +1273,7 @@ class UserInterface:
             if 'UnreportableReason' not in self.report:
                 if ((('SourcePackage' not in self.report and 'Dependencies' not in self.report) or
                     (not self.report.get('ProblemType', '').startswith('Kernel') and
-                    'Package' not in self.report)) and ('SnapSource' not in self.report and
-                                                        'Snap' not in self.report)):
+                     'Package' not in self.report)) and ('SnapSource' not in self.report and 'Snap' not in self.report)):
                     self.ui_error_message(_('Invalid problem report'),
                                           _('Could not determine the package or source package name.'))
                     # TODO This is not called consistently, is it really needed?
@@ -1306,7 +1328,8 @@ class UserInterface:
                 for e in env:
                     if e.startswith(b'DBUS_SESSION_BUS_ADDRESS='):
                         sudo_prefix.append('DBUS_SESSION_BUS_ADDRESS=' + e.split(b'=', 1)[1].decode())
-                        break
+                    if e.startswith(b'XDG_DATA_DIRS='):
+                        sudo_prefix.append('XDG_DATA_DIRS=' + e.split(b'=', 1)[1].decode())
             except (subprocess.CalledProcessError, IOError):
                 pass
 
