@@ -4,21 +4,22 @@
 
 """Functions around users and groups."""
 
+import dataclasses
 import os
 
 
+@dataclasses.dataclass()
 class UserGroupID:
     """Pair of user and group ID."""
 
-    def __init__(self, uid, gid):
-        self.uid = uid
-        self.gid = gid
+    uid: int
+    gid: int
 
-    def is_root(self):
+    def is_root(self) -> bool:
         """Check if the user or group ID is root."""
         return self.uid == 0 or self.gid == 0
 
 
-def get_process_user_and_group():
-    """Return the current process's real user and group."""
+def get_process_user_and_group() -> UserGroupID:
+    """Return the current process’s real user and group."""
     return UserGroupID(os.getuid(), os.getgid())
