@@ -109,7 +109,7 @@ class SourceEntry(object):
         """ equal operator for two sources.list entries """
         return (self.disabled == other.disabled and
                 self.type == other.type and
-                self.uri == other.uri and
+                self.uri.rstrip('/') == other.uri.rstrip('/') and
                 self.dist == other.dist and
                 self.comps == other.comps)
 
@@ -280,7 +280,8 @@ class SourcesList(object):
         self.list = []
         # read sources.list
         file = apt_pkg.config.find_file("Dir::Etc::sourcelist")
-        self.load(file)
+        if os.path.exists(file):
+            self.load(file)
         # read sources.list.d
         partsdir = apt_pkg.config.find_dir("Dir::Etc::sourceparts")
         for file in glob.glob("%s/*.list" % partsdir):
@@ -297,7 +298,10 @@ class SourcesList(object):
             yield entry
 
     def __find(self, *predicates, **attrs):
+        uri = attrs.pop('uri', None)
         for source in self.list:
+            if uri and uri.rstrip('/') != source.uri.rstrip('/'):
+                continue
             if (all(getattr(source, key) == attrs[key] for key in attrs) and
                     all(predicate(source) for predicate in predicates)):
                 yield source
