@@ -67,7 +67,7 @@ def handle(name: str, cfg: Config, cloud: Cloud, args: list) -> None:
         write_apt_snippet("0", LOG, DEFAULT_FILE)
     elif apt_pipe_value_s in ("none", "unchanged", "os"):
         return
-    elif apt_pipe_value_s in [str(b) for b in range(6)]:
+    elif apt_pipe_value_s in [str(b) for b in range(0, 6)]:
         write_apt_snippet(apt_pipe_value_s, LOG, DEFAULT_FILE)
     else:
         LOG.warning("Invalid option for apt_pipelining: %s", apt_pipe_value)
