@@ -10,25 +10,106 @@
 import logging
 import os
 import shlex
-from pathlib import Path
+from textwrap import dedent
 
-from cloudinit import performance, subp, util
+from cloudinit import subp, util
 from cloudinit.cloud import Cloud
 from cloudinit.config import Config
-from cloudinit.config.schema import MetaSchema
+from cloudinit.config.schema import MetaSchema, get_meta_doc
 from cloudinit.distros import ALL_DISTROS
 from cloudinit.settings import PER_INSTANCE
 
+# Define the commands to use
+SFDISK_CMD = subp.which("sfdisk")
+SGDISK_CMD = subp.which("sgdisk")
+LSBLK_CMD = subp.which("lsblk")
+BLKID_CMD = subp.which("blkid")
+BLKDEV_CMD = subp.which("blockdev")
+PARTPROBE_CMD = subp.which("partprobe")
+WIPEFS_CMD = subp.which("wipefs")
+
 LANG_C_ENV = {"LANG": "C"}
 LOG = logging.getLogger(__name__)
 
+MODULE_DESCRIPTION = """\
+This module is able to configure simple partition tables and filesystems.
+
+.. note::
+    for more detail about configuration options for disk setup, see the disk
+    setup example
+
+.. note::
+    if a swap partition is being created via ``disk_setup`` then a ``fs_entry``
+    entry is also needed in order for mkswap to be run, otherwise when swap
+    activation is later attempted it will fail.
+
+For convenience, aliases can be specified for disks using the
+``device_aliases`` config key, which takes a dictionary of alias: path
+mappings. There are automatic aliases for ``swap`` and ``ephemeral<X>``, where
+``swap`` will always refer to the active swap partition and ``ephemeral<X>``
+will refer to the block device of the ephemeral image.
+
+Disk partitioning is done using the ``disk_setup`` directive. This config
+directive accepts a dictionary where each key is either a path to a block
+device or an alias specified in ``device_aliases``, and each value is the
+configuration options for the device. File system configuration is done using
+the ``fs_setup`` directive. This config directive accepts a list of
+filesystem configs.
+"""
+
 meta: MetaSchema = {
     "id": "cc_disk_setup",
+    "name": "Disk Setup",
+    "title": "Configure partitions and filesystems",
+    "description": MODULE_DESCRIPTION,
     "distros": [ALL_DISTROS],
     "frequency": PER_INSTANCE,
+    "examples": [
+        dedent(
+            """\
+            device_aliases:
+              my_alias: /dev/sdb
+              swap_disk: /dev/sdc
+            disk_setup:
+              my_alias:
+                table_type: gpt
+                layout: [50, 50]
+                overwrite: true
+              swap_disk:
+                table_type: gpt
+                layout: [[100, 82]]
+                overwrite: true
+              /dev/sdd:
+                table_type: mbr
+                layout: true
+                overwrite: true
+            fs_setup:
+            - label: fs1
+              filesystem: ext4
+              device: my_alias.1
+              cmd: mkfs -t %(filesystem)s -L %(label)s %(device)s
+            - label: fs2
+              device: my_alias.2
+              filesystem: ext4
+            - label: swap
+              device: swap_disk.1
+              filesystem: swap
+            - label: fs3
+              device: /dev/sdd1
+              filesystem: ext4
+            mounts:
+            - ["my_alias.1", "/mnt1"]
+            - ["my_alias.2", "/mnt2"]
+            - ["swap_disk.1", "none", "swap", "sw", "0", "0"]
+            - ["/dev/sdd1", "/mnt3"]
+            """
+        )
+    ],
     "activate_by_schema_keys": ["disk_setup", "fs_setup"],
 }
 
+__doc__ = get_meta_doc(meta)
+
 
 def handle(name: str, cfg: Config, cloud: Cloud, args: list) -> None:
     """
@@ -51,10 +132,13 @@ def handle(name: str, cfg: Config, cloud: Cloud, args: list) -> None:
                 continue
 
             try:
-                with performance.Timed(
-                    f"Creating partition on {disk}",
-                ):
-                    mkpart(disk, definition)
+                LOG.debug("Creating new partition table/disk")
+                util.log_time(
+                    logfunc=LOG.debug,
+                    msg="Creating partition on %s" % disk,
+                    func=mkpart,
+                    args=(disk, definition),
+                )
             except Exception as e:
                 util.logexc(LOG, "Failed partitioning operation\n%s" % e)
 
@@ -68,8 +152,14 @@ def handle(name: str, cfg: Config, cloud: Cloud, args: list) -> None:
                 continue
 
             try:
-                with performance.Timed("Creating new filesystem"):
-                    mkfs(definition)
+                LOG.debug("Creating new filesystem.")
+                device = definition.get("device")
+                util.log_time(
+                    logfunc=LOG.debug,
+                    msg="Creating fs for %s" % device,
+                    func=mkfs,
+                    args=(definition,),
+                )
             except Exception as e:
                 util.logexc(LOG, "Failed during filesystem operation\n%s" % e)
 
@@ -167,7 +257,7 @@ def enumerate_disk(device, nodeps=False):
     """
 
     lsblk_cmd = [
-        "lsblk",
+        LSBLK_CMD,
         "--pairs",
         "--output",
         "NAME,TYPE,FSTYPE,LABEL",
@@ -241,7 +331,7 @@ def check_fs(device):
     """
     out, label, fs_type, uuid = None, None, None, None
 
-    blkid_cmd = ["blkid", "-c", "/dev/null", device]
+    blkid_cmd = [BLKID_CMD, "-c", "/dev/null", device]
     try:
         out, _err = subp.subp(blkid_cmd, rcs=[0, 2])
     except Exception as e:
@@ -348,8 +438,8 @@ def is_disk_used(device):
 
 def get_hdd_size(device):
     try:
-        size_in_bytes, _ = subp.subp(["blockdev", "--getsize64", device])
-        sector_size, _ = subp.subp(["blockdev", "--getss", device])
+        size_in_bytes, _ = subp.subp([BLKDEV_CMD, "--getsize64", device])
+        sector_size, _ = subp.subp([BLKDEV_CMD, "--getss", device])
     except Exception as e:
         raise RuntimeError("Failed to get %s size\n%s" % (device, e)) from e
 
@@ -365,8 +455,7 @@ def check_partition_mbr_layout(device, layout):
     """
 
     read_parttbl(device)
-
-    prt_cmd = ["sfdisk", "-l", device]
+    prt_cmd = [SFDISK_CMD, "-l", device]
     try:
         out, _err = subp.subp(prt_cmd, data="%s\n" % layout)
     except Exception as e:
@@ -397,7 +486,7 @@ def check_partition_mbr_layout(device, layout):
 
 
 def check_partition_gpt_layout(device, layout):
-    prt_cmd = ["sgdisk", "-p", device]
+    prt_cmd = [SGDISK_CMD, "-p", device]
     try:
         out, _err = subp.subp(prt_cmd, update_env=LANG_C_ENV)
     except Exception as e:
@@ -581,13 +670,13 @@ def purge_disk_ptable(device):
 
 def purge_disk(device):
     """
-    Remove partition table entries
+    Remove parition table entries
     """
 
     # wipe any file systems first
     for d in enumerate_disk(device):
         if d["type"] not in ["disk", "crypt"]:
-            wipefs_cmd = ["wipefs", "--all", "/dev/%s" % d["name"]]
+            wipefs_cmd = [WIPEFS_CMD, "--all", "/dev/%s" % d["name"]]
             try:
                 LOG.info("Purging filesystem on /dev/%s", d["name"])
                 subp.subp(wipefs_cmd)
@@ -620,11 +709,10 @@ def read_parttbl(device):
     `Partprobe` is preferred over `blkdev` since it is more reliably
     able to probe the partition table.
     """
-    partprobe = "partprobe"
-    if subp.which(partprobe):
-        probe_cmd = [partprobe, device]
+    if PARTPROBE_CMD is not None:
+        probe_cmd = [PARTPROBE_CMD, device]
     else:
-        probe_cmd = ["blockdev", "--rereadpt", device]
+        probe_cmd = [BLKDEV_CMD, "--rereadpt", device]
     util.udevadm_settle()
     try:
         subp.subp(probe_cmd)
@@ -640,7 +728,7 @@ def exec_mkpart_mbr(device, layout):
     types, i.e. gpt
     """
     # Create the partitions
-    prt_cmd = ["sfdisk", "--force", device]
+    prt_cmd = [SFDISK_CMD, "--force", device]
     try:
         subp.subp(prt_cmd, data="%s\n" % layout)
     except Exception as e:
@@ -653,12 +741,12 @@ def exec_mkpart_mbr(device, layout):
 
 def exec_mkpart_gpt(device, layout):
     try:
-        subp.subp(["sgdisk", "-Z", device])
+        subp.subp([SGDISK_CMD, "-Z", device])
         for index, (partition_type, (start, end)) in enumerate(layout):
             index += 1
             subp.subp(
                 [
-                    "sgdisk",
+                    SGDISK_CMD,
                     "-n",
                     "{}:{}:{}".format(index, start, end),
                     device,
@@ -669,7 +757,7 @@ def exec_mkpart_gpt(device, layout):
                 # 82 -> 8200.  'Linux' -> 'Linux'
                 pinput = str(partition_type).ljust(4, "0")
                 subp.subp(
-                    ["sgdisk", "-t", "{}:{}".format(index, pinput), device]
+                    [SGDISK_CMD, "-t", "{}:{}".format(index, pinput), device]
                 )
     except Exception:
         LOG.warning("Failed to partition device %s", device)
@@ -703,7 +791,7 @@ def mkpart(device, definition):
 
             The following are supported values in the dict:
                 overwrite: Should the partition table be created regardless
-                            of any pre-existing data?
+                            of any pre-exisiting data?
                 layout: the layout of the partition table
                 table_type: Which partition table to use, defaults to MBR
                 device: the device to work on.
@@ -827,15 +915,7 @@ def mkfs(fs_cfg):
     if not partition or partition.isdigit():
         # Handle manual definition of partition
         if partition.isdigit():
-            # nvme support
-            # https://github.com/torvalds/linux/blob/45db3ab/block/partitions
-            # /core.c#L330
-            if device[-1].isdigit():
-                device = f"{device}p"
             device = "%s%s" % (device, partition)
-            if not Path(device).is_block_device():
-                LOG.warning("Path %s does not exist or is not a block device")
-                return
             LOG.debug(
                 "Manual request of partition %s for %s", partition, device
             )
@@ -857,15 +937,15 @@ def mkfs(fs_cfg):
                 LOG.debug("Device %s has required file system", device)
                 return
             else:
-                LOG.debug("Destroying filesystem on %s", device)
+                LOG.warning("Destroying filesystem on %s", device)
 
         else:
-            LOG.debug("Device %s is cleared for formatting", device)
+            LOG.debug("Device %s is cleared for formating", device)
 
     elif partition and str(partition).lower() in ("auto", "any"):
         # For auto devices, we match if the filesystem does exist
         odevice = device
-        LOG.debug("Identifying device to create %s filesystem on", label)
+        LOG.debug("Identifying device to create %s filesytem on", label)
 
         # 'any' means pick the first match on the device with matching fs_type
         label_match = True
@@ -882,7 +962,7 @@ def mkfs(fs_cfg):
         LOG.debug("Automatic device for %s identified as %s", odevice, device)
 
         if reuse:
-            LOG.debug("Found filesystem match, skipping formatting.")
+            LOG.debug("Found filesystem match, skipping formating.")
             return
 
         if not reuse and fs_replace and device:
@@ -974,7 +1054,11 @@ def mkfs(fs_cfg):
         fs_cmd.append(device)
 
     LOG.debug("Creating file system %s on %s", label, device)
+    LOG.debug("     Using cmd: %s", str(fs_cmd))
     try:
         subp.subp(fs_cmd, shell=shell)
     except Exception as e:
         raise RuntimeError("Failed to exec of '%s':\n%s" % (fs_cmd, e)) from e
+
+
+# vi: ts=4 expandtab
