@@ -20,22 +20,20 @@ from contextlib import suppress
 from pathlib import Path
 from typing import Optional, Tuple
 
-from cloudinit import subp, temp_utils, util
+from cloudinit import lifecycle, performance, subp, temp_utils, util
 from cloudinit.cloud import Cloud
 from cloudinit.config import Config
 from cloudinit.config.schema import MetaSchema
 from cloudinit.distros import ALL_DISTROS, Distro
 from cloudinit.settings import PER_ALWAYS
 
-MODULE_DESCRIPTION = """\
-"""
 frequency = PER_ALWAYS
 meta: MetaSchema = {
     "id": "cc_growpart",
     "distros": [ALL_DISTROS],
     "frequency": frequency,
     "activate_by_schema_keys": [],
-}  # type: ignore
+}
 
 DEFAULT_CONFIG = {
     "mode": "auto",
@@ -65,12 +63,10 @@ class Resizer(ABC):
         self._distro = distro
 
     @abstractmethod
-    def available(self, devices: list) -> bool:
-        ...
+    def available(self, devices: list) -> bool: ...
 
     @abstractmethod
-    def resize(self, diskdev, partnum, partdev, fs):
-        ...
+    def resize(self, diskdev, partnum, partdev, fs): ...
 
 
 class ResizeGrowPart(Resizer):
@@ -320,7 +316,9 @@ def resize_encrypted(blockdev, partition) -> Tuple[str, str]:
     if not KEYDATA_PATH.exists():
         return (RESIZE.SKIPPED, "No encryption keyfile found")
     try:
-        with KEYDATA_PATH.open() as f:
+        with performance.Timed(
+            f"Reading {KEYDATA_PATH}"
+        ), KEYDATA_PATH.open() as f:
             keydata = json.load(f)
         key = keydata["key"]
         decoded_key = base64.b64decode(key)
@@ -542,7 +540,7 @@ def handle(name: str, cfg: Config, cloud: Cloud, args: list) -> None:
     mode = mycfg.get("mode", "auto")
     if util.is_false(mode):
         if mode != "off":
-            util.deprecate(
+            lifecycle.deprecate(
                 deprecated=f"Growpart's 'mode' key with value '{mode}'",
                 deprecated_version="22.2",
                 extra_message="Use 'off' instead.",
@@ -569,12 +567,8 @@ def handle(name: str, cfg: Config, cloud: Cloud, args: list) -> None:
             raise e
         return
 
-    resized = util.log_time(
-        logfunc=LOG.debug,
-        msg="resize_devices",
-        func=resize_devices,
-        args=(resizer, devices, cloud.distro),
-    )
+    with performance.Timed("Resizing devices"):
+        resized = resize_devices(resizer, devices, cloud.distro)
     for entry, action, msg in resized:
         if action == RESIZE.CHANGED:
             LOG.info("'%s' resized: %s", entry, msg)
