@@ -96,7 +96,7 @@ def fetch_idevs():
             # something bad has happened, continue to log the error
             raise
     except Exception:
-        util.logexc(LOG, "grub-probe failed to execute for grub-dpkg")
+        util.logexc(LOG, "grub-probe failed to execute for grub_dpkg")
 
     if not disk or not os.path.exists(disk):
         # If we failed to detect a disk, we can return early
@@ -154,7 +154,7 @@ def handle(name: str, cfg: Config, cloud: Cloud, args: list) -> None:
     LOG.debug("Setting grub debconf-set-selections with '%s'", dconf_sel)
 
     try:
-        subp.subp(["debconf-set-selections"], dconf_sel)
+        subp.subp(["debconf-set-selections"], data=dconf_sel)
     except Exception as e:
         util.logexc(
             LOG, "Failed to run debconf-set-selections for grub_dpkg: %s", e
