@@ -1,25 +1,23 @@
 # This file is part of cloud-init. See LICENSE file for license information.
 """Install hotplug udev rules if supported and enabled"""
+import logging
 import os
 from textwrap import dedent
 
-from cloudinit import util
-from cloudinit import subp
-from cloudinit import stages
-from cloudinit.config.schema import get_schema_doc, validate_cloudconfig_schema
+from cloudinit import stages, subp, util
+from cloudinit.cloud import Cloud
+from cloudinit.config import Config
+from cloudinit.config.schema import MetaSchema, get_meta_doc
 from cloudinit.distros import ALL_DISTROS
-from cloudinit.event import EventType, EventScope
+from cloudinit.event import EventScope, EventType
 from cloudinit.settings import PER_INSTANCE
 
-
-frequency = PER_INSTANCE
-distros = [ALL_DISTROS]
-
-schema = {
+meta: MetaSchema = {
     "id": "cc_install_hotplug",
     "name": "Install Hotplug",
-    "title": "Install hotplug if supported and enabled",
-    "description": dedent("""\
+    "title": "Install hotplug udev rules if supported and enabled",
+    "description": dedent(
+        """\
         This module will install the udev rules to enable hotplug if
         supported by the datasource and enabled in the userdata. The udev
         rules will be installed as
@@ -32,56 +30,33 @@ schema = {
         network configuration.
 
         Currently supported datasources: Openstack, EC2
-    """),
-    "distros": distros,
+    """
+    ),
+    "distros": [ALL_DISTROS],
+    "frequency": PER_INSTANCE,
     "examples": [
-        dedent("""\
+        dedent(
+            """\
             # Enable hotplug of network devices
             updates:
               network:
                 when: ["hotplug"]
-        """),
-        dedent("""\
+        """
+        ),
+        dedent(
+            """\
             # Enable network hotplug alongside boot event
             updates:
               network:
                 when: ["boot", "hotplug"]
-        """),
+        """
+        ),
     ],
-    "frequency": frequency,
-    "type": "object",
-    "properties": {
-        "updates": {
-            "type": "object",
-            "additionalProperties": False,
-            "properties": {
-                "network": {
-                    "type": "object",
-                    "required": ["when"],
-                    "additionalProperties": False,
-                    "properties": {
-                        "when": {
-                            "type": "array",
-                            "additionalProperties": False,
-                            "items": {
-                                "type": "string",
-                                "additionalProperties": False,
-                                "enum": [
-                                    "boot-new-instance",
-                                    "boot-legacy",
-                                    "boot",
-                                    "hotplug",
-                                ]
-                            }
-                        }
-                    }
-                }
-            }
-        }
-    }
+    "activate_by_schema_keys": [],
 }
 
-__doc__ = get_schema_doc(schema)
+__doc__ = get_meta_doc(meta)
+LOG = logging.getLogger(__name__)
 
 
 HOTPLUG_UDEV_PATH = "/etc/udev/rules.d/10-cloud-init-hook-hotplug.rules"
@@ -94,17 +69,17 @@ LABEL="cloudinit_end"
 """
 
 
-def handle(_name, cfg, cloud, log, _args):
-    validate_cloudconfig_schema(cfg, schema)
+def handle(name: str, cfg: Config, cloud: Cloud, args: list) -> None:
     network_hotplug_enabled = (
-        'updates' in cfg and
-        'network' in cfg['updates'] and
-        'when' in cfg['updates']['network'] and
-        'hotplug' in cfg['updates']['network']['when']
+        "updates" in cfg
+        and "network" in cfg["updates"]
+        and "when" in cfg["updates"]["network"]
+        and "hotplug" in cfg["updates"]["network"]["when"]
     )
     hotplug_supported = EventType.HOTPLUG in (
-        cloud.datasource.get_supported_events(
-            [EventType.HOTPLUG]).get(EventScope.NETWORK, set())
+        cloud.datasource.get_supported_events([EventType.HOTPLUG]).get(
+            EventScope.NETWORK, set()
+        )
     )
     hotplug_enabled = stages.update_event_enabled(
         datasource=cloud.datasource,
@@ -114,19 +89,19 @@ def handle(_name, cfg, cloud, log, _args):
     )
     if not (hotplug_supported and hotplug_enabled):
         if os.path.exists(HOTPLUG_UDEV_PATH):
-            log.debug("Uninstalling hotplug, not enabled")
+            LOG.debug("Uninstalling hotplug, not enabled")
             util.del_file(HOTPLUG_UDEV_PATH)
             subp.subp(["udevadm", "control", "--reload-rules"])
         elif network_hotplug_enabled:
-            log.warning(
+            LOG.warning(
                 "Hotplug is unsupported by current datasource. "
                 "Udev rules will NOT be installed."
             )
         else:
-            log.debug("Skipping hotplug install, not enabled")
+            LOG.debug("Skipping hotplug install, not enabled")
         return
     if not subp.which("udevadm"):
-        log.debug("Skipping hotplug install, udevadm not found")
+        LOG.debug("Skipping hotplug install, udevadm not found")
         return
 
     # This may need to turn into a distro property at some point
