@@ -6,11 +6,22 @@
 #
 # This file is part of cloud-init. See LICENSE file for license information.
 
-"""
-Mounts
-------
-**Summary:** configure mount points and swap files
+"""Mounts: Configure mount points and swap files"""
 
+import logging
+import math
+import os
+import re
+from string import whitespace
+from textwrap import dedent
+
+from cloudinit import subp, type_utils, util
+from cloudinit.cloud import Cloud
+from cloudinit.config import Config
+from cloudinit.config.schema import MetaSchema, get_meta_doc
+from cloudinit.settings import PER_INSTANCE
+
+MODULE_DESCRIPTION = """\
 This module can add or remove mountpoints from ``/etc/fstab`` as well as
 configure swap. The ``mounts`` config key takes a list of fstab entries to add.
 Each entry is specified as a list of ``[ fs_spec, fs_file, fs_vfstype,
@@ -19,64 +30,105 @@ consult the manual for ``/etc/fstab``. When specifying the ``fs_spec``, if the
 device name starts with one of ``xvd``, ``sd``, ``hd``, or ``vd``, the leading
 ``/dev`` may be omitted.
 
-In order to remove a previously listed mount, an entry can be added to the
-mounts list containing ``fs_spec`` for the device to be removed but no
-mountpoint (i.e. ``[ sda1 ]`` or ``[ sda1, null ]``).
+Any mounts that do not appear to either an attached block device or network
+resource will be skipped with a log like "Ignoring nonexistent mount ...".
 
-The ``mount_default_fields`` config key allows default options to be specified
-for the values in a ``mounts`` entry that are not specified, aside from the
-``fs_spec`` and the ``fs_file``. If specified, this must be a list containing 6
-values. It defaults to::
+Cloud-init will attempt to add the following mount directives if available and
+unconfigured in `/etc/fstab`::
 
-    mount_default_fields: [none, none, "auto", "defaults,nobootwait", "0", "2"]
+    mounts:
+        - ["ephemeral0", "/mnt", "auto",\
+"defaults,nofail,x-systemd.requires=cloud-init.service", "0", "2"]
+        - ["swap", "none", "swap", "sw", "0", "0"]
 
-On a systemd booted system that default is the mostly equivalent::
+In order to remove a previously listed mount, an entry can be added to
+the `mounts` list containing ``fs_spec`` for the device to be removed but no
+mountpoint (i.e. ``[ swap ]`` or ``[ swap, null ]``).
 
-    mount_default_fields: [none, none, "auto",
-       "defaults,nofail,x-systemd.requires=cloud-init.service", "0", "2"]
+The ``mount_default_fields`` config key allows default values to be specified
+for the fields in a ``mounts`` entry that are not specified, aside from the
+``fs_spec`` and the ``fs_file`` fields. If specified, this must be a list
+containing 6 values. It defaults to::
 
-Note that `nobootwait` is an upstart specific boot option that somewhat
-equates to the more standard `nofail`.
+    mount_default_fields: [none, none, "auto",\
+"defaults,nofail,x-systemd.requires=cloud-init.service", "0", "2"]
+
+Non-systemd init systems will vary in ``mount_default_fields``.
 
 Swap files can be configured by setting the path to the swap file to create
 with ``filename``, the size of the swap file with ``size`` maximum size of
 the swap file if using an ``size: auto`` with ``maxsize``. By default no
 swap file is created.
 
-**Internal name:** ``cc_mounts``
+.. note::
+    If multiple mounts are specified where a subsequent mount's mountpoint is
+    inside of a previously declared mount's mountpoint (i.e. the 1st mount has
+    a mountpoint of ``/abc`` and the 2nd mount has a mountpoint of
+    ``/abc/def``) then this will not work as expected - ``cc_mounts`` first
+    creates the directories for all the mountpoints **before** it starts to
+    perform any mounts and so the sub-mountpoint directory will not be created
+    correctly inside the parent mountpoint.
 
-**Module frequency:** per instance
-
-**Supported distros:** all
-
-**Config keys**::
+    For systems using util-linux's ``mount`` program this issue can be
+    worked around by specifying ``X-mount.mkdir`` as part of a ``fs_mntops``
+    value for the subsequent mount entry.
+"""
 
+example = dedent(
+    """\
+    # Mount ephemeral0 with "noexec" flag, /dev/sdc with mount_default_fields,
+    # and /dev/xvdh with custom fs_passno "0" to avoid fsck on the mount.
+    # Also provide an automatically sized swap with a max size of 10485760
+    # bytes.
     mounts:
         - [ /dev/ephemeral0, /mnt, auto, "defaults,noexec" ]
         - [ sdc, /opt/data ]
-        - [ xvdh, /opt/data, "auto", "defaults,nofail", "0", "0" ]
-    mount_default_fields: [None, None, "auto", "defaults,nofail", "0", "2"]
+        - [ xvdh, /opt/data, auto, "defaults,nofail", "0", "0" ]
+    mount_default_fields: [None, None, auto, "defaults,nofail", "0", "2"]
     swap:
-        filename: <file>
-        size: <"auto"/size in bytes>
-        maxsize: <size in bytes>
-"""
+        filename: /my/swapfile
+        size: auto
+        maxsize: 10485760
+    """
+)
 
-from string import whitespace
+distros = ["all"]
 
-import logging
-import os.path
-import re
+meta: MetaSchema = {
+    "id": "cc_mounts",
+    "name": "Mounts",
+    "title": "Configure mount points and swap files",
+    "description": MODULE_DESCRIPTION,
+    "distros": distros,
+    "examples": [
+        example,
+        dedent(
+            """\
+        # Create a 2 GB swap file at /swapfile using human-readable values
+        swap:
+            filename: /swapfile
+            size: 2G
+            maxsize: 2G
+        """
+        ),
+    ],
+    "frequency": PER_INSTANCE,
+    "activate_by_schema_keys": [],
+}
 
-from cloudinit import type_utils
-from cloudinit import util
+__doc__ = get_meta_doc(meta)
 
 # Shortname matches 'sda', 'sda1', 'xvda', 'hda', 'sdb', xvdb, vda, vdd1, sr0
 DEVICE_NAME_FILTER = r"^([x]{0,1}[shv]d[a-z][0-9]*|sr[0-9]+)$"
 DEVICE_NAME_RE = re.compile(DEVICE_NAME_FILTER)
+# Name matches 'server:/path'
+NETWORK_NAME_FILTER = r"^.+:.*"
+NETWORK_NAME_RE = re.compile(NETWORK_NAME_FILTER)
 WS = re.compile("[%s]+" % (whitespace))
 FSTAB_PATH = "/etc/fstab"
 MNT_COMMENT = "comment=cloudconfig"
+MB = 2**20
+GB = 2**30
 
 LOG = logging.getLogger(__name__)
 
@@ -93,45 +145,64 @@ def is_meta_device_name(name):
     return False
 
 
+def is_network_device(name):
+    # return true if this is a network device
+    if NETWORK_NAME_RE.match(name):
+        return True
+    return False
+
+
 def _get_nth_partition_for_device(device_path, partition_number):
-    potential_suffixes = [str(partition_number), 'p%s' % (partition_number,),
-                          '-part%s' % (partition_number,)]
+    potential_suffixes = [
+        str(partition_number),
+        "p%s" % (partition_number,),
+        "-part%s" % (partition_number,),
+    ]
     for suffix in potential_suffixes:
-        potential_partition_device = '%s%s' % (device_path, suffix)
+        potential_partition_device = "%s%s" % (device_path, suffix)
         if os.path.exists(potential_partition_device):
             return potential_partition_device
     return None
 
 
 def _is_block_device(device_path, partition_path=None):
-    device_name = os.path.realpath(device_path).split('/')[-1]
-    sys_path = os.path.join('/sys/block/', device_name)
+    device_name = os.path.realpath(device_path).split("/")[-1]
+    sys_path = os.path.join("/sys/block/", device_name)
     if partition_path is not None:
         sys_path = os.path.join(
-            sys_path, os.path.realpath(partition_path).split('/')[-1])
+            sys_path, os.path.realpath(partition_path).split("/")[-1]
+        )
     return os.path.exists(sys_path)
 
 
-def sanitize_devname(startname, transformer, log):
-    log.debug("Attempting to determine the real name of %s", startname)
+def sanitize_devname(startname, transformer, aliases=None):
+    LOG.debug("Attempting to determine the real name of %s", startname)
 
     # workaround, allow user to specify 'ephemeral'
     # rather than more ec2 correct 'ephemeral0'
     devname = startname
     if devname == "ephemeral":
         devname = "ephemeral0"
-        log.debug("Adjusted mount option from ephemeral to ephemeral0")
+        LOG.debug("Adjusted mount option from ephemeral to ephemeral0")
+
+    if is_network_device(startname):
+        return startname
 
     device_path, partition_number = util.expand_dotted_devname(devname)
+    orig = device_path
+
+    if aliases:
+        device_path = aliases.get(device_path, device_path)
+        if orig != device_path:
+            LOG.debug("Mapped device alias %s to %s", orig, device_path)
 
     if is_meta_device_name(device_path):
-        orig = device_path
         device_path = transformer(device_path)
         if not device_path:
             return None
         if not device_path.startswith("/"):
             device_path = "/dev/%s" % (device_path,)
-        log.debug("Mapped metadata name %s to %s", orig, device_path)
+        LOG.debug("Mapped metadata name %s to %s", orig, device_path)
     else:
         if DEVICE_NAME_RE.match(startname):
             device_path = "/dev/%s" % (device_path,)
@@ -140,8 +211,9 @@ def sanitize_devname(startname, transformer, log):
     if partition_number is None:
         partition_path = _get_nth_partition_for_device(device_path, 1)
     else:
-        partition_path = _get_nth_partition_for_device(device_path,
-                                                       partition_number)
+        partition_path = _get_nth_partition_for_device(
+            device_path, partition_number
+        )
         if partition_path is None:
             return None
 
@@ -155,61 +227,42 @@ def sanitize_devname(startname, transformer, log):
 def suggested_swapsize(memsize=None, maxsize=None, fsys=None):
     # make a suggestion on the size of swap for this system.
     if memsize is None:
-        memsize = util.read_meminfo()['total']
+        memsize = util.read_meminfo()["total"]
 
-    GB = 2 ** 30
-    sugg_max = 8 * GB
+    sugg_max = memsize * 2
 
-    info = {'avail': 'na', 'max_in': maxsize, 'mem': memsize}
+    info = {"avail": "na", "max_in": maxsize, "mem": memsize}
 
     if fsys is None and maxsize is None:
-        # set max to 8GB default if no filesystem given
+        # set max to default if no filesystem given
         maxsize = sugg_max
     elif fsys:
         statvfs = os.statvfs(fsys)
         avail = statvfs.f_frsize * statvfs.f_bfree
-        info['avail'] = avail
+        info["avail"] = avail
 
         if maxsize is None:
             # set to 25% of filesystem space
             maxsize = min(int(avail / 4), sugg_max)
-        elif maxsize > ((avail * .9)):
+        elif maxsize > ((avail * 0.9)):
             # set to 90% of available disk space
-            maxsize = int(avail * .9)
+            maxsize = int(avail * 0.9)
     elif maxsize is None:
         maxsize = sugg_max
 
-    info['max'] = maxsize
+    info["max"] = maxsize
 
-    formulas = [
-        # < 1G: swap = double memory
-        (1 * GB, lambda x: x * 2),
-        # < 2G: swap = 2G
-        (2 * GB, lambda x: 2 * GB),
-        # < 4G: swap = memory
-        (4 * GB, lambda x: x),
-        # < 16G: 4G
-        (16 * GB, lambda x: 4 * GB),
-        # < 64G: 1/2 M up to max
-        (64 * GB, lambda x: x / 2),
-    ]
+    if memsize < 4 * GB:
+        minsize = memsize
+    elif memsize < 16 * GB:
+        minsize = 4 * GB
+    else:
+        minsize = round(math.sqrt(memsize / GB)) * GB
 
-    size = None
-    for top, func in formulas:
-        if memsize <= top:
-            size = min(func(memsize), maxsize)
-            # if less than 1/2 memory and not much, return 0
-            if size < (memsize / 2) and size < 4 * GB:
-                size = 0
-                break
-            break
+    size = min(minsize, maxsize)
 
-    if size is not None:
-        size = maxsize
+    info["size"] = size
 
-    info['size'] = size
-
-    MB = 2 ** 20
     pinfo = {}
     for k, v in info.items():
         if isinstance(v, int):
@@ -217,9 +270,14 @@ def suggested_swapsize(memsize=None, maxsize=None, fsys=None):
         else:
             pinfo[k] = v
 
-    LOG.debug("suggest %s swap for %s memory with '%s'"
-              " disk given max=%s [max=%s]'", pinfo['size'], pinfo['mem'],
-              pinfo['avail'], pinfo['max_in'], pinfo['max'])
+    LOG.debug(
+        "suggest %s swap for %s memory with '%s' disk given max=%s [max=%s]'",
+        pinfo["size"],
+        pinfo["mem"],
+        pinfo["avail"],
+        pinfo["max_in"],
+        pinfo["max"],
+    )
     return size
 
 
@@ -229,40 +287,54 @@ def create_swapfile(fname: str, size: str) -> None:
     errmsg = "Failed to create swapfile '%s' of size %sMB via %s: %s"
 
     def create_swap(fname, size, method):
-        LOG.debug("Creating swapfile in '%s' on fstype '%s' using '%s'",
-                  fname, fstype, method)
+        LOG.debug(
+            "Creating swapfile in '%s' on fstype '%s' using '%s'",
+            fname,
+            fstype,
+            method,
+        )
 
         if method == "fallocate":
-            cmd = ['fallocate', '-l', '%sM' % size, fname]
+            cmd = ["fallocate", "-l", "%sM" % size, fname]
         elif method == "dd":
-            cmd = ['dd', 'if=/dev/zero', 'of=%s' % fname, 'bs=1M',
-                   'count=%s' % size]
+            cmd = [
+                "dd",
+                "if=/dev/zero",
+                "of=%s" % fname,
+                "bs=1M",
+                "count=%s" % size,
+            ]
 
         try:
-            util.subp(cmd, capture=True)
-        except util.ProcessExecutionError as e:
-            LOG.warning(errmsg, fname, size, method, e)
+            subp.subp(cmd, capture=True)
+        except subp.ProcessExecutionError as e:
+            LOG.info(errmsg, fname, size, method, e)
             util.del_file(fname)
+            raise
 
     swap_dir = os.path.dirname(fname)
     util.ensure_dir(swap_dir)
 
     fstype = util.get_mount_info(swap_dir)[1]
 
-    if fstype in ("xfs", "btrfs"):
+    if fstype == "btrfs":
+        subp.subp(["truncate", "-s", "0", fname])
+        subp.subp(["chattr", "+C", fname])
+
+    if fstype == "xfs" and util.kernel_version() < (4, 18):
         create_swap(fname, size, "dd")
     else:
         try:
             create_swap(fname, size, "fallocate")
-        except util.ProcessExecutionError as e:
-            LOG.warning(errmsg, fname, size, "dd", e)
-            LOG.warning("Will attempt with dd.")
+        except subp.ProcessExecutionError:
+            LOG.info("fallocate swap creation failed, will attempt with dd")
             create_swap(fname, size, "dd")
 
-    util.chmod(fname, 0o600)
+    if os.path.exists(fname):
+        util.chmod(fname, 0o600)
     try:
-        util.subp(['mkswap', fname])
-    except util.ProcessExecutionError:
+        subp.subp(["mkswap", fname])
+    except subp.ProcessExecutionError:
         util.del_file(fname)
         raise
 
@@ -274,39 +346,44 @@ def setup_swapfile(fname, size=None, maxsize=None):
     maxsize: the maximum size
     """
     swap_dir = os.path.dirname(fname)
-    mibsize = str(int(size / (2 ** 20)))
     if str(size).lower() == "auto":
         try:
-            memsize = util.read_meminfo()['total']
+            memsize = util.read_meminfo()["total"]
         except IOError:
             LOG.debug("Not creating swap: failed to read meminfo")
             return
 
         util.ensure_dir(swap_dir)
-        size = suggested_swapsize(fsys=swap_dir, maxsize=maxsize,
-                                  memsize=memsize)
+        size = suggested_swapsize(
+            fsys=swap_dir, maxsize=maxsize, memsize=memsize
+        )
 
+    mibsize = str(int(size / (2**20)))
     if not size:
         LOG.debug("Not creating swap: suggested size was 0")
         return
 
-    util.log_time(LOG.debug, msg="Setting up swap file", func=create_swapfile,
-                  args=[fname, mibsize])
+    util.log_time(
+        LOG.debug,
+        msg="Setting up swap file",
+        func=create_swapfile,
+        args=[fname, mibsize],
+    )
 
     return fname
 
 
 def handle_swapcfg(swapcfg):
     """handle the swap config, calling setup_swap if necessary.
-       return None or (filename, size)
+    return None or (filename, size)
     """
     if not isinstance(swapcfg, dict):
         LOG.warning("input for swap config was not a dict.")
         return None
 
-    fname = swapcfg.get('filename', '/swap.img')
-    size = swapcfg.get('size', 0)
-    maxsize = swapcfg.get('maxsize', None)
+    fname = swapcfg.get("filename", "/swap.img")
+    size = swapcfg.get("size", 0)
+    maxsize = swapcfg.get("maxsize", None)
 
     if not (size and fname):
         LOG.debug("no need to setup swap")
@@ -314,18 +391,21 @@ def handle_swapcfg(swapcfg):
 
     if os.path.exists(fname):
         if not os.path.exists("/proc/swaps"):
-            LOG.debug("swap file %s exists, but no /proc/swaps exists, "
-                      "being safe", fname)
+            LOG.debug(
+                "swap file %s exists, but no /proc/swaps exists, being safe",
+                fname,
+            )
             return fname
         try:
-            for line in util.load_file("/proc/swaps").splitlines():
+            for line in util.load_text_file("/proc/swaps").splitlines():
                 if line.startswith(fname + " "):
                     LOG.debug("swap file %s already in use", fname)
                     return fname
             LOG.debug("swap file %s exists, but not in /proc/swaps", fname)
         except Exception:
-            LOG.warning("swap file %s exists. Error reading /proc/swaps",
-                        fname)
+            LOG.warning(
+                "swap file %s exists. Error reading /proc/swaps", fname
+            )
             return fname
 
     try:
@@ -341,19 +421,23 @@ def handle_swapcfg(swapcfg):
     return None
 
 
-def handle(_name, cfg, cloud, log, _args):
+def handle(name: str, cfg: Config, cloud: Cloud, args: list) -> None:
     # fs_spec, fs_file, fs_vfstype, fs_mntops, fs-freq, fs_passno
     def_mnt_opts = "defaults,nobootwait"
     uses_systemd = cloud.distro.uses_systemd()
     if uses_systemd:
-        def_mnt_opts = "defaults,nofail,x-systemd.requires=cloud-init.service"
+        def_mnt_opts = (
+            "defaults,nofail,x-systemd.requires=cloud-init.service,_netdev"
+        )
 
     defvals = [None, None, "auto", def_mnt_opts, "0", "2"]
     defvals = cfg.get("mount_default_fields", defvals)
 
     # these are our default set of mounts
-    defmnts = [["ephemeral0", "/mnt", "auto", defvals[3], "0", "2"],
-               ["swap", "none", "swap", "sw", "0", "0"]]
+    defmnts: list = [
+        ["ephemeral0", "/mnt", "auto", defvals[3], "0", "2"],
+        ["swap", "none", "swap", "sw", "0", "0"],
+    ]
 
     cfgmnt = []
     if "mounts" in cfg:
@@ -365,36 +449,47 @@ def handle(_name, cfg, cloud, log, _args):
     fstab_devs = {}
     fstab_removed = []
 
-    for line in util.load_file(FSTAB_PATH).splitlines():
-        if MNT_COMMENT in line:
-            fstab_removed.append(line)
-            continue
+    if os.path.exists(FSTAB_PATH):
+        for line in util.load_text_file(FSTAB_PATH).splitlines():
+            if MNT_COMMENT in line:
+                fstab_removed.append(line)
+                continue
 
-        try:
-            toks = WS.split(line)
-        except Exception:
-            pass
-        fstab_devs[toks[0]] = line
-        fstab_lines.append(line)
+            try:
+                toks = WS.split(line)
+            except Exception:
+                pass
+            fstab_devs[toks[0]] = line
+            fstab_lines.append(line)
+
+    device_aliases = cfg.get("device_aliases", {})
 
     for i in range(len(cfgmnt)):
         # skip something that wasn't a list
         if not isinstance(cfgmnt[i], list):
-            log.warning("Mount option %s not a list, got a %s instead",
-                        (i + 1), type_utils.obj_name(cfgmnt[i]))
+            LOG.warning(
+                "Mount option %s not a list, got a %s instead",
+                (i + 1),
+                type_utils.obj_name(cfgmnt[i]),
+            )
             continue
 
         start = str(cfgmnt[i][0])
-        sanitized = sanitize_devname(start, cloud.device_name_to_device, log)
+        sanitized = sanitize_devname(
+            start, cloud.device_name_to_device, aliases=device_aliases
+        )
         if sanitized != start:
-            log.debug("changed %s => %s" % (start, sanitized))
+            LOG.debug("changed %s => %s", start, sanitized)
 
         if sanitized is None:
-            log.debug("Ignoring nonexistent named mount %s", start)
+            LOG.debug("Ignoring nonexistent named mount %s", start)
             continue
         elif sanitized in fstab_devs:
-            log.info("Device %s already defined in fstab: %s",
-                     sanitized, fstab_devs[sanitized])
+            LOG.info(
+                "Device %s already defined in fstab: %s",
+                sanitized,
+                fstab_devs[sanitized],
+            )
             continue
 
         cfgmnt[i][0] = sanitized
@@ -427,16 +522,21 @@ def handle(_name, cfg, cloud, log, _args):
     # entry has the same device name
     for defmnt in defmnts:
         start = defmnt[0]
-        sanitized = sanitize_devname(start, cloud.device_name_to_device, log)
+        sanitized = sanitize_devname(
+            start, cloud.device_name_to_device, aliases=device_aliases
+        )
         if sanitized != start:
-            log.debug("changed default device %s => %s" % (start, sanitized))
+            LOG.debug("changed default device %s => %s", start, sanitized)
 
         if sanitized is None:
-            log.debug("Ignoring nonexistent default named mount %s", start)
+            LOG.debug("Ignoring nonexistent default named mount %s", start)
             continue
         elif sanitized in fstab_devs:
-            log.debug("Device %s already defined in fstab: %s",
-                      sanitized, fstab_devs[sanitized])
+            LOG.debug(
+                "Device %s already defined in fstab: %s",
+                sanitized,
+                fstab_devs[sanitized],
+            )
             continue
 
         defmnt[0] = sanitized
@@ -448,8 +548,7 @@ def handle(_name, cfg, cloud, log, _args):
                 break
 
         if cfgmnt_has:
-            log.debug(("Not including %s, already"
-                       " previously included"), start)
+            LOG.debug("Not including %s, already previously included", start)
             continue
         cfgmnt.append(defmnt)
 
@@ -458,38 +557,39 @@ def handle(_name, cfg, cloud, log, _args):
     actlist = []
     for x in cfgmnt:
         if x[1] is None:
-            log.debug("Skipping nonexistent device named %s", x[0])
+            LOG.debug("Skipping nonexistent device named %s", x[0])
         else:
             actlist.append(x)
 
-    swapret = handle_swapcfg(cfg.get('swap', {}))
+    swapret = handle_swapcfg(cfg.get("swap", {}))
     if swapret:
         actlist.append([swapret, "none", "swap", "sw", "0", "0"])
 
     if len(actlist) == 0:
-        log.debug("No modifications to fstab needed")
+        LOG.debug("No modifications to fstab needed")
         return
 
     cc_lines = []
     needswap = False
     need_mount_all = False
     dirs = []
-    for line in actlist:
+    for entry in actlist:
         # write 'comment' in the fs_mntops, entry,  claiming this
-        line[3] = "%s,%s" % (line[3], MNT_COMMENT)
-        if line[2] == "swap":
+        entry[3] = "%s,%s" % (entry[3], MNT_COMMENT)
+        if entry[2] == "swap":
             needswap = True
-        if line[1].startswith("/"):
-            dirs.append(line[1])
-        cc_lines.append('\t'.join(line))
+        if entry[1].startswith("/"):
+            dirs.append(entry[1])
+        cc_lines.append("\t".join(entry))
 
-    mount_points = [v['mountpoint'] for k, v in util.mounts().items()
-                    if 'mountpoint' in v]
+    mount_points = [
+        v["mountpoint"] for k, v in util.mounts().items() if "mountpoint" in v
+    ]
     for d in dirs:
         try:
             util.ensure_dir(d)
         except Exception:
-            util.logexc(log, "Failed to make '%s' config-mount", d)
+            util.logexc(LOG, "Failed to make '%s' config-mount", d)
         # dirs is list of directories on which a volume should be mounted.
         # If any of them does not already show up in the list of current
         # mount points, we will definitely need to do mount -a.
@@ -499,11 +599,12 @@ def handle(_name, cfg, cloud, log, _args):
     sadds = [WS.sub(" ", n) for n in cc_lines]
     sdrops = [WS.sub(" ", n) for n in fstab_removed]
 
-    sops = (["- " + drop for drop in sdrops if drop not in sadds] +
-            ["+ " + add for add in sadds if add not in sdrops])
+    sops = ["- " + drop for drop in sdrops if drop not in sadds] + [
+        "+ " + add for add in sadds if add not in sdrops
+    ]
 
     fstab_lines.extend(cc_lines)
-    contents = "%s\n" % ('\n'.join(fstab_lines))
+    contents = "%s\n" % "\n".join(fstab_lines)
     util.write_file(FSTAB_PATH, contents)
 
     activate_cmds = []
@@ -511,9 +612,9 @@ def handle(_name, cfg, cloud, log, _args):
         activate_cmds.append(["swapon", "-a"])
 
     if len(sops) == 0:
-        log.debug("No changes to /etc/fstab made.")
+        LOG.debug("No changes to /etc/fstab made.")
     else:
-        log.debug("Changes to fstab: %s", sops)
+        LOG.debug("Changes to fstab: %s", sops)
         need_mount_all = True
 
     if need_mount_all:
@@ -523,12 +624,10 @@ def handle(_name, cfg, cloud, log, _args):
 
     fmt = "Activating swap and mounts with: %s"
     for cmd in activate_cmds:
-        fmt = "Activate mounts: %s:" + ' '.join(cmd)
+        fmt = "Activate mounts: %s:" + " ".join(cmd)
         try:
-            util.subp(cmd)
-            log.debug(fmt, "PASS")
-        except util.ProcessExecutionError:
-            log.warning(fmt, "FAIL")
-            util.logexc(log, fmt, "FAIL")
-
-# vi: ts=4 expandtab
+            subp.subp(cmd)
+            LOG.debug(fmt, "PASS")
+        except subp.ProcessExecutionError:
+            LOG.warning(fmt, "FAIL")
+            util.logexc(LOG, fmt, "FAIL")
