@@ -12,32 +12,19 @@ import logging
 from cloudinit import ssh_util, util
 from cloudinit.cloud import Cloud
 from cloudinit.config import Config
-from cloudinit.config.schema import MetaSchema, get_meta_doc
+from cloudinit.config.schema import MetaSchema
 from cloudinit.distros import ALL_DISTROS, ug_util
+from cloudinit.log import log_util
 from cloudinit.settings import PER_INSTANCE
 from cloudinit.simpletable import SimpleTable
 
-MODULE_DESCRIPTION = """\
-Write fingerprints of authorized keys for each user to log. This is enabled by
-default, but can be disabled using ``no_ssh_fingerprints``. The hash type for
-the keys can be specified, but defaults to ``sha256``.
-"""
-
 meta: MetaSchema = {
     "id": "cc_ssh_authkey_fingerprints",
-    "name": "SSH AuthKey Fingerprints",
-    "title": "Log fingerprints of user SSH keys",
-    "description": MODULE_DESCRIPTION,
     "distros": [ALL_DISTROS],
     "frequency": PER_INSTANCE,
-    "examples": [
-        "no_ssh_fingerprints: true",
-        "authkey_hash: sha512",
-    ],
     "activate_by_schema_keys": [],
 }
 
-__doc__ = get_meta_doc(meta)
 LOG = logging.getLogger(__name__)
 
 
@@ -81,7 +68,7 @@ def _pprint_key_entries(
             "%sno authorized SSH keys fingerprints found for user %s.\n"
             % (prefix, user)
         )
-        util.multi_log(message, console=True, stderr=False)
+        log_util.multi_log(message, console=True, stderr=False)
         return
     tbl_fields = [
         "Keytype",
@@ -111,7 +98,7 @@ def _pprint_key_entries(
     ]
     lines.extend(authtbl_lines)
     for line in lines:
-        util.multi_log(
+        log_util.multi_log(
             text="%s%s\n" % (prefix, line), stderr=False, console=True
         )
 
@@ -126,7 +113,7 @@ def handle(name: str, cfg: Config, cloud: Cloud, args: list) -> None:
 
     hash_meth = util.get_cfg_option_str(cfg, "authkey_hash", "sha256")
     (users, _groups) = ug_util.normalize_users_groups(cfg, cloud.distro)
-    for (user_name, _cfg) in users.items():
+    for user_name, _cfg in users.items():
         if _cfg.get("no_create_home") or _cfg.get("system"):
             LOG.debug(
                 "Skipping printing of ssh fingerprints for user '%s' because "
@@ -137,6 +124,3 @@ def handle(name: str, cfg: Config, cloud: Cloud, args: list) -> None:
 
         (key_fn, key_entries) = ssh_util.extract_authorized_keys(user_name)
         _pprint_key_entries(user_name, key_fn, key_entries, hash_meth)
-
-
-# vi: ts=4 expandtab
