@@ -12,15 +12,7 @@
 
 from cloudinit.distros import rhel
 
-from cloudinit import log as logging
-
-LOG = logging.getLogger(__name__)
-
 
 class Distro(rhel.Distro):
-
     def update_package_sources(self):
         return None
-
-
-# vi: ts=4 expandtab
