@@ -1,11 +1,11 @@
-import logging
 import platform
-import re
 from typing import List, Optional
 
 import cloudinit.net.netops.bsd_netops as bsd_netops
-from cloudinit import distros, helpers, net, subp, util
-from cloudinit.distros import PackageList, bsd_utils
+from cloudinit import distros, helpers
+from cloudinit import log as logging
+from cloudinit import net, subp, util
+from cloudinit.distros import bsd_utils
 from cloudinit.distros.networking import BSDNetworking
 
 LOG = logging.getLogger(__name__)
@@ -28,12 +28,12 @@ class BSD(distros.Distro):
     # There is no update/upgrade on OpenBSD
     pkg_cmd_update_prefix: Optional[List[str]] = None
     pkg_cmd_upgrade_prefix: Optional[List[str]] = None
-    net_ops = bsd_netops.BsdNetOps
+    net_ops = bsd_netops.BsdNetOps  # type: ignore
 
     def __init__(self, name, cfg, paths):
         super().__init__(name, cfg, paths)
         # This will be used to restrict certain
-        # calls from repeatedly happening (when they
+        # calls from repeatly happening (when they
         # should only happen say once per instance...)
         self._runner = helpers.Runners(paths)
         cfg["ssh_svcname"] = "sshd"
@@ -97,7 +97,7 @@ class BSD(distros.Distro):
             )
         return nconf
 
-    def install_packages(self, pkglist: PackageList):
+    def install_packages(self, pkglist):
         self.update_package_sources()
         self.package_command("install", pkgs=pkglist)
 
@@ -121,8 +121,6 @@ class BSD(distros.Distro):
             if not self.pkg_cmd_upgrade_prefix:
                 return
             cmd = self.pkg_cmd_upgrade_prefix
-        else:
-            cmd = []
 
         if args and isinstance(args, str):
             cmd.append(args)
@@ -133,7 +131,7 @@ class BSD(distros.Distro):
         cmd.extend(pkglist)
 
         # Allow the output of this to flow outwards (ie not be captured)
-        subp.subp(cmd, update_env=self._get_pkg_cmd_environ(), capture=False)
+        subp.subp(cmd, env=self._get_pkg_cmd_environ(), capture=False)
 
     def set_timezone(self, tz):
         distros.set_etc_timezone(tz=tz, tz_file=self._find_tz_file(tz))
@@ -144,32 +142,3 @@ class BSD(distros.Distro):
     def chpasswd(self, plist_in: list, hashed: bool):
         for name, password in plist_in:
             self.set_passwd(name, password, hashed=hashed)
-
-    @staticmethod
-    def get_proc_ppid(pid):
-        """
-        Return the parent pid of a process by checking ps
-        """
-        ppid, _ = subp.subp(["ps", "-oppid=", "-p", str(pid)])
-        return int(ppid.strip())
-
-    @staticmethod
-    def get_mapped_device(blockdev: str) -> Optional[str]:
-        return None
-
-    @staticmethod
-    def device_part_info(devpath: str) -> tuple:
-        # FreeBSD doesn't know of sysfs so just get everything we need from
-        # the device, like /dev/vtbd0p2.
-        part = util.find_freebsd_part(devpath)
-        if part:
-            fpart = f"/dev/{part}"
-            # Handle both GPT partitions and MBR slices with partitions
-            m = re.search(
-                r"^(?P<dev>/dev/.+)[sp](?P<part_slice>\d+[a-z]*)$", fpart
-            )
-            if m:
-                return m["dev"], m["part_slice"]
-
-        # the input is bogus and we need to bail
-        raise ValueError(f"Invalid value for devpath: '{devpath}'")
