@@ -5,13 +5,14 @@
 # This file is part of cloud-init. See LICENSE file for license information.
 
 import copy
-import logging
 import re
 from collections import defaultdict
 from functools import lru_cache
 from ipaddress import IPv4Address, IPv4Interface, IPv6Interface
 from typing import Dict, List, Optional, Tuple, Union
 
+from cloudinit import log as logging
+
 LOG = logging.getLogger(__name__)
 
 MAC_RE = r"""([0-9a-fA-F]{2}:){5}[0-9a-fA-F]{2}"""
