@@ -8,7 +8,8 @@
 #
 # This file is part of cloud-init. See LICENSE file for license information.
 
-from cloudinit import log as logging
+import logging
+
 from cloudinit import util
 from cloudinit.distros.parsers.sys_conf import SysConf
 
@@ -47,6 +48,3 @@ def read_sysconfig_file(fn):
     except IOError:
         contents = []
     return (exists, SysConf(contents))
-
-
-# vi: ts=4 expandtab
