@@ -13,7 +13,6 @@ import copy
 
 from cloudinit.distros import PREFERRED_NTP_CLIENTS, debian
 from cloudinit.distros.package_management.snap import Snap
-from cloudinit.net.netplan import CLOUDINIT_NETPLAN_FILE
 
 
 class Distro(debian.Distro):
@@ -22,7 +21,7 @@ class Distro(debian.Distro):
         # Ubuntu specific network cfg locations
         self.network_conf_fn = {
             "eni": "/etc/network/interfaces.d/50-cloud-init.cfg",
-            "netplan": CLOUDINIT_NETPLAN_FILE,
+            "netplan": "/etc/netplan/50-cloud-init.yaml",
         }
         self.renderer_configs = {
             "eni": {
@@ -40,8 +39,7 @@ class Distro(debian.Distro):
 
     def package_command(self, command, args=None, pkgs=None):
         super().package_command(command, args, pkgs)
-        if self.snap.available():
-            self.snap.upgrade_packages()
+        self.snap.upgrade_packages()
 
     @property
     def preferred_ntp_clients(self):
