@@ -7,9 +7,9 @@
 
 """gpg.py - Collection of gpg key related functions"""
 
-import logging
 import time
 
+from cloudinit import log as logging
 from cloudinit import subp
 
 LOG = logging.getLogger(__name__)
@@ -41,7 +41,7 @@ def dearmor(key):
 
     note: man gpg(1) makes no mention of an --armour spelling, only --armor
     """
-    return subp.subp(["gpg", "--dearmor"], data=key, decode=False).stdout
+    return subp.subp(["gpg", "--dearmor"], data=key, decode=False)[0]
 
 
 def list(key_file, human_output=False):
@@ -137,3 +137,6 @@ def getkeybyid(keyid, keyserver="keyserver.ubuntu.com"):
             delete_key(keyid)
 
     return armour
+
+
+# vi: ts=4 expandtab
