@@ -5,7 +5,7 @@
 # This file is part of cloud-init. See LICENSE file for license information.
 
 
-class Merger:
+class Merger(object):
     def __init__(self, _merger, opts):
         self._append = "append" in opts
 
@@ -26,3 +26,6 @@ class Merger:
         if not self._append:
             return merge_with
         return value + merge_with
+
+
+# vi: ts=4 expandtab
