@@ -15,7 +15,6 @@ from typing import Any, Callable, Dict, List, Optional, Tuple
 from urllib.parse import urlparse
 
 from cloudinit import subp, util
-from cloudinit.net.netops.iproute2 import Iproute2
 from cloudinit.url_helper import UrlError, readurl
 
 LOG = logging.getLogger(__name__)
@@ -74,7 +73,7 @@ def read_sys_net(
 ):
     dev_path = sys_dev_path(devname, path)
     try:
-        contents = util.load_text_file(dev_path)
+        contents = util.load_file(dev_path)
     except (OSError, IOError) as e:
         e_errno = getattr(e, "errno", None)
         if e_errno in (errno.ENOENT, errno.ENOTDIR):
@@ -243,7 +242,7 @@ def get_dev_features(devname):
 
 
 def has_netfail_standby_feature(devname):
-    """Return True if VIRTIO_NET_F_STANDBY bit (62) is set.
+    """ Return True if VIRTIO_NET_F_STANDBY bit (62) is set.
 
     https://github.com/torvalds/linux/blob/ \
         089cf7f6ecb266b6a4164919a2e69bd2f938374a/ \
@@ -555,8 +554,10 @@ def find_fallback_nic_on_linux() -> Optional[str]:
     return None
 
 
-def generate_fallback_config(config_driver=None) -> Optional[dict]:
+def generate_fallback_config(config_driver=None):
     """Generate network cfg v2 for dhcp on the NIC most likely connected."""
+    if not config_driver:
+        config_driver = False
 
     target_name = find_fallback_nic()
     if not target_name:
@@ -570,16 +571,11 @@ def generate_fallback_config(config_driver=None) -> Optional[dict]:
         match = {
             "macaddress": read_sys_net_safe(target_name, "address").lower()
         }
+    cfg = {"dhcp4": True, "set-name": target_name, "match": match}
     if config_driver:
         driver = device_driver(target_name)
         if driver:
-            match["driver"] = driver
-    cfg = {
-        "dhcp4": True,
-        "dhcp6": True,
-        "set-name": target_name,
-        "match": match,
-    }
+            cfg["match"]["driver"] = driver
     nconf = {"ethernets": {target_name: cfg}, "version": 2}
     return nconf
 
@@ -640,7 +636,7 @@ def interface_has_own_mac(ifname, strict=False):
     are bonds or vlans that inherit their mac from another device.
     Possible values are:
       0: permanent address    2: stolen from another device
-    1: randomly generated   3: set using dev_set_mac_address"""
+      1: randomly generated   3: set using dev_set_mac_address"""
 
     assign_type = read_sys_net_int(ifname, "addr_assign_type")
     if assign_type is None:
@@ -668,7 +664,7 @@ def _get_current_rename_info(check_downable=True):
          }}
     """
     cur_info = {}
-    for name, mac, driver, device_id in get_interfaces():
+    for (name, mac, driver, device_id) in get_interfaces():
         cur_info[name] = {
             "downable": None,
             "device_id": device_id,
@@ -720,7 +716,16 @@ def _rename_interfaces(
     LOG.debug("Detected interfaces %s", cur_info)
 
     def update_byname(bymac):
-        return dict((data["name"], data) for data in bymac.values())
+        return dict((data["name"], data) for data in cur_info.values())
+
+    def rename(cur, new):
+        subp.subp(["ip", "link", "set", cur, "name", new], capture=True)
+
+    def down(name):
+        subp.subp(["ip", "link", "set", name, "down"], capture=True)
+
+    def up(name):
+        subp.subp(["ip", "link", "set", name, "up"], capture=True)
 
     ops = []
     errors = []
@@ -825,23 +830,15 @@ def _rename_interfaces(
         cur_byname = update_byname(cur_info)
         ops += cur_ops
 
-    opmap = {
-        "rename": Iproute2.link_rename,
-        "down": Iproute2.link_down,
-        "up": Iproute2.link_up,
-    }
+    opmap = {"rename": rename, "down": down, "up": up}
 
     if len(ops) + len(ups) == 0:
         if len(errors):
-            LOG.warning(
-                "Unable to rename interfaces: %s due to errors: %s",
-                renames,
-                errors,
-            )
+            LOG.debug("unable to do any work for renaming of %s", renames)
         else:
             LOG.debug("no work necessary for renaming of %s", renames)
     else:
-        LOG.debug("Renamed %s with ops %s", renames, ops + ups)
+        LOG.debug("achieving renaming of %s with ops %s", renames, ops + ups)
 
         for op, mac, new_name, params in ops + ups:
             try:
@@ -1281,54 +1278,10 @@ def is_ipv6_network(address: str) -> bool:
     )
 
 
-def is_ip_in_subnet(address: str, subnet: str) -> bool:
-    """Returns a bool indicating if ``s`` is in subnet.
-
-    :param address:
-        The string of IP address.
-
-    :param subnet:
-        The string of subnet.
-
-    :return:
-        A bool indicating if the string is in subnet.
-    """
-    ip_address = ipaddress.ip_address(address)
-    subnet_network = ipaddress.ip_network(subnet, strict=False)
-    return ip_address in subnet_network
-
-
-def should_add_gateway_onlink_flag(gateway: str, subnet: str) -> bool:
-    """Returns a bool indicating if should add gateway onlink flag.
-
-    :param gateway:
-        The string of gateway address.
-
-    :param subnet:
-        The string of subnet.
-
-    :return:
-        A bool indicating if the string is in subnet.
-    """
-    try:
-        return not is_ip_in_subnet(gateway, subnet)
-    except ValueError as e:
-        LOG.warning(
-            "Failed to check whether gateway %s"
-            " is contained within subnet %s: %s",
-            gateway,
-            subnet,
-            e,
-        )
-        return False
-
-
 def subnet_is_ipv6(subnet) -> bool:
     """Common helper for checking network_state subnets for ipv6."""
     # 'static6', 'dhcp6', 'ipv6_dhcpv6-stateful', 'ipv6_dhcpv6-stateless' or
     # 'ipv6_slaac'
-    # This function is inappropriate for v2-based routes as routes defined
-    # under v2 subnets can contain ipv4 and ipv6 simultaneously
     if subnet["type"].endswith("6") or subnet["type"] in IPV6_DYNAMIC_TYPES:
         # This is a request either static6 type or DHCPv6.
         return True
