@@ -9,7 +9,7 @@ import abc
 import io
 from typing import Optional
 
-from cloudinit.net.network_state import NetworkState
+from cloudinit.net.network_state import NetworkState, parse_net_config_data
 from cloudinit.net.udev import generate_udev_rule
 
 
@@ -17,6 +17,10 @@ def filter_by_type(match_type):
     return lambda iface: match_type == iface["type"]
 
 
+def filter_by_name(match_name):
+    return lambda iface: match_name == iface["name"]
+
+
 def filter_by_attr(match_name):
     return lambda iface: (match_name in iface and iface[match_name])
 
@@ -24,7 +28,7 @@ def filter_by_attr(match_name):
 filter_by_physical = filter_by_type("physical")
 
 
-class Renderer(abc.ABC):
+class Renderer(object):
     def __init__(self, config=None):
         pass
 
@@ -53,3 +57,18 @@ class Renderer(abc.ABC):
         target=None,
     ) -> None:
         """Render network state."""
+
+    def render_network_config(
+        self,
+        network_config: dict,
+        templates: Optional[dict] = None,
+        target=None,
+    ):
+        return self.render_network_state(
+            network_state=parse_net_config_data(network_config),
+            templates=templates,
+            target=target,
+        )
+
+
+# vi: ts=4 expandtab
