@@ -10,6 +10,7 @@ from base64 import b64decode
 from cloudinit import dmi
 from cloudinit import log as logging
 from cloudinit import sources
+from cloudinit.sources import DataSourceHostname
 from cloudinit.sources.helpers.cloudsigma import SERIAL_PORT, Cepko
 
 LOG = logging.getLogger(__name__)
@@ -30,7 +31,8 @@ class DataSourceCloudSigma(sources.DataSource):
         self.ssh_public_key = ""
         sources.DataSource.__init__(self, sys_cfg, distro, paths)
 
-    def is_running_in_cloudsigma(self):
+    @staticmethod
+    def ds_detect():
         """
         Uses dmi data to detect if this instance of cloud-init is running
         in the CloudSigma's infrastructure.
@@ -50,8 +52,6 @@ class DataSourceCloudSigma(sources.DataSource):
         as userdata.
         """
         dsmode = None
-        if not self.is_running_in_cloudsigma():
-            return False
 
         try:
             server_context = self.cepko.all().result
@@ -90,9 +90,10 @@ class DataSourceCloudSigma(sources.DataSource):
         the first part from uuid is being used.
         """
         if re.match(r"^[A-Za-z0-9 -_\.]+$", self.metadata["name"]):
-            return self.metadata["name"][:61]
+            ret = self.metadata["name"][:61]
         else:
-            return self.metadata["uuid"].split("-")[0]
+            ret = self.metadata["uuid"].split("-")[0]
+        return DataSourceHostname(ret, False)
 
     def get_public_ssh_keys(self):
         return [self.ssh_public_key]
