@@ -6,8 +6,9 @@
 # DigitalOcean Droplet API:
 # https://developers.digitalocean.com/documentation/metadata/
 
+import logging
+
 import cloudinit.sources.helpers.digitalocean as do_helper
-from cloudinit import log as logging
 from cloudinit import sources, util
 
 LOG = logging.getLogger(__name__)
@@ -40,12 +41,25 @@ class DataSourceDigitalOcean(sources.DataSource):
                 BUILTIN_DS_CONFIG,
             ]
         )
+        self._deprecate()
         self.metadata_address = self.ds_cfg["metadata_url"]
         self.retries = self.ds_cfg.get("retries", MD_RETRIES)
         self.timeout = self.ds_cfg.get("timeout", MD_TIMEOUT)
         self.use_ip4LL = self.ds_cfg.get("use_ip4LL", MD_USE_IPV4LL)
         self.wait_retry = self.ds_cfg.get("wait_retry", MD_WAIT_RETRY)
         self._network_config = None
+        self.metadata_full = None
+
+    def _unpickle(self, ci_pkl_version: int) -> None:
+        super()._unpickle(ci_pkl_version)
+        self._deprecate()
+
+    def _deprecate(self):
+        util.deprecate(
+            deprecated="DataSourceDigitalOcean",
+            deprecated_version="23.2",
+            extra_message="Deprecated in favour of DataSourceConfigDrive.",
+        )
 
     def _get_sysinfo(self):
         return do_helper.read_sysinfo()
@@ -80,7 +94,7 @@ class DataSourceDigitalOcean(sources.DataSource):
         self.userdata_raw = md.get("user_data", None)
 
         if ipv4LL_nic:
-            do_helper.del_ipv4_link_local(ipv4LL_nic)
+            do_helper.del_ipv4_link_local(self.distro, ipv4LL_nic)
 
         return True
 
@@ -120,6 +134,3 @@ datasources = [
 # Return a list of data sources that match this set of dependencies
 def get_datasource_list(depends):
     return sources.list_from_depends(depends, datasources)
-
-
-# vi: ts=4 expandtab
