@@ -4,15 +4,14 @@
 
 import datetime
 import json
-import logging
 from base64 import b64decode
+from contextlib import suppress as noop
 
-from cloudinit import dmi, net, sources, url_helper, util
+from cloudinit import dmi
+from cloudinit import log as logging
+from cloudinit import sources, url_helper, util
 from cloudinit.distros import ug_util
-from cloudinit.event import EventScope, EventType
-from cloudinit.net.dhcp import NoDHCPLeaseError
-from cloudinit.net.ephemeral import EphemeralDHCPv4
-from cloudinit.sources import DataSourceHostname
+from cloudinit.net.dhcp import EphemeralDHCPv4
 
 LOG = logging.getLogger(__name__)
 
@@ -25,10 +24,9 @@ GUEST_ATTRIBUTES_URL = (
 )
 HOSTKEY_NAMESPACE = "hostkeys"
 HEADERS = {"Metadata-Flavor": "Google"}
-DEFAULT_PRIMARY_INTERFACE = "ens4"
 
 
-class GoogleMetadataFetcher:
+class GoogleMetadataFetcher(object):
     def __init__(self, metadata_address, num_retries, sec_between_retries):
         self.metadata_address = metadata_address
         self.num_retries = num_retries
@@ -64,12 +62,6 @@ class DataSourceGCE(sources.DataSource):
 
     dsname = "GCE"
     perform_dhcp_setup = False
-    default_update_events = {
-        EventScope.NETWORK: {
-            EventType.BOOT_NEW_INSTANCE,
-            EventType.BOOT,
-        }
-    }
 
     def __init__(self, sys_cfg, distro, paths):
         sources.DataSource.__init__(self, sys_cfg, distro, paths)
@@ -88,50 +80,10 @@ class DataSourceGCE(sources.DataSource):
 
     def _get_data(self):
         url_params = self.get_url_params()
+        network_context = noop()
         if self.perform_dhcp_setup:
-            candidate_nics = net.find_candidate_nics()
-            if DEFAULT_PRIMARY_INTERFACE in candidate_nics:
-                candidate_nics.remove(DEFAULT_PRIMARY_INTERFACE)
-                candidate_nics.insert(0, DEFAULT_PRIMARY_INTERFACE)
-            LOG.debug("Looking for the primary NIC in: %s", candidate_nics)
-            assert (
-                len(candidate_nics) >= 1
-            ), "The instance has to have at least one candidate NIC"
-            for candidate_nic in candidate_nics:
-                network_context = EphemeralDHCPv4(
-                    self.distro,
-                    iface=candidate_nic,
-                )
-                try:
-                    with network_context:
-                        try:
-                            ret = util.log_time(
-                                LOG.debug,
-                                "Crawl of GCE metadata service",
-                                read_md,
-                                kwargs={
-                                    "address": self.metadata_address,
-                                    "url_params": url_params,
-                                },
-                            )
-                        except Exception as e:
-                            LOG.debug(
-                                "Error fetching IMD with candidate NIC %s: %s",
-                                candidate_nic,
-                                e,
-                            )
-                            continue
-                except NoDHCPLeaseError:
-                    continue
-                if ret["success"]:
-                    self._fallback_interface = candidate_nic
-                    LOG.debug("Primary NIC found: %s.", candidate_nic)
-                    break
-            if self._fallback_interface is None:
-                LOG.warning(
-                    "Did not find a fallback interface on %s.", self.cloud_name
-                )
-        else:
+            network_context = EphemeralDHCPv4(self.fallback_interface)
+        with network_context:
             ret = util.log_time(
                 LOG.debug,
                 "Crawl of GCE metadata service",
@@ -170,9 +122,7 @@ class DataSourceGCE(sources.DataSource):
 
     def get_hostname(self, fqdn=False, resolve_ip=False, metadata_only=False):
         # GCE has long FDQN's and has asked for short hostnames.
-        return DataSourceHostname(
-            self.metadata["local-hostname"].split(".")[0], False
-        )
+        return self.metadata["local-hostname"].split(".")[0]
 
     @property
     def availability_zone(self):
@@ -398,3 +348,5 @@ if __name__ == "__main__":
             data["user-data-b64"] = b64encode(data["user-data"]).decode()
 
     print(json.dumps(data, indent=1, sort_keys=True, separators=(",", ": ")))
+
+# vi: ts=4 expandtab
