@@ -6,7 +6,6 @@
 # https://developer.scaleway.com/#metadata
 
 import json
-import logging
 import os
 import socket
 import time
@@ -19,7 +18,9 @@ from requests.exceptions import ConnectionError
 from urllib3.connection import HTTPConnection
 from urllib3.poolmanager import PoolManager
 
-from cloudinit import dmi, sources, url_helper, util
+from cloudinit import dmi
+from cloudinit import log as logging
+from cloudinit import net, sources, url_helper, util
 from cloudinit.event import EventScope, EventType
 from cloudinit.net.dhcp import NoDHCPLeaseError
 from cloudinit.net.ephemeral import EphemeralDHCPv4, EphemeralIPv6Network
@@ -78,7 +79,7 @@ def query_data_api_once(api_address, timeout, requests_session):
             api_address,
             data=None,
             timeout=timeout,
-            # It's the caller's responsibility to recall this function in case
+            # It's the caller's responsability to recall this function in case
             # of exception. Don't let url_helper.readurl() retry by itself.
             retries=0,
             session=requests_session,
@@ -171,9 +172,9 @@ class DataSourceScaleway(sources.DataSource):
         self.retries = int(self.ds_cfg.get("retries", DEF_MD_RETRIES))
         self.timeout = int(self.ds_cfg.get("timeout", DEF_MD_TIMEOUT))
         self.max_wait = int(self.ds_cfg.get("max_wait", DEF_MD_MAX_WAIT))
+        self._fallback_interface = None
         self._network_config = sources.UNSET
         self.metadata_urls = DS_BASE_URLS
-        self.metadata_url = None
         self.userdata_url = None
         self.vendordata_url = None
         self.ephemeral_fixed_address = None
@@ -181,26 +182,12 @@ class DataSourceScaleway(sources.DataSource):
         if "metadata_urls" in self.ds_cfg.keys():
             self.metadata_urls += self.ds_cfg["metadata_urls"]
 
-    def _unpickle(self, ci_pkl_version: int) -> None:
-        super()._unpickle(ci_pkl_version)
-        attr_defaults = {
-            "ephemeral_fixed_address": None,
-            "has_ipv4": True,
-            "max_wait": DEF_MD_MAX_WAIT,
-            "metadata_urls": DS_BASE_URLS,
-            "userdata_url": None,
-            "vendordata_url": None,
-        }
-        for attr in attr_defaults:
-            if not hasattr(self, attr):
-                setattr(self, attr, attr_defaults[attr])
-
     def _set_metadata_url(self, urls):
         """
         Define metadata_url based upon api-metadata URL availability.
         """
 
-        start_time = time.monotonic()
+        start_time = time.time()
         avail_url, _ = url_helper.wait_for_url(
             urls=urls,
             max_wait=self.max_wait,
@@ -217,7 +204,7 @@ class DataSourceScaleway(sources.DataSource):
             LOG.debug(
                 "Unable to reach api-metadata at %s after %s seconds",
                 urls,
-                int(time.monotonic() - start_time),
+                int(time.time() - start_time),
             )
             raise ConnectionError
 
@@ -281,6 +268,9 @@ class DataSourceScaleway(sources.DataSource):
 
     def _get_data(self):
 
+        if self._fallback_interface is None:
+            self._fallback_interface = net.find_fallback_nic()
+
         # The DataSource uses EventType.BOOT so we are called more than once.
         # Try to crawl metadata on IPv4 first and set has_ipv4 to False if we
         # timeout so we do not try to crawl on IPv4 more than once.
@@ -291,7 +281,7 @@ class DataSourceScaleway(sources.DataSource):
                 # it will only reach timeout on VMs with only IPv6 addresses.
                 with EphemeralDHCPv4(
                     self.distro,
-                    self.distro.fallback_interface,
+                    self._fallback_interface,
                 ) as ipv4:
                     util.log_time(
                         logfunc=LOG.debug,
@@ -321,8 +311,7 @@ class DataSourceScaleway(sources.DataSource):
         if not self.has_ipv4:
             try:
                 with EphemeralIPv6Network(
-                    self.distro,
-                    self.distro.fallback_interface,
+                    self._fallback_interface,
                 ):
                     util.log_time(
                         logfunc=LOG.debug,
@@ -337,7 +326,7 @@ class DataSourceScaleway(sources.DataSource):
                         func=self._crawl_metadata,
                     )
                     self.metadata["net_in_use"] = "ipv6"
-            except ConnectionError:
+            except (ConnectionError):
                 return False
         return True
 
@@ -357,6 +346,9 @@ class DataSourceScaleway(sources.DataSource):
         if self._network_config != sources.UNSET:
             return self._network_config
 
+        if self._fallback_interface is None:
+            self._fallback_interface = net.find_fallback_nic()
+
         if self.metadata["private_ip"] is None:
             # New method of network configuration
 
@@ -385,13 +377,13 @@ class DataSourceScaleway(sources.DataSource):
                             ip_cfg["routes"] += [route]
                         else:
                             ip_cfg["routes"] = [route]
-            netcfg[self.distro.fallback_interface] = ip_cfg
+            netcfg[self._fallback_interface] = ip_cfg
             self._network_config = {"version": 2, "ethernets": netcfg}
         else:
             # Kept for backward compatibility
             netcfg = {
                 "type": "physical",
-                "name": "%s" % self.distro.fallback_interface,
+                "name": "%s" % self._fallback_interface,
             }
             subnets = [{"type": "dhcp4"}]
             if self.metadata["ipv6"]:
