@@ -5,30 +5,37 @@
 # Vultr Metadata API:
 # https://www.vultr.com/metadata/
 
-from cloudinit import log as log
-from cloudinit import sources
-from cloudinit import util
-
 import cloudinit.sources.helpers.vultr as vultr
+from cloudinit import log as log
+from cloudinit import sources, util, version
 
 LOG = log.getLogger(__name__)
 BUILTIN_DS_CONFIG = {
-    'url': 'http://169.254.169.254',
-    'retries': 30,
-    'timeout': 2,
-    'wait': 2
+    "url": "http://169.254.169.254",
+    "retries": 30,
+    "timeout": 10,
+    "wait": 5,
+    "user-agent": "Cloud-Init/%s - OS: %s Variant: %s"
+    % (
+        version.version_string(),
+        util.system_info()["system"],
+        util.system_info()["variant"],
+    ),
 }
 
 
 class DataSourceVultr(sources.DataSource):
 
-    dsname = 'Vultr'
+    dsname = "Vultr"
 
     def __init__(self, sys_cfg, distro, paths):
         super(DataSourceVultr, self).__init__(sys_cfg, distro, paths)
-        self.ds_cfg = util.mergemanydict([
-            util.get_cfg_by_path(sys_cfg, ["datasource", "Vultr"], {}),
-            BUILTIN_DS_CONFIG])
+        self.ds_cfg = util.mergemanydict(
+            [
+                util.get_cfg_by_path(sys_cfg, ["datasource", "Vultr"], {}),
+                BUILTIN_DS_CONFIG,
+            ]
+        )
 
     # Initiate data and check if Vultr
     def _get_data(self):
@@ -40,22 +47,17 @@ class DataSourceVultr(sources.DataSource):
         LOG.debug("Machine is a Vultr instance")
 
         # Fetch metadata
-        md = self.get_metadata()
-
-        self.metadata_full = md
-        self.metadata['instanceid'] = md['instanceid']
-        self.metadata['local-hostname'] = md['hostname']
-        self.metadata['public-keys'] = md["public-keys"]
-        self.userdata_raw = md["user-data"]
+        self.metadata = self.get_metadata()
+        self.userdata_raw = self.metadata["user-data"]
 
         # Generate config and process data
-        self.get_datasource_data(md)
+        self.get_datasource_data(self.metadata)
 
         # Dump some data so diagnosing failures is manageable
         LOG.debug("Vultr Vendor Config:")
-        LOG.debug(md['vendor-data']['config'])
-        LOG.debug("SUBID: %s", self.metadata['instanceid'])
-        LOG.debug("Hostname: %s", self.metadata['local-hostname'])
+        LOG.debug(util.json_dumps(self.metadata["vendor-data"]))
+        LOG.debug("SUBID: %s", self.metadata["instance-id"])
+        LOG.debug("Hostname: %s", self.metadata["local-hostname"])
         if self.userdata_raw is not None:
             LOG.debug("User-Data:")
             LOG.debug(self.userdata_raw)
@@ -64,35 +66,35 @@ class DataSourceVultr(sources.DataSource):
 
     # Process metadata
     def get_datasource_data(self, md):
-        # Grab config
-        config = md['vendor-data']['config']
-
         # Generate network config
-        self.netcfg = vultr.generate_network_config(md['interfaces'])
-
-        # This requires info generated in the vendor config
-        user_scripts = vultr.generate_user_scripts(md, self.netcfg['config'])
+        if "cloud_interfaces" in md:
+            # In the future we will just drop pre-configured
+            # network configs into the array. They need names though.
+            vultr.add_interface_names(md["cloud_interfaces"])
+            self.netcfg = md["cloud_interfaces"]
+        else:
+            self.netcfg = vultr.generate_network_config(md["interfaces"])
+        # Grab vendordata
+        self.vendordata_raw = md["vendor-data"]
 
         # Default hostname is "guest" for whitelabel
-        if self.metadata['local-hostname'] == "":
-            self.metadata['local-hostname'] = "guest"
+        if self.metadata["local-hostname"] == "":
+            self.metadata["local-hostname"] = "guest"
 
         self.userdata_raw = md["user-data"]
         if self.userdata_raw == "":
             self.userdata_raw = None
 
-        # Assemble vendor-data
-        # This adds provided scripts and the config
-        self.vendordata_raw = []
-        self.vendordata_raw.extend(user_scripts)
-        self.vendordata_raw.append("#cloud-config\n%s" % config)
-
     # Get the metadata by flag
     def get_metadata(self):
-        return vultr.get_metadata(self.ds_cfg['url'],
-                                  self.ds_cfg['timeout'],
-                                  self.ds_cfg['retries'],
-                                  self.ds_cfg['wait'])
+        return vultr.get_metadata(
+            self.ds_cfg["url"],
+            self.ds_cfg["timeout"],
+            self.ds_cfg["retries"],
+            self.ds_cfg["wait"],
+            self.ds_cfg["user-agent"],
+            tmp_dir=self.distro.get_tmp_exec_path(),
+        )
 
     # Compare subid as instance id
     def check_instance_id(self, sys_cfg):
@@ -103,7 +105,7 @@ class DataSourceVultr(sources.DataSource):
         if vultr.is_baremetal():
             return False
 
-        subid = vultr.get_sysinfo()['subid']
+        subid = vultr.get_sysinfo()["subid"]
         return sources.instance_id_matches_system_uuid(subid)
 
     # Currently unsupported
@@ -118,7 +120,7 @@ class DataSourceVultr(sources.DataSource):
 
 # Used to match classes to dependencies
 datasources = [
-    (DataSourceVultr, (sources.DEP_FILESYSTEM, )),
+    (DataSourceVultr, (sources.DEP_FILESYSTEM,)),
 ]
 
 
@@ -134,14 +136,17 @@ if __name__ == "__main__":
         print("Machine is not a Vultr instance")
         sys.exit(1)
 
-    md = vultr.get_metadata(BUILTIN_DS_CONFIG['url'],
-                            BUILTIN_DS_CONFIG['timeout'],
-                            BUILTIN_DS_CONFIG['retries'],
-                            BUILTIN_DS_CONFIG['wait'])
-    config = md['vendor-data']['config']
+    md = vultr.get_metadata(
+        BUILTIN_DS_CONFIG["url"],
+        BUILTIN_DS_CONFIG["timeout"],
+        BUILTIN_DS_CONFIG["retries"],
+        BUILTIN_DS_CONFIG["wait"],
+        BUILTIN_DS_CONFIG["user-agent"],
+    )
+    config = md["vendor-data"]
     sysinfo = vultr.get_sysinfo()
 
     print(util.json_dumps(sysinfo))
-    print(config)
+    print(util.json_dumps(config))
 
 # vi: ts=4 expandtab
