@@ -10,12 +10,12 @@ import abc
 import base64
 import copy
 import functools
-import logging
 import os
 
+from cloudinit import ec2_utils
+from cloudinit import log as logging
 from cloudinit import net, sources, subp, url_helper, util
 from cloudinit.sources import BrokenMetadata
-from cloudinit.sources.helpers import ec2
 
 # See https://docs.openstack.org/user-guide/cli-config-drive.html
 
@@ -82,7 +82,7 @@ class NonReadable(IOError):
     pass
 
 
-class SourceMixin:
+class SourceMixin(object):
     def _ec2_name_to_device(self, name):
         if not self.ec2_metadata:
             return None
@@ -515,7 +515,7 @@ class MetadataReader(BaseReader):
         return url_helper.combine_url(base, *add_ons)
 
     def _read_ec2_metadata(self):
-        return ec2.get_instance_metadata(
+        return ec2_utils.get_instance_metadata(
             ssl_details=self.ssl_details,
             timeout=self.timeout,
             retries=self.retries,
@@ -642,14 +642,6 @@ def convert_net_json(network_json=None, known_macs=None):
                     }
                 )
 
-            dns_nameservers = [
-                service["address"]
-                for service in network.get("services", [])
-                if service.get("type") == "dns"
-            ]
-            if dns_nameservers:
-                subnet["dns_nameservers"] = dns_nameservers
-
             # Enable accept_ra for stateful and legacy ipv6_dhcp types
             if network["type"] in ["ipv6_dhcpv6-stateful", "ipv6_dhcp"]:
                 cfg.update({"accept-ra": True})
@@ -764,3 +756,6 @@ def convert_net_json(network_json=None, known_macs=None):
         config.append(cfg)
 
     return {"version": 1, "config": config}
+
+
+# vi: ts=4 expandtab
