@@ -10,30 +10,40 @@
 #
 # This file is part of cloud-init. See LICENSE file for license information.
 
+# noqa: E402
+
 import collections
+import logging
 import re
 import sys
+from typing import Any
 
-try:
-    from jinja2 import DebugUndefined as JUndefined
-    from jinja2 import Template as JTemplate
-
-    JINJA_AVAILABLE = True
-except (ImportError, AttributeError):
-    JINJA_AVAILABLE = False
-    JUndefined = object
-
-from cloudinit import log as logging
 from cloudinit import type_utils as tu
 from cloudinit import util
 from cloudinit.atomic_helper import write_file
 
+# After bionic EOL, mypy==1.0.0 will be able to type-analyse dynamic
+# base types, substitute this by:
+# JUndefined: typing.Type
+JUndefined: Any
+try:
+    from jinja2 import DebugUndefined as _DebugUndefined
+    from jinja2 import Template as JTemplate
+
+    JINJA_AVAILABLE = True
+    JUndefined = _DebugUndefined
+except (ImportError, AttributeError):
+    JINJA_AVAILABLE = False
+    JUndefined = object
+
 LOG = logging.getLogger(__name__)
 TYPE_MATCHER = re.compile(r"##\s*template:(.*)", re.I)
 BASIC_MATCHER = re.compile(r"\$\{([A-Za-z0-9_.]+)\}|\$([A-Za-z0-9_.]+)")
 MISSING_JINJA_PREFIX = "CI_MISSING_JINJA_VAR/"
 
 
+# Mypy, and the PEP 484 ecosystem in general, does not support creating
+# classes with dynamic base types: https://stackoverflow.com/a/59636248
 class UndefinedJinjaVariable(JUndefined):
     """Class used to represent any undefined jinja template variable."""
 
@@ -142,12 +152,6 @@ def render_to_file(fn, outfn, params, mode=0o644):
     util.write_file(outfn, contents, mode=mode)
 
 
-def render_string_to_file(content, outfn, params, mode=0o644):
-    """Render string"""
-    contents = render_string(content, params)
-    util.write_file(outfn, contents, mode=mode)
-
-
 def render_string(content, params):
     """Render string"""
     if not params:
@@ -156,13 +160,17 @@ def render_string(content, params):
     return renderer(content, params)
 
 
-def render_cloudcfg(variant, template, output):
-
+def render_template(variant, template, output, is_yaml, prefix=None):
     with open(template, "r") as fh:
         contents = fh.read()
-    tpl_params = {"variant": variant}
+    tpl_params = {"variant": variant, "prefix": prefix}
     contents = (render_string(contents, tpl_params)).rstrip() + "\n"
-    util.load_yaml(contents)
+    if is_yaml:
+        out = util.load_yaml(contents, default=True)
+        if not out:
+            raise RuntimeError(
+                "Cannot render template file %s - invalid yaml." % template
+            )
     if output == "-":
         sys.stdout.write(contents)
     else:
