@@ -19,7 +19,6 @@ import gzip
 import hashlib
 import io
 import json
-import logging
 import os
 import os.path
 import platform
@@ -52,9 +51,9 @@ from typing import (
 )
 from urllib import parse
 
+from cloudinit import features, importer
+from cloudinit import log as logging
 from cloudinit import (
-    features,
-    importer,
     mergers,
     net,
     safeyaml,
@@ -1166,8 +1165,7 @@ def read_cc_from_cmdline(cmdline=None):
     if cmdline is None:
         cmdline = get_cmdline()
 
-    cmdline = f" {cmdline}"
-    tag_begin = " cc:"
+    tag_begin = "cc:"
     tag_end = "end_cc"
     begin_l = len(tag_begin)
     end_l = len(tag_end)
@@ -2143,6 +2141,29 @@ def safe_int(possible_int):
         return None
 
 
+def compare_permission(mode1, mode2):
+    """Compare two file modes in octal.
+
+    If mode1 is less restrictive than mode2 return 1
+    If mode1 is more restrictive than mode2 return -1
+    If mode1 is same as mode2, return 0
+
+    The comparison starts from the permission of the
+    set of users in "others" and then works up to the
+    permission of "user" set.
+    """
+    # Convert modes to octal and reverse the last 3 digits
+    # so 0o640 would be become 0o046
+    mode1_oct = oct(mode1)[2:].rjust(3, "0")
+    mode2_oct = oct(mode2)[2:].rjust(3, "0")
+    m1 = int(mode1_oct[:-3] + mode1_oct[-3:][::-1], 8)
+    m2 = int(mode2_oct[:-3] + mode2_oct[-3:][::-1], 8)
+
+    # Then do a traditional cmp()
+    # https://docs.python.org/3.0/whatsnew/3.0.html#ordering-comparisons
+    return (m1 > m2) - (m1 < m2)
+
+
 def chmod(path, mode):
     real_mode = safe_int(mode)
     if path and real_mode:
@@ -2294,6 +2315,10 @@ def make_header(comment_char="#", base="created"):
     return header
 
 
+def abs_join(base, *paths):
+    return os.path.abspath(os.path.join(base, *paths))
+
+
 # shellify, takes a list of commands
 #  for each entry in the list
 #    if it is an array, shell protect it (with single ticks)
@@ -3210,7 +3235,6 @@ def deprecate(
     deprecated_version: str,
     extra_message: Optional[str] = None,
     schedule: int = 5,
-    return_log: bool = False,
 ):
     """Mark a "thing" as deprecated. Deduplicated deprecations are
     logged.
@@ -3227,8 +3251,6 @@ def deprecate(
     @param schedule: Manually set the deprecation schedule. Defaults to
         5 years. Leave a comment explaining your reason for deviation if
         setting this value.
-    @param return_log: Return log text rather than logging it. Useful for
-        running prior to logging setup.
 
     Note: uses keyword-only arguments to improve legibility
     """
@@ -3238,17 +3260,15 @@ def deprecate(
     dedup = hash(deprecated + message + deprecated_version + str(schedule))
     version = Version.from_str(deprecated_version)
     version_removed = Version(version.major + schedule, version.minor)
-    deprecate_msg = (
-        f"{deprecated} is deprecated in "
-        f"{deprecated_version} and scheduled to be removed in "
-        f"{version_removed}. {message}"
-    ).rstrip()
-    if return_log:
-        return deprecate_msg
     if dedup not in deprecate._log:  # type: ignore
         deprecate._log.add(dedup)  # type: ignore
+        deprecate_msg = (
+            f"{deprecated} is deprecated in "
+            f"{deprecated_version} and scheduled to be removed in "
+            f"{version_removed}. {message}"
+        ).rstrip()
         if hasattr(LOG, "deprecated"):
-            LOG.deprecated(deprecate_msg)  # type: ignore
+            LOG.deprecated(deprecate_msg)
         else:
             LOG.warning(deprecate_msg)
 
