@@ -48,19 +48,31 @@ class DistroDataOutdated(Exception):
     """Distribution data outdated."""
 
     def __init__(self):
-        super(DistroDataOutdated, self).__init__(
-            "Distribution data outdated. "
-            "Please check for an update for distro-info-data. See "
-            "/usr/share/doc/distro-info-data/README.Debian for details.")
+        super().__init__(
+            "Distribution data outdated. Please check for an update for distro-info-data. "
+            "See /usr/share/doc/distro-info-data/README.Debian for details."
+        )
 
 
-class DistroRelease(object):
+class DistroRelease:
     """Represents a distributions release"""
+
     # pylint: disable=too-few-public-methods
     # pylint: disable=too-many-instance-attributes
 
-    def __init__(self, version, codename, series, created=None, release=None, eol=None,
-                 eol_server=None, eol_esm=None):
+    def __init__(
+        self,
+        version,
+        codename,
+        series,
+        created=None,
+        release=None,
+        eol=None,
+        eol_esm=None,
+        eol_lts=None,
+        eol_elts=None,
+        eol_server=None,
+    ):
         # pylint: disable=too-many-arguments
         self.version = version
         self.codename = codename
@@ -68,20 +80,25 @@ class DistroRelease(object):
         self.created = created
         self.release = release
         self.eol = eol
-        self.eol_server = eol_server
+        self.eol_lts = eol_lts
+        self.eol_elts = eol_elts
         self.eol_esm = eol_esm
+        self.eol_server = eol_server
 
     def is_supported(self, date):
         """Check whether this release is supported on the given date."""
-        return date >= self.created and (self.eol is None or date <= self.eol or (
-            self.eol_server is not None and date <= self.eol_server))
+        return date >= self.created and (
+            self.eol is None
+            or date <= self.eol
+            or (self.eol_server is not None and date <= self.eol_server)
+        )
 
 
 def _get_date(row, column):
     return convert_date(row[column]) if column in row else None
 
 
-class DistroInfo(object):
+class DistroInfo:
     """Base class for distribution information.
     Use DebianDistroInfo or UbuntuDistroInfo instead of using this directly.
     """
@@ -89,16 +106,23 @@ class DistroInfo(object):
     def __init__(self, distro):
         self._distro = distro
         filename = os.path.join(_get_data_dir(), distro.lower() + ".csv")
-        csvfile = open(filename)
-        csv_reader = csv.DictReader(csvfile)
-        self._releases = []
-        for row in csv_reader:
-            release = DistroRelease(row['version'], row['codename'], row['series'],
-                                    _get_date(row, 'created'), _get_date(row, 'release'),
-                                    _get_date(row, 'eol'), _get_date(row, 'eol-server'),
-                                    _get_date(row, 'eol-esm'))
-            self._releases.append(release)
-        csvfile.close()
+        with open(filename, encoding="utf-8") as csvfile:
+            csv_reader = csv.DictReader(csvfile)
+            self._releases = []
+            for row in csv_reader:
+                release = DistroRelease(
+                    row["version"],
+                    row["codename"],
+                    row["series"],
+                    _get_date(row, "created"),
+                    _get_date(row, "release"),
+                    _get_date(row, "eol"),
+                    _get_date(row, "eol-esm"),
+                    _get_date(row, "eol-lts"),
+                    _get_date(row, "eol-elts"),
+                    _get_date(row, "eol-server"),
+                )
+                self._releases.append(release)
         self._date = datetime.date.today()
 
     @property
@@ -130,8 +154,11 @@ class DistroInfo(object):
         """Get latest development distribution based on the given date."""
         if date is None:
             date = self._date
-        distros = [x for x in self._avail(date) if x.release is None or
-                   (date < x.release and (x.eol is None or date <= x.eol))]
+        distros = [
+            x
+            for x in self._avail(date)
+            if x.release is None or (date < x.release and (x.eol is None or date <= x.eol))
+        ]
         if not distros:
             raise DistroDataOutdated()
         return self._format(result, distros[-1])
@@ -147,15 +174,20 @@ class DistroInfo(object):
         if format_string == "release":
             return release.version
 
-        raise ValueError("Only codename, fullname, object, and release are allowed "
-                         "result values, but not '" + format_string + "'.")
+        raise ValueError(
+            "Only codename, fullname, object, and release are allowed "
+            "result values, but not '" + format_string + "'."
+        )
 
     def stable(self, date=None, result="codename"):
         """Get latest stable distribution based on the given date."""
         if date is None:
             date = self._date
-        distros = [x for x in self._avail(date) if x.release is not None and
-                   date >= x.release and (x.eol is None or date <= x.eol)]
+        distros = [
+            x
+            for x in self._avail(date)
+            if x.release is not None and date >= x.release and (x.eol is None or date <= x.eol)
+        ]
         if not distros:
             raise DistroDataOutdated()
         return self._format(result, distros[-1])
@@ -173,8 +205,7 @@ class DistroInfo(object):
         if date is None:
             date = self._date
         supported = self.supported(date)
-        distros = [self._format(result, x) for x in self._avail(date)
-                   if x.series not in supported]
+        distros = [self._format(result, x) for x in self._avail(date) if x.series not in supported]
         return distros
 
 
@@ -182,7 +213,7 @@ class DebianDistroInfo(DistroInfo):
     """provides information about Debian's distributions"""
 
     def __init__(self):
-        super(DebianDistroInfo, self).__init__("Debian")
+        super().__init__("Debian")
 
     def codename(self, release, date=None, default=None):
         """Map 'unstable', 'testing', etc. to their codenames."""
@@ -202,8 +233,11 @@ class DebianDistroInfo(DistroInfo):
         """Get latest development distribution based on the given date."""
         if date is None:
             date = self._date
-        distros = [x for x in self._avail(date) if x.release is None or
-                   (date < x.release and (x.eol is None or date <= x.eol))]
+        distros = [
+            x
+            for x in self._avail(date)
+            if x.release is None or (date < x.release and (x.eol is None or date <= x.eol))
+        ]
         if len(distros) < 2:
             raise DistroDataOutdated()
         return self._format(result, distros[-2])
@@ -212,51 +246,85 @@ class DebianDistroInfo(DistroInfo):
         """Get old (stable) Debian distribution based on the given date."""
         if date is None:
             date = self._date
-        distros = [x for x in self._avail(date)
-                   if x.release is not None and date >= x.release]
+        distros = [x for x in self._avail(date) if x.release is not None and date >= x.release]
         if len(distros) < 2:
             raise DistroDataOutdated()
         return self._format(result, distros[-2])
 
     def supported(self, date=None, result="codename"):
         """Get list of all supported Debian distributions based on the given
-           date."""
+        date."""
         if date is None:
             date = self._date
-        distros = [self._format(result, x) for x in self._avail(date)
-                   if x.eol is None or date <= x.eol]
+        distros = [
+            self._format(result, x) for x in self._avail(date) if x.eol is None or date <= x.eol
+        ]
+        return distros
+
+    def lts_supported(self, date=None, result="codename"):
+        """Get list of all LTS supported Debian distributions based on the given
+        date."""
+        if date is None:
+            date = self._date
+        distros = [
+            self._format(result, x)
+            for x in self._avail(date)
+            if (x.eol is not None and date > x.eol)
+            and (x.eol_lts is not None and date <= x.eol_lts)
+        ]
+        return distros
+
+    def elts_supported(self, date=None, result="codename"):
+        """Get list of all Extended LTS supported Debian distributions based on
+        the given date."""
+        if date is None:
+            date = self._date
+        distros = [
+            self._format(result, x)
+            for x in self._avail(date)
+            if (x.eol_lts is not None and date > x.eol_lts)
+            and (x.eol_elts is not None and date <= x.eol_elts)
+        ]
         return distros
 
     def testing(self, date=None, result="codename"):
         """Get latest testing Debian distribution based on the given date."""
         if date is None:
             date = self._date
-        distros = [x for x in self._avail(date) if (x.release is None and x.version) or
-                   (x.release is not None and date < x.release and
-                    (x.eol is None or date <= x.eol))]
+        distros = [
+            x
+            for x in self._avail(date)
+            if (x.release is None and x.version)
+            or (x.release is not None and date < x.release and (x.eol is None or date <= x.eol))
+        ]
         if not distros:
             raise DistroDataOutdated()
         return self._format(result, distros[-1])
 
     def valid(self, codename):
         """Check if the given codename is known."""
-        return (DistroInfo.valid(self, codename) or
-                codename in ["unstable", "testing", "stable", "oldstable"])
+        return DistroInfo.valid(self, codename) or codename in [
+            "unstable",
+            "testing",
+            "stable",
+            "oldstable",
+        ]
 
 
 class UbuntuDistroInfo(DistroInfo):
     """provides information about Ubuntu's distributions"""
 
     def __init__(self):
-        super(UbuntuDistroInfo, self).__init__("Ubuntu")
+        super().__init__("Ubuntu")
 
     def lts(self, date=None, result="codename"):
         """Get latest long term support (LTS) Ubuntu distribution based on the
-           given date."""
+        given date."""
         if date is None:
             date = self._date
-        distros = [x for x in self._releases if x.version.find("LTS") >= 0 and
-                   x.release <= date <= x.eol]
+        distros = [
+            x for x in self._releases if x.version.find("LTS") >= 0 and x.release <= date <= x.eol
+        ]
         if not distros:
             raise DistroDataOutdated()
         return self._format(result, distros[-1])
@@ -270,19 +338,24 @@ class UbuntuDistroInfo(DistroInfo):
 
     def supported(self, date=None, result="codename"):
         """Get list of all supported Ubuntu distributions based on the given
-           date."""
+        date."""
         if date is None:
             date = self._date
-        distros = [self._format(result, x) for x in self._avail(date)
-                   if date <= x.eol or
-                   (x.eol_server is not None and date <= x.eol_server)]
+        distros = [
+            self._format(result, x)
+            for x in self._avail(date)
+            if date <= x.eol or (x.eol_server is not None and date <= x.eol_server)
+        ]
         return distros
 
     def supported_esm(self, date=None, result="codename"):
         """Get list of all ESM supported Ubuntu distributions based on the
-           given date."""
+        given date."""
         if date is None:
             date = self._date
-        distros = [self._format(result, x) for x in self._avail(date)
-                   if x.eol_esm is not None and date <= x.eol_esm]
+        distros = [
+            self._format(result, x)
+            for x in self._avail(date)
+            if x.eol_esm is not None and date <= x.eol_esm
+        ]
         return distros
