@@ -1,5 +1,6 @@
 import os
 import platform
+import pathlib
 
 
 def _settings_root_XP():
@@ -18,10 +19,10 @@ def _data_root_Windows():
 
 def _data_root_Linux():
     """
-    Use freedesktop.org Base Dir Specfication to determine storage
+    Use freedesktop.org Base Dir Specification to determine storage
     location.
     """
-    fallback = os.path.expanduser('~/.local/share')
+    fallback = pathlib.Path.home() / '.local/share'
     root = os.environ.get('XDG_DATA_HOME', None) or fallback
     return os.path.join(root, 'python_keyring')
 
@@ -52,11 +53,11 @@ def _check_old_config_root():
 
 def _config_root_Linux():
     """
-    Use freedesktop.org Base Dir Specfication to determine config
+    Use freedesktop.org Base Dir Specification to determine config
     location.
     """
     _check_old_config_root()
-    fallback = os.path.expanduser('~/.config')
+    fallback = pathlib.Path.home() / '.config'
     key = 'XDG_CONFIG_HOME'
     root = os.environ.get(key, None) or fallback
     return os.path.join(root, 'python_keyring')
