@@ -353,6 +353,10 @@ class URI:
         else:
             return not equal
 
+    def __hash__(self):
+        return hash((
+            self.scheme, self.authority, self.path, self.query, self.fragment))
+
     def replace(self, **parts):
         """Replace one or more parts of the URI, returning the result."""
         if not parts:
