@@ -11,7 +11,6 @@
 # option) any later version.  See http://www.gnu.org/copyleft/gpl.html for
 # the full text of the license.
 
-import locale
 import zlib, base64, time, sys, gzip, struct, os
 from email.encoders import encode_base64
 from email.mime.multipart import MIMEMultipart
@@ -270,24 +269,6 @@ class ProblemReport(UserDict):
             raise ValueError('%s has no binary content' %
                              [item for item, element in b64_block.items() if element is False])
 
-    def get_timestamp(self) -> int:
-        '''Get timestamp (seconds since epoch) from Date field
-
-        Return None if it is not present.
-        '''
-        # report time is from asctime(), not in locale representation
-        orig_ctime = locale.getlocale(locale.LC_TIME)
-        try:
-            try:
-                locale.setlocale(locale.LC_TIME, 'C')
-                return int(time.mktime(time.strptime(self['Date'])))
-            except KeyError:
-                return None
-            finally:
-                locale.setlocale(locale.LC_TIME, orig_ctime)
-        except locale.Error:
-            return None
-
     def has_removed_fields(self):
         '''Check if the report has any keys which were not loaded.
 
