@@ -12,17 +12,13 @@ import hashlib
 import json
 import logging
 import os
+import re
 import shutil
 import tempfile
-import fnmatch
 
 from concurrent.futures import ThreadPoolExecutor
 from datetime import datetime
 from pwd import getpwuid
-from textwrap import fill
-
-import sos.cleaner.preppers
-
 from sos import __version__
 from sos.component import SoSComponent
 from sos.cleaner.parsers.ip_parser import SoSIPParser
@@ -30,13 +26,13 @@ from sos.cleaner.parsers.mac_parser import SoSMacParser
 from sos.cleaner.parsers.hostname_parser import SoSHostnameParser
 from sos.cleaner.parsers.keyword_parser import SoSKeywordParser
 from sos.cleaner.parsers.username_parser import SoSUsernameParser
-from sos.cleaner.parsers.ipv6_parser import SoSIPv6Parser
 from sos.cleaner.archives.sos import (SoSReportArchive, SoSReportDirectory,
                                       SoSCollectorArchive,
                                       SoSCollectorDirectory)
 from sos.cleaner.archives.generic import DataDirArchive, TarballArchive
 from sos.cleaner.archives.insights import InsightsArchive
-from sos.utilities import get_human_readable, import_module, ImporterHelper
+from sos.utilities import get_human_readable
+from textwrap import fill
 
 
 class SoSCleaner(SoSComponent):
@@ -59,14 +55,11 @@ class SoSCleaner(SoSComponent):
     that future iterations will maintain the same consistent obfuscation
     pairing.
 
-    In the case of IP addresses, support is for IPv4 and IPv6 - effort is made
-    to keep network topology intact so that later analysis is as accurate and
+    In the case of IP addresses, support is for IPv4 and efforts are made to
+    keep network topology intact so that later analysis is as accurate and
     easily understandable as possible. If an IP address is encountered that we
     cannot determine the netmask for, a random IP address is used instead.
 
-    For IPv6, note that IPv4-mapped addresses, e.g. ::ffff:10.11.12.13, are
-    NOT supported currently, and will remain unobfuscated.
-
     For hostnames, domains are obfuscated as whole units, leaving the TLD in
     place.
 
@@ -83,8 +76,6 @@ class SoSCleaner(SoSComponent):
     arg_defaults = {
         'archive_type': 'auto',
         'domains': [],
-        'disable_parsers': [],
-        'skip_cleaning_files': [],
         'jobs': 4,
         'keywords': [],
         'keyword_file': None,
@@ -99,7 +90,7 @@ class SoSCleaner(SoSComponent):
                  hook_commons=None):
         if not in_place:
             # we are running `sos clean` directly
-            super().__init__(parser, args, cmdline)
+            super(SoSCleaner, self).__init__(parser, args, cmdline)
             self.from_cmdline = True
         else:
             # we are being hooked by either SoSReport or SoSCollector, don't
@@ -120,8 +111,6 @@ class SoSCleaner(SoSComponent):
             # when obfuscating a SoSCollector run during archive extraction
             os.makedirs(os.path.join(self.tmpdir, 'cleaner'), exist_ok=True)
 
-        self.review_parser_values()
-
         self.cleaner_mapping = self.load_map_file()
         os.umask(0o77)
         self.in_place = in_place
@@ -129,27 +118,15 @@ class SoSCleaner(SoSComponent):
 
         self.cleaner_md = self.manifest.components.add_section('cleaner')
 
-        skip_cleaning_files = self.opts.skip_cleaning_files
         self.parsers = [
-            SoSHostnameParser(self.cleaner_mapping, skip_cleaning_files),
-            SoSIPParser(self.cleaner_mapping, skip_cleaning_files),
-            SoSIPv6Parser(self.cleaner_mapping, skip_cleaning_files),
-            SoSMacParser(self.cleaner_mapping, skip_cleaning_files),
-            SoSKeywordParser(self.cleaner_mapping, skip_cleaning_files),
-            SoSUsernameParser(self.cleaner_mapping, skip_cleaning_files)
+            SoSHostnameParser(self.cleaner_mapping, self.opts.domains),
+            SoSIPParser(self.cleaner_mapping),
+            SoSMacParser(self.cleaner_mapping),
+            SoSKeywordParser(self.cleaner_mapping, self.opts.keywords,
+                             self.opts.keyword_file),
+            SoSUsernameParser(self.cleaner_mapping, self.opts.usernames)
         ]
 
-        for _parser in self.opts.disable_parsers:
-            for _loaded in self.parsers:
-                _loaded_name = _loaded.name.lower().split('parser')[0].strip()
-                if _parser.lower().strip() == _loaded_name:
-                    self.log_info(f"Disabling parser: {_loaded_name}")
-                    self.ui_log.warning(
-                        f"Disabling the '{_parser}' parser. Be aware that this"
-                        " may leave sensitive plain-text data in the archive."
-                    )
-                    self.parsers.remove(_loaded)
-
         self.archive_types = [
             SoSReportDirectory,
             SoSReportArchive,
@@ -162,11 +139,11 @@ class SoSCleaner(SoSComponent):
         ]
         self.nested_archive = None
 
-        self.log_info(
-            f"Cleaner initialized. From cmdline: {self.from_cmdline}")
+        self.log_info("Cleaner initialized. From cmdline: %s"
+                      % self.from_cmdline)
 
     def _fmt_log_msg(self, msg, caller=None):
-        return f"[cleaner{f':{caller}' if caller else ''}] {msg}"
+        return "[cleaner%s] %s" % (":%s" % caller if caller else '', msg)
 
     def log_debug(self, msg, caller=None):
         self.soslog.debug(self._fmt_log_msg(msg, caller))
@@ -198,13 +175,13 @@ class SoSCleaner(SoSComponent):
         _conf = {}
         default_map = '/etc/sos/cleaner/default_mapping'
         if os.path.isdir(self.opts.map_file):
-            raise Exception(f"Requested map file {self.opts.map_file} is a "
-                            "directory")
+            raise Exception("Requested map file %s is a directory"
+                            % self.opts.map_file)
         if not os.path.exists(self.opts.map_file):
             if self.opts.map_file != default_map:
                 self.log_error(
-                    f"ERROR: map file {self.opts.map_file} does not exist, "
-                    "will not load any obfuscation matches")
+                    "ERROR: map file %s does not exist, will not load any "
+                    "obfuscation matches" % self.opts.map_file)
         else:
             with open(self.opts.map_file, 'r') as mf:
                 try:
@@ -213,8 +190,8 @@ class SoSCleaner(SoSComponent):
                     self.log_error("ERROR: Unable to parse map file, json is "
                                    "malformed. Will not load any mappings.")
                 except Exception as err:
-                    self.log_error("ERROR: Could not load "
-                                   f"'{self.opts.map_file}': {err}")
+                    self.log_error("ERROR: Could not load '%s': %s"
+                                   % (self.opts.map_file, err))
         return _conf
 
     def print_disclaimer(self):
@@ -235,7 +212,7 @@ Users should review any resulting data and/or archives generated or processed \
 by this utility for remaining sensitive content before being passed to a \
 third party.
 """)
-        self.ui_log.info(f"\nsos clean (version {__version__})\n")
+        self.ui_log.info("\nsos clean (version %s)\n" % __version__)
         self.ui_log.info(msg)
         if not self.opts.batch:
             try:
@@ -262,15 +239,6 @@ third party.
                                      'was generated as'))
         clean_grp.add_argument('--domains', action='extend', default=[],
                                help='List of domain names to obfuscate')
-        clean_grp.add_argument('--disable-parsers', action='extend',
-                               default=[], dest='disable_parsers',
-                               help=('Disable specific parsers, so that those '
-                                     'elements are not obfuscated'))
-        clean_grp.add_argument('--skip-cleaning-files', '--skip-masking-files',
-                               action='extend', default=[],
-                               dest='skip_cleaning_files',
-                               help=('List of files to skip/ignore during '
-                                     'cleaning. Globs are supported.'))
         clean_grp.add_argument('-j', '--jobs', default=4, type=int,
                                help='Number of concurrent archives to clean')
         clean_grp.add_argument('--keywords', action='extend', default=[],
@@ -332,21 +300,6 @@ third party.
         if self.nested_archive:
             self.nested_archive.ui_name = self.nested_archive.description
 
-    def review_parser_values(self):
-        """Check any values passed to the parsers via the commandline:
-        - For the --domains option, ensure that they are valid for the parser
-          in question.
-        - Convert --skip-cleaning-files from globs to regular expressions.
-        """
-        for _dom in self.opts.domains:
-            if len(_dom.split('.')) < 2:
-                raise Exception(
-                    f"Invalid value '{_dom}' given: --domains values must be "
-                    "actual domains"
-                )
-        self.opts.skip_cleaning_files = [fnmatch.translate(p) for p in
-                                         self.opts.skip_cleaning_files]
-
     def execute(self):
         """SoSCleaner will begin by inspecting the TARGET option to determine
         if it is a directory, archive, or archive of archives.
@@ -363,8 +316,8 @@ third party.
             self.print_disclaimer()
         self.report_paths = []
         if not os.path.exists(self.opts.target):
-            self.ui_log.error("Invalid target: no such file or directory "
-                              f"{self.opts.target}")
+            self.ui_log.error("Invalid target: no such file or directory %s"
+                              % self.opts.target)
             self._exit(1)
 
         self.inspect_target_archive()
@@ -375,11 +328,6 @@ third party.
 
         # we have at least one valid target to obfuscate
         self.completed_reports = []
-        # TODO: as we separate mappings and parsers further, do this in a less
-        # janky manner
-        for parser in self.parsers:
-            if parser.name == 'Hostname Parser':
-                parser.mapping.set_initial_counts()
         self.preload_all_archives_into_maps()
         self.generate_parser_item_regexes()
         self.obfuscate_report_paths()
@@ -390,8 +338,8 @@ third party.
             self.ui_log.info("No reports obfuscated, aborting...\n")
             self._exit(1)
 
-        self.ui_log.info("\nSuccessfully obfuscated "
-                         f"{len(self.completed_reports)} report(s)\n")
+        self.ui_log.info("\nSuccessfully obfuscated %s report(s)\n"
+                         % len(self.completed_reports))
 
         _map = self.compile_mapping_dict()
         map_path = self.write_map_for_archive(_map)
@@ -411,36 +359,30 @@ third party.
             checksum = self.get_new_checksum(arc.final_archive_path)
             if checksum is not None:
                 chksum_name = self.obfuscate_string(
-                    f"{arc_path.split('/')[-1]}.{self.hash_name}"
+                    "%s.%s" % (arc_path.split('/')[-1], self.hash_name)
                 )
                 with open(os.path.join(self.sys_tmp, chksum_name), 'w') as cf:
                     cf.write(checksum)
             self.write_cleaner_log()
 
-        final_path = os.path.join(
-            self.sys_tmp,
-            self.obfuscate_string(arc_path.split('/')[-1])
+        final_path = self.obfuscate_string(
+            os.path.join(self.sys_tmp, arc_path.split('/')[-1])
         )
         shutil.move(arc_path, final_path)
         arcstat = os.stat(final_path)
 
-        # while these messages won't be included in the log file in the archive
-        # some facilities, such as our avocado test suite, will sometimes not
-        # capture print() output, so leverage the ui_log to print to console
-        self.ui_log.info(
-            f"A mapping of obfuscated elements is available at\n\t{map_path}"
-        )
-        self.ui_log.info(
-            f"\nThe obfuscated archive is available at\n\t{final_path}\n"
-        )
+        # logging will have been shutdown at this point
+        print("A mapping of obfuscated elements is available at\n\t%s"
+              % map_path)
 
-        self.ui_log.info(f"\tSize\t{get_human_readable(arcstat.st_size)}")
-        self.ui_log.info(f"\tOwner\t{getpwuid(arcstat.st_uid).pw_name}\n")
-        self.ui_log.info("Please send the obfuscated archive to your support "
-                         "representative and keep the mapping file private")
+        print("\nThe obfuscated archive is available at\n\t%s\n" % final_path)
+        print("\tSize\t%s" % get_human_readable(arcstat.st_size))
+        print("\tOwner\t%s\n" % getpwuid(arcstat.st_uid).pw_name)
+
+        print("Please send the obfuscated archive to your support "
+              "representative and keep the mapping file private")
 
         self.cleanup()
-        return None
 
     def rebuild_nested_archive(self):
         """Handles repacking the nested tarball, now containing only obfuscated
@@ -453,7 +395,7 @@ third party.
             arc_dest = archive.final_archive_path.split('/')[-1]
             checksum = self.get_new_checksum(archive.final_archive_path)
             if checksum is not None:
-                dname = f"checksums/{arc_dest}.{self.hash_name}"
+                dname = "checksums/%s.%s" % (arc_dest, self.hash_name)
                 self.archive.add_string(checksum, dest=dname)
         for dirn, dirs, files in os.walk(self.nested_archive.extracted_path):
             for filename in files:
@@ -475,7 +417,7 @@ third party.
         _map = {}
         for parser in self.parsers:
             _map[parser.map_file_key] = {}
-            _map[parser.map_file_key].update(parser.get_map_contents())
+            _map[parser.map_file_key].update(parser.mapping.dataset)
 
         return _map
 
@@ -489,13 +431,12 @@ third party.
 
     def write_map_for_archive(self, _map):
         try:
-            map_path = os.path.join(
-                self.sys_tmp,
-                self.obfuscate_string(f"{self.arc_name}-private_map")
+            map_path = self.obfuscate_string(
+                os.path.join(self.sys_tmp, "%s-private_map" % self.arc_name)
             )
             return self.write_map_to_file(_map, map_path)
         except Exception as err:
-            self.log_error(f"Could not write private map file: {err}")
+            self.log_error("Could not write private map file: %s" % err)
             return None
 
     def write_map_for_config(self, _map):
@@ -510,9 +451,10 @@ third party.
             try:
                 os.makedirs(cleaner_dir, exist_ok=True)
                 self.write_map_to_file(_map, self.opts.map_file)
-                self.log_debug(f"Wrote mapping to {self.opts.map_file}")
+                self.log_debug("Wrote mapping to %s" % self.opts.map_file)
             except Exception as err:
-                self.log_error(f"Could not update mapping config file: {err}")
+                self.log_error("Could not update mapping config file: %s"
+                               % err)
 
     def write_cleaner_log(self, archive=False):
         """When invoked via the command line, the logging from SoSCleaner will
@@ -520,7 +462,7 @@ third party.
         separately to disk
         """
         log_name = os.path.join(
-            self.sys_tmp, f"{self.arc_name}-obfuscation.log"
+            self.sys_tmp, "%s-obfuscation.log" % self.arc_name
         )
         with open(log_name, 'w') as logfile:
             self.sos_log_file.seek(0)
@@ -537,16 +479,17 @@ third party.
         """
         try:
             hash_size = 1024**2  # Hash 1MiB of content at a time.
-            with open(archive_path, 'rb') as archive_fp:
-                digest = hashlib.new(self.hash_name)
-                while True:
-                    hashdata = archive_fp.read(hash_size)
-                    if not hashdata:
-                        break
-                    digest.update(hashdata)
-                return digest.hexdigest() + '\n'
+            archive_fp = open(archive_path, 'rb')
+            digest = hashlib.new(self.hash_name)
+            while True:
+                hashdata = archive_fp.read(hash_size)
+                if not hashdata:
+                    break
+                digest.update(hashdata)
+            archive_fp.close()
+            return digest.hexdigest() + '\n'
         except Exception as err:
-            self.log_debug(f"Could not generate new checksum: {err}")
+            self.log_debug("Could not generate new checksum: %s" % err)
         return None
 
     def obfuscate_report_paths(self):
@@ -558,8 +501,8 @@ third party.
         """
         try:
             msg = (
-                f"Found {len(self.report_paths)} total reports to obfuscate, "
-                f"processing up to {self.opts.jobs} concurrently\n"
+                "Found %s total reports to obfuscate, processing up to %s "
+                "concurrently\n" % (len(self.report_paths), self.opts.jobs)
             )
             self.ui_log.info(msg)
             if self.opts.keep_binary_files:
@@ -599,63 +542,6 @@ third party.
         for parser in self.parsers:
             parser.generate_item_regexes()
 
-    def _prepare_archive_with_prepper(self, archive, prepper):
-        """
-        For each archive we've determined we need to operate on, pass it to
-        each prepper so that we can extract necessary files and/or items for
-        direct regex replacement. Preppers define these methods per parser,
-        so it is possible that a single prepper will read the same file for
-        different parsers/mappings. This is preferable to the alternative of
-        building up monolithic lists of file paths, as we'd still need to
-        manipulate these on a per-archive basis.
-
-        :param archive: The archive we are currently using to prepare our
-                        mappings with
-        :type archive:  ``SoSObfuscationArchive`` subclass
-
-        :param prepper: The individual prepper we're using to source items
-        :type prepper:  ``SoSPrepper`` subclass
-        """
-        for _parser in self.parsers:
-            pname = _parser.name.lower().split()[0].strip()
-            for _file in prepper.get_parser_file_list(pname, archive):
-                content = archive.get_file_content(_file)
-                if not content:
-                    continue
-                self.log_debug(f"Prepping {pname} parser with file {_file} "
-                               f"from {archive.ui_name}")
-                for line in content.splitlines():
-                    try:
-                        _parser.parse_line(line)
-                    except Exception as err:
-                        self.log_debug(
-                            f"Failed to prep {pname} map from {_file}: {err}"
-                        )
-            map_items = prepper.get_items_for_map(pname, archive)
-            if map_items:
-                self.log_debug(f"Prepping {pname} mapping with items from "
-                               f"{archive.ui_name}")
-                for item in map_items:
-                    _parser.mapping.add(item)
-
-            for ritem in prepper.regex_items[pname]:
-                _parser.mapping.add_regex_item(ritem)
-
-    def get_preppers(self):
-        """
-        Discover all locally available preppers so that we can prepare the
-        mappings with obfuscation matches in a controlled manner
-
-        :returns: All preppers that can be leveraged locally
-        :rtype:   A generator of `SoSPrepper` items
-        """
-        helper = ImporterHelper(sos.cleaner.preppers)
-        preps = []
-        for _prep in helper.get_modules():
-            preps.extend(import_module(f"sos.cleaner.preppers.{_prep}"))
-        for prepper in sorted(preps, key=lambda x: x.priority):
-            yield prepper(options=self.opts)
-
     def preload_all_archives_into_maps(self):
         """Before doing the actual obfuscation, if we have multiple archives
         to obfuscate then we need to preload each of them into the mappings
@@ -663,11 +549,44 @@ third party.
         obfuscated in node1's archive.
         """
         self.log_info("Pre-loading all archives into obfuscation maps")
-        for prepper in self.get_preppers():
-            for archive in self.report_paths:
-                self._prepare_archive_with_prepper(archive, prepper)
+        for _arc in self.report_paths:
+            for _parser in self.parsers:
+                try:
+                    pfile = _arc.prep_files[_parser.name.lower().split()[0]]
+                    if not pfile:
+                        continue
+                except (IndexError, KeyError):
+                    continue
+                if isinstance(pfile, str):
+                    pfile = [pfile]
+                for parse_file in pfile:
+                    self.log_debug("Attempting to load %s" % parse_file)
+                    try:
+                        content = _arc.get_file_content(parse_file)
+                        if not content:
+                            continue
+                        if isinstance(_parser, SoSUsernameParser):
+                            _parser.load_usernames_into_map(content)
+                        elif isinstance(_parser, SoSHostnameParser):
+                            if 'hostname' in parse_file:
+                                _parser.load_hostname_into_map(
+                                    content.splitlines()[0]
+                                )
+                            elif 'etc/hosts' in parse_file:
+                                _parser.load_hostname_from_etc_hosts(
+                                    content
+                                )
+                        else:
+                            for line in content.splitlines():
+                                self.obfuscate_line(line)
+                    except Exception as err:
+                        self.log_info(
+                            "Could not prepare %s from %s (archive: %s): %s"
+                            % (_parser.name, parse_file, _arc.archive_name,
+                               err)
+                        )
 
-    def obfuscate_report(self, archive):  # pylint: disable=too-many-branches
+    def obfuscate_report(self, archive):
         """Individually handle each archive or directory we've discovered by
         running through each file therein.
 
@@ -684,7 +603,8 @@ third party.
                 archive.extract()
             archive.report_msg("Beginning obfuscation...")
 
-            for fname in archive.get_file_list():
+            file_list = archive.get_file_list()
+            for fname in file_list:
                 short_name = fname.split(archive.archive_name + '/')[1]
                 if archive.should_skip_file(short_name):
                     continue
@@ -698,18 +618,12 @@ third party.
                     if count:
                         archive.update_sub_count(short_name, count)
                 except Exception as err:
-                    self.log_debug(f"Unable to parse file {short_name}: {err}")
-
+                    self.log_debug("Unable to parse file %s: %s"
+                                   % (short_name, err))
             try:
                 self.obfuscate_directory_names(archive)
             except Exception as err:
-                self.log_info(f"Failed to obfuscate directories: {err}",
-                              caller=archive.archive_name)
-
-            try:
-                self.obfuscate_symlinks(archive)
-            except Exception as err:
-                self.log_info(f"Failed to obfuscate symlinks: {err}",
+                self.log_info("Failed to obfuscate directories: %s" % err,
                               caller=archive.archive_name)
 
             # if the archive was already a tarball, repack it
@@ -723,10 +637,10 @@ third party.
                         )
                         archive.compress(method)
                     except Exception as err:
-                        self.log_debug(f"Archive {archive.archive_name} failed"
-                                       f" to compress: {err}")
-                        archive.report_msg(
-                            f"Failed to re-compress archive: {err}")
+                        self.log_debug("Archive %s failed to compress: %s"
+                                       % (archive.archive_name, err))
+                        archive.report_msg("Failed to re-compress archive: %s"
+                                           % err)
                         return
                 self.completed_reports.append(archive)
 
@@ -739,14 +653,13 @@ third party.
             if archive.removed_file_count:
                 rmsg = " [removed %s unprocessable files]"
                 rmsg = rmsg % archive.removed_file_count
-            archive.report_msg(f"Obfuscation completed{rmsg}")
+            archive.report_msg("Obfuscation completed%s" % rmsg)
 
         except Exception as err:
-            self.ui_log.info("Exception while processing "
-                             f"{archive.archive_name}: {err}")
+            self.ui_log.info("Exception while processing %s: %s"
+                             % (archive.archive_name, err))
 
     def obfuscate_file(self, filename, short_name=None, arc_name=None):
-        # pylint: disable=too-many-locals
         """Obfuscate and individual file, line by line.
 
         Lines processed, even if no substitutions occur, are then written to a
@@ -762,47 +675,47 @@ third party.
         """
         if not filename:
             # the requested file doesn't exist in the archive
-            return None
+            return
         subs = 0
         if not short_name:
             short_name = filename.split('/')[-1]
         if not os.path.islink(filename):
             # don't run the obfuscation on the link, but on the actual file
             # at some other point.
-            _parsers = [
-                _p for _p in self.parsers if not
-                any(
-                    _skip.match(short_name) for _skip in _p.skip_patterns
-                )
-            ]
-            if not _parsers:
-                self.log_debug(
-                    f"Skipping obfuscation of {short_name or filename} due to "
-                    f"matching file skip pattern"
-                )
-                return 0
-            self.log_debug(f"Obfuscating {short_name or filename}",
+            self.log_debug("Obfuscating %s" % short_name or filename,
                            caller=arc_name)
             tfile = tempfile.NamedTemporaryFile(mode='w', dir=self.tmpdir)
-            with open(filename, 'r', errors='replace') as fname:
+            _parsers = [
+                _p for _p in self.parsers if not
+                any([
+                    re.match(p, short_name) for p in _p.skip_files
+                ])
+            ]
+            with open(filename, 'r') as fname:
                 for line in fname:
                     try:
                         line, count = self.obfuscate_line(line, _parsers)
                         subs += count
                         tfile.write(line)
                     except Exception as err:
-                        self.log_debug(f"Unable to obfuscate {short_name}: "
-                                       f"{err}", caller=arc_name)
+                        self.log_debug("Unable to obfuscate %s: %s"
+                                       % (short_name, err), caller=arc_name)
             tfile.seek(0)
             if subs:
-                shutil.copyfile(tfile.name, filename)
+                shutil.copy(tfile.name, filename)
             tfile.close()
 
         _ob_short_name = self.obfuscate_string(short_name.split('/')[-1])
         _ob_filename = short_name.replace(short_name.split('/')[-1],
                                           _ob_short_name)
+        _sym_changed = False
+        if os.path.islink(filename):
+            _link = os.readlink(filename)
+            _ob_link = self.obfuscate_string(_link)
+            if _ob_link != _link:
+                _sym_changed = True
 
-        if _ob_filename != short_name:
+        if (_ob_filename != short_name) or _sym_changed:
             arc_path = filename.split(short_name)[0]
             _ob_path = os.path.join(arc_path, _ob_filename)
             # ensure that any plugin subdirs that contain obfuscated strings
@@ -822,58 +735,12 @@ third party.
 
         return subs
 
-    def obfuscate_symlinks(self, archive):
-        """Iterate over symlinks in the archive and obfuscate their names.
-        The content of the link target will have already been cleaned, and this
-        second pass over just the names of the links is to ensure we avoid a
-        possible race condition dependent on the order in which the link or the
-        target get obfuscated.
-
-        :param archive:     The archive being obfuscated
-        :type archive:      ``SoSObfuscationArchive``
-        """
-        self.log_info("Obfuscating symlink names", caller=archive.archive_name)
-        for symlink in archive.get_symlinks():
-            try:
-                # relative name of the symlink in the archive
-                _sym = symlink.split(archive.extracted_path)[1].lstrip('/')
-                # don't obfuscate symlinks for files that we skipped the first
-                # obfuscation of, as that would create broken links
-                _parsers = [
-                    _p for _p in self.parsers if not
-                    any(_skip.match(_sym) for _skip in _p.skip_patterns)
-                ]
-                if not _parsers:
-                    self.log_debug(
-                        f"Skipping obfuscation of symlink {_sym} due to skip "
-                        f"pattern match"
-                    )
-                    continue
-                self.log_debug(f"Obfuscating symlink {_sym}",
-                               caller=archive.archive_name)
-                # current target of symlink, again relative to the archive
-                _target = os.readlink(symlink)
-                # get the potentially renamed symlink name, this time the full
-                # path as it exists on disk
-                _ob_sym_name = os.path.join(archive.extracted_path,
-                                            self.obfuscate_string(_sym))
-                # get the potentially renamed relative target filename
-                _ob_target = self.obfuscate_string(_target)
-
-                # if either the symlink name or the target name has changed,
-                # recreate the symlink
-                if (_ob_sym_name != symlink) or (_ob_target != _target):
-                    os.remove(symlink)
-                    os.symlink(_ob_target, _ob_sym_name)
-            except Exception as err:
-                self.log_info(f"Error obfuscating symlink '{symlink}': {err}")
-
     def obfuscate_directory_names(self, archive):
         """For all directories that exist within the archive, obfuscate the
         directory name if it contains sensitive strings found during execution
         """
-        self.log_info("Obfuscating directory names in archive "
-                      f"{archive.archive_name}")
+        self.log_info("Obfuscating directory names in archive %s"
+                      % archive.archive_name)
         for dirpath in sorted(archive.get_directory_list(), reverse=True):
             for _name in os.listdir(dirpath):
                 _dirname = os.path.join(dirpath, _name)
@@ -893,8 +760,8 @@ third party.
         for parser in self.parsers:
             try:
                 string_data = parser.parse_string_for_keys(string_data)
-            except Exception as err:
-                self.log_info(f"Error obfuscating string data: {err}")
+            except Exception:
+                pass
         return string_data
 
     def obfuscate_line(self, line, parsers=None):
@@ -922,7 +789,7 @@ third party.
                 line, _count = parser.parse_line(line)
                 count += _count
             except Exception as err:
-                self.log_debug(f"failed to parse line: {err}", parser.name)
+                self.log_debug("failed to parse line: %s" % err, parser.name)
         return line, count
 
     def write_stats_to_manifest(self):
