@@ -21,10 +21,11 @@ class SoSKeywordMap(SoSMap):
     is an incrementing integer.
     """
 
+    match_full_words_only = True
     word_count = 0
 
     def sanitize_item(self, item):
-        _ob_item = "obfuscatedword%s" % self.word_count
+        _ob_item = f"obfuscatedword{self.word_count}"
         self.word_count += 1
         if _ob_item in self.dataset.values():
             return self.sanitize_item(item)
