@@ -52,11 +52,11 @@ class SoSMacMap(SoSMap):
 
     def add(self, item):
         item = item.replace('-', ':').lower().strip('=.,').strip()
-        return super().add(item)
+        return super(SoSMacMap, self).add(item)
 
     def get(self, item):
         item = item.replace('-', ':').lower().strip('=.,').strip()
-        return super().get(item)
+        return super(SoSMacMap, self).get(item)
 
     def sanitize_item(self, item):
         """Randomize the device hextets, and append those to our 'vendor'
@@ -77,4 +77,3 @@ class SoSMacMap(SoSMap):
         # match 48-bit IPv4 MAC addresses
         if re.match('([0-9a-fA-F][:_]?){12}', item):
             return self.mac_template % hextets
-        return None
