@@ -10,7 +10,7 @@
 
 import os
 
-from pipes import quote
+from shlex import quote
 from sos.collector.clusters import Cluster
 from sos.utilities import is_executable
 
@@ -69,7 +69,8 @@ class ocp(Cluster):
         ('role', 'master', 'Colon delimited list of roles to filter on'),
         ('kubeconfig', '', 'Path to the kubeconfig file'),
         ('token', '', 'Service account token to use for oc authorization'),
-        ('with-api', False, 'Collect OCP API data from a master node')
+        ('with-api', False, 'Collect OCP API data from a master node'),
+        ('api-url', '', 'Alternate API URL of an external control-plane'),
     ]
 
     @property
@@ -90,31 +91,33 @@ class ocp(Cluster):
                         "Unable to to determine PATH for 'oc' command, "
                         "node enumeration may fail."
                     )
-                    self.log_debug("Locating 'oc' failed: %s"
-                                   % _oc_path['output'])
+                    self.log_debug(
+                        f"Locating 'oc' failed: {_oc_path['output']}")
             if self.get_option('kubeconfig'):
-                self._oc_cmd += " --config %s" % self.get_option('kubeconfig')
-            self.log_debug("oc base command set to %s" % self._oc_cmd)
+                self._oc_cmd += " --kubeconfig " \
+                        f"{self.get_option('kubeconfig')}"
+            self.log_debug(f"oc base command set to {self._oc_cmd}")
         return self._oc_cmd
 
     def fmt_oc_cmd(self, cmd):
         """Format the oc command to optionall include the kubeconfig file if
         one is specified
         """
-        return "%s %s" % (self.oc_cmd, cmd)
+        return f"{self.oc_cmd} {cmd}"
 
     def _attempt_oc_login(self):
         """Attempt to login to the API using the oc command using a provided
         token
         """
         _res = self.exec_primary_cmd(
-            self.fmt_oc_cmd("login --insecure-skip-tls-verify=True --token=%s"
-                            % self.token)
+            self.fmt_oc_cmd("login --insecure-skip-tls-verify=True "
+                            f"--token={self.token} "
+                            f"{self.get_option('api-url')}")
         )
         return _res['status'] == 0
 
     def check_enabled(self):
-        if super(ocp, self).check_enabled():
+        if super().check_enabled():
             return True
         self.token = self.get_option('token') or os.getenv('SOSOCPTOKEN', None)
         if self.token:
@@ -127,7 +130,7 @@ class ocp(Cluster):
         collection via a container image
         """
         if not self.set_transport_type() == 'oc':
-            return
+            return None
 
         out = self.exec_primary_cmd(self.fmt_oc_cmd("auth can-i '*' '*'"))
         self.oc_cluster_admin = out['status'] == 0
@@ -137,34 +140,67 @@ class ocp(Cluster):
             raise Exception("Insufficient permissions to create temporary "
                             "collection project.\nAborting...")
 
-        self.log_info("Creating new temporary project '%s'" % self.project)
+        self.log_info(f"Creating new temporary project '{self.project}'")
         ret = self.exec_primary_cmd(
-            self.fmt_oc_cmd("new-project %s" % self.project)
+            self.fmt_oc_cmd(f"new-project {self.project}")
         )
         if ret['status'] == 0:
+            self._label_sos_project()
             return True
 
-        self.log_debug("Failed to create project: %s" % ret['output'])
+        self.log_debug(f"Failed to create project: {ret['output']}")
         raise Exception("Failed to create temporary project for collection. "
                         "\nAborting...")
 
+    def _label_sos_project(self):
+        """Add pertinent labels to the temporary project we've created so that
+        our privileged containers can properly run.
+        """
+        labels = [
+            "security.openshift.io/scc.podSecurityLabelSync=false",
+            "pod-security.kubernetes.io/enforce=privileged"
+        ]
+        for label in labels:
+            ret = self.exec_primary_cmd(
+                self.fmt_oc_cmd(
+                    f"label namespace {self.project} {label} --overwrite"
+                )
+            )
+            if not ret['status'] == 0:
+                raise Exception(
+                    f"Error applying namespace labels: {ret['output']}"
+                )
+
     def cleanup(self):
         """Remove the project we created to execute within
         """
         if self.project:
-            ret = self.exec_primary_cmd(
-                self.fmt_oc_cmd("delete project %s" % self.project)
-            )
-            if not ret['status'] == 0:
-                self.log_error("Error deleting temporary project: %s"
-                               % ret['output'])
-            ret = self.exec_primary_cmd(
-                self.fmt_oc_cmd("wait namespace/%s --for=delete --timeout=30s"
-                                % self.project)
-            )
-            if not ret['status'] == 0:
-                self.log_error("Error waiting for temporary project to be "
-                               "deleted: %s" % ret['output'])
+            try:
+                ret = self.exec_primary_cmd(
+                    self.fmt_oc_cmd(f"delete project {self.project}"),
+                    timeout=30
+                )
+                if not ret['status'] == 0:
+                    self.log_error(
+                        f"Error deleting temporary project: {ret['output']}"
+                    )
+                ret = self.exec_primary_cmd(
+                    self.fmt_oc_cmd(
+                        f"wait namespace/{self.project} --for=delete "
+                        f"--timeout=30s"
+                    )
+                )
+                if not ret['status'] == 0:
+                    self.log_error(
+                        f"Error waiting for temporary project to be deleted: "
+                        f"{ret['output']}"
+                    )
+            except Exception as err:
+                self.log_error(
+                    f"Failed attempting to remove temporary project "
+                    f"'sos-collect-tmp': {err}\n"
+                    f"Please manually remove the temporary project."
+                )
             # don't leave the config on a non-existing project
             self.exec_primary_cmd(self.fmt_oc_cmd("project default"))
             self.project = None
@@ -190,6 +226,8 @@ class ocp(Cluster):
                 try:
                     idx[state] = statline.index(state.upper())
                 except Exception:
+                    # label is not available, which is not fatal for our dict
+                    # construction here
                     pass
             for node in nodelist:
                 _node = node.split()
@@ -205,7 +243,7 @@ class ocp(Cluster):
             return 'oc'
         self.log_info("Local installation of 'oc' not found or is not "
                       "correctly configured. Will use ControlPersist.")
-        self.ui_log.warn(
+        self.ui_log.warning(
             "Preferred transport 'oc' not available, will fallback to SSH."
         )
         if not self.opts.batch:
@@ -219,7 +257,7 @@ class ocp(Cluster):
         cmd = 'get nodes -o wide'
         if self.get_option('label'):
             labels = ','.join(self.get_option('label').split(':'))
-            cmd += " -l %s" % quote(labels)
+            cmd += f" -l {quote(labels)}"
         res = self.exec_primary_cmd(self.fmt_oc_cmd(cmd))
         if res['status'] == 0:
             if self.get_option('role') == 'master':
@@ -231,8 +269,9 @@ class ocp(Cluster):
             for node_name, node in self.node_dict.items():
                 if roles:
                     for role in roles:
-                        if role == node['roles']:
+                        if role in node['roles']:
                             nodes.append(node_name)
+                            break
                 else:
                     nodes.append(node_name)
         else:
@@ -279,7 +318,7 @@ class ocp(Cluster):
         else:
             _opt = 'no-oc'
             _val = 'off' if use_api else 'on'
-        node.plugopts.append("openshift.%s=%s" % (_opt, _val))
+        node.plugopts.append(f"openshift.{_opt}={_val}")
 
     def set_primary_options(self, node):
 
@@ -312,7 +351,7 @@ class ocp(Cluster):
                 # cannot do remotely
                 if node.file_exists('/root/.kube/config', need_root=True):
                     _oc_cmd += ' --kubeconfig /host/root/.kube/config'
-            can_oc = node.run_command("%s whoami" % _oc_cmd,
+            can_oc = node.run_command(f"{_oc_cmd} whoami",
                                       use_container=node.host.containerized,
                                       # container is available only to root
                                       # and if rhel, need to run sos as root
@@ -331,14 +370,14 @@ class ocp(Cluster):
                 # if the with-api option is turned on
                 if not _kubeconfig == master_kube:
                     node.plugopts.append(
-                        "openshift.kubeconfig=%s" % _kubeconfig
+                        f"openshift.kubeconfig={_kubeconfig}"
                     )
                 self._toggle_api_opt(node, True)
                 self.api_collect_enabled = True
             if self.api_collect_enabled:
-                msg = ("API collections will be performed on %s\nNote: API "
-                       "collections may extend runtime by 10s of minutes\n"
-                       % node.address)
+                msg = (f"API collections will be performed on {node.address}\n"
+                       "Note: API collections may extend runtime by 10s of "
+                       "minutes\n")
                 self.soslog.info(msg)
                 self.ui_log.info(msg)
 
