@@ -8,8 +8,6 @@
 #
 # See the LICENSE file in the source distribution for further information.
 
-# pylint: disable=too-many-branches
-
 import os
 import re
 
@@ -22,7 +20,6 @@ from sos.policies.init_systems.systemd import SystemdInit
 from sos.policies.runtimes.crio import CrioContainerRuntime
 from sos.policies.runtimes.podman import PodmanContainerRuntime
 from sos.policies.runtimes.docker import DockerContainerRuntime
-from sos.policies.runtimes.lxd import LxdContainerRuntime
 
 from sos.utilities import (shell_out, is_executable, bold,
                            sos_get_command_output)
@@ -34,12 +31,6 @@ try:
 except ImportError:
     REQUESTS_LOADED = False
 
-try:
-    import boto3
-    BOTO3_LOADED = True
-except ImportError:
-    BOTO3_LOADED = False
-
 # Container environment variables for detecting if we're in a container
 ENV_CONTAINER = 'container'
 ENV_HOST_SYSROOT = 'HOST'
@@ -64,23 +55,11 @@ class LinuxPolicy(Policy):
     _upload_user = None
     _upload_password = None
     _upload_method = None
-    _upload_s3_endpoint = 'https://s3.amazonaws.com'
-    _upload_s3_bucket = None
-    _upload_s3_access_key = None
-    _upload_s3_secret_key = None
-    _upload_s3_region = None
-    _upload_s3_object_prefix = ''
     default_container_runtime = 'docker'
     _preferred_hash_name = None
     upload_url = None
     upload_user = None
     upload_password = None
-    upload_s3_endpoint = None
-    upload_s3_bucket = None
-    upload_s3_access_key = None
-    upload_s3_secret_key = None
-    upload_s3_region = None
-    upload_s3_object_prefix = None
     # collector-focused class attrs
     containerized = False
     container_image = None
@@ -93,9 +72,9 @@ class LinuxPolicy(Policy):
 
     def __init__(self, sysroot=None, init=None, probe_runtime=True,
                  remote_exec=None):
-        super().__init__(sysroot=sysroot,
-                         probe_runtime=probe_runtime,
-                         remote_exec=remote_exec)
+        super(LinuxPolicy, self).__init__(sysroot=sysroot,
+                                          probe_runtime=probe_runtime,
+                                          remote_exec=remote_exec)
 
         if sysroot:
             self.sysroot = sysroot
@@ -116,8 +95,7 @@ class LinuxPolicy(Policy):
             _crun = [
                 PodmanContainerRuntime(policy=self),
                 DockerContainerRuntime(policy=self),
-                CrioContainerRuntime(policy=self),
-                LxdContainerRuntime(policy=self),
+                CrioContainerRuntime(policy=self)
             ]
             for runtime in _crun:
                 if runtime.check_is_active():
@@ -163,7 +141,7 @@ class LinuxPolicy(Policy):
         if cls == LinuxPolicy:
             cls.display_self_help(section)
         else:
-            section.set_title(f"{cls.distro} Distribution Policy")
+            section.set_title("%s Distribution Policy" % cls.distro)
             cls.display_distro_help(section)
 
     @classmethod
@@ -188,26 +166,30 @@ class LinuxPolicy(Policy):
         # information like $PATH and loaded presets
         _pol = cls(None, None, False)
         section.add_text(
-            f"Default --upload location: {_pol._upload_url}"
+            "Default --upload location: %s" % _pol._upload_url
         )
         section.add_text(
-            f"Default container runtime: {_pol.default_container_runtime}",
+            "Default container runtime: %s" % _pol.default_container_runtime,
             newline=False
         )
         section.add_text(
-            f"$PATH used when running report: {_pol.PATH}",
+            "$PATH used when running report: %s" % _pol.PATH,
             newline=False
         )
 
         refsec = section.add_section('Reference URLs')
         for url in cls.vendor_urls:
-            refsec.add_text(f"{' ':>8}{url[0]:<30}{url[1]:<40}", newline=False)
+            refsec.add_text(
+                "{:>8}{:<30}{:<40}".format(' ', url[0], url[1]),
+                newline=False
+            )
 
         presec = section.add_section('Presets Available With This Policy\n')
         presec.add_text(
             bold(
-                f"{' ':>8}{'Preset Name':<20}{'Description':<45}"
-                f"{'Enabled Options':<30}"
+                "{:>8}{:<20}{:<45}{:<30}".format(' ', 'Preset Name',
+                                                 'Description',
+                                                 'Enabled Options')
             ),
             newline=False
         )
@@ -215,7 +197,9 @@ class LinuxPolicy(Policy):
             _preset = _pol.presets[preset]
             _opts = ' '.join(_preset.opts.to_args())
             presec.add_text(
-                f"{' ':>8}{preset:<20}{_preset.desc:<45}{_opts:<30}",
+                "{:>8}{:<20}{:<45}{:<30}".format(
+                    ' ', preset, _preset.desc, _opts
+                ),
                 newline=False
             )
 
@@ -251,15 +235,15 @@ class LinuxPolicy(Policy):
 
         # next, include kernel builtins
         builtins = self.join_sysroot(
-            f"/usr/lib/modules/{release}/modules.builtin"
+            "/usr/lib/modules/%s/modules.builtin" % release
         )
         try:
             with open(builtins, "r") as mfile:
                 for line in mfile:
                     kmod = line.split('/')[-1].split('.ko')[0]
                     self.kernel_mods.append(kmod)
-        except IOError as err:
-            self.soslog.warning(f"Unable to read kernel builtins: {err}")
+        except IOError:
+            pass
 
         # finally, parse kconfig looking for specific kconfig strings that
         # have been verified to not appear in either lsmod or modules.builtin
@@ -269,15 +253,15 @@ class LinuxPolicy(Policy):
             'dm_mod': 'CONFIG_BLK_DEV_DM'
         }
 
-        booted_config = self.join_sysroot(f"/boot/config-{release}")
+        booted_config = self.join_sysroot("/boot/config-%s" % release)
         kconfigs = []
         try:
             with open(booted_config, "r") as kfile:
                 for line in kfile:
                     if '=y' in line:
                         kconfigs.append(line.split('=y')[0])
-        except IOError as err:
-            self.soslog.warning(f"Unable to read booted kernel config: {err}")
+        except IOError:
+            pass
 
         for builtin in config_strings:
             if config_strings[builtin] in kconfigs:
@@ -306,13 +290,6 @@ class LinuxPolicy(Policy):
         self.upload_password = cmdline_opts.upload_pass
         self.upload_archive_name = ''
 
-        self.upload_s3_endpoint = cmdline_opts.upload_s3_endpoint
-        self.upload_s3_region = cmdline_opts.upload_s3_region
-        self.upload_s3_access_key = cmdline_opts.upload_s3_access_key
-        self.upload_s3_bucket = cmdline_opts.upload_s3_bucket
-        self.upload_s3_object_prefix = cmdline_opts.upload_s3_object_prefix
-        self.upload_s3_secret_key = cmdline_opts.upload_s3_secret_key
-
         # set or query for case id
         if not cmdline_opts.batch and not \
                 cmdline_opts.quiet:
@@ -335,15 +312,9 @@ class LinuxPolicy(Policy):
                 cmdline_opts.quiet:
             try:
                 # Policies will need to handle the prompts for user information
-                if cmdline_opts.upload and self.get_upload_url() and \
-                        not cmdline_opts.upload_protocol == 's3':
+                if cmdline_opts.upload and self.get_upload_url():
                     self.prompt_for_upload_user()
                     self.prompt_for_upload_password()
-                elif cmdline_opts.upload_protocol == 's3':
-                    self.prompt_for_upload_s3_bucket()
-                    self.prompt_for_upload_s3_endpoint()
-                    self.prompt_for_upload_s3_access_key()
-                    self.prompt_for_upload_s3_secret_key()
                 self.ui_log.info('')
             except KeyboardInterrupt:
                 raise
@@ -381,64 +352,12 @@ class LinuxPolicy(Policy):
         except Exception as err:
             self.soslog.error(f"Error setting report niceness to 19: {err}")
 
-    def prompt_for_upload_s3_access_key(self):
-        """Should be overridden by policies to determine if an access key needs
-        to be provided for upload or not
-        """
-        if not self.get_upload_s3_access_key():
-
-            msg = (
-                "Please provide the upload access key for bucket"
-                f" {self.get_upload_s3_bucket()} via endpoint"
-                f" {self.get_upload_s3_endpoint()}: "
-            )
-            self.upload_s3_access_key = input(_(msg))
-
-    def prompt_for_upload_s3_secret_key(self):
-        """Should be overridden by policies to determine if a secret key needs
-        to be provided for upload or not
-        """
-        if not self.get_upload_s3_secret_key():
-            msg = (
-                "Please provide the upload secret key for bucket"
-                f" {self.get_upload_s3_bucket()} via endpoint"
-                f" {self.get_upload_s3_endpoint()}: "
-            )
-            self.upload_s3_secret_key = getpass(msg)
-
-    def prompt_for_upload_s3_bucket(self):
-        """Should be overridden by policies to determine if a bucket needs to
-        be provided for upload or not
-        """
-        if not self.upload_s3_bucket:
-            if self.upload_url and self.upload_url.startswith('s3://'):
-                self.upload_s3_bucket = self.upload_url[5:]
-            else:
-                user_input = input(_("Please provide the upload bucket: "))
-                self.upload_s3_bucket = user_input.strip('/')
-        return self.upload_s3_bucket
-
-    def prompt_for_upload_s3_endpoint(self):
-        """Should be overridden by policies to determine if an endpoint needs
-        to be provided for upload or not
-        """
-        default_endpoint = self._upload_s3_endpoint
-        if not self.upload_s3_endpoint:
-            msg = (
-                "Please provide the upload endpoint for bucket"
-                f" {self.get_upload_s3_bucket()}"
-                f" (default: {default_endpoint}): "
-            )
-            user_input = input(_(msg))
-            self.upload_s3_endpoint = user_input or default_endpoint
-        return self.upload_s3_endpoint
-
     def prompt_for_upload_user(self):
         """Should be overridden by policies to determine if a user needs to
         be provided or not
         """
         if not self.get_upload_user():
-            msg = f"Please provide upload user for {self.get_upload_url()}: "
+            msg = "Please provide upload user for %s: " % self.get_upload_url()
             self.upload_user = input(_(msg))
 
     def prompt_for_upload_password(self):
@@ -447,8 +366,8 @@ class LinuxPolicy(Policy):
         """
         if not self.get_upload_password() and (self.get_upload_user() !=
                                                self._upload_user):
-            msg = ("Please provide the upload password for "
-                   f"{self.get_upload_user()}: ")
+            msg = ("Please provide the upload password for %s: "
+                   % self.get_upload_user())
             self.upload_password = getpass(msg)
 
     def upload_archive(self, archive):
@@ -456,7 +375,7 @@ class LinuxPolicy(Policy):
         Entry point for sos attempts to upload the generated archive to a
         policy or user specified location.
 
-        Currently there is support for HTTPS, SFTP, and FTP. HTTPS uploads are
+        Curerntly there is support for HTTPS, SFTP, and FTP. HTTPS uploads are
         preferred for policy-defined defaults.
 
         Policies that need to override uploading methods should override the
@@ -517,8 +436,7 @@ class LinuxPolicy(Policy):
         prots = {
             'ftp': self.upload_ftp,
             'sftp': self.upload_sftp,
-            'https': self.upload_https,
-            's3': self.upload_s3
+            'https': self.upload_https
         }
         if self.commons['cmdlineopts'].upload_protocol in prots.keys():
             return prots[self.commons['cmdlineopts'].upload_protocol]
@@ -526,7 +444,7 @@ class LinuxPolicy(Policy):
             raise Exception("Must provide protocol in upload URL")
         prot, url = self.upload_url.split('://')
         if prot not in prots.keys():
-            raise Exception(f"Unsupported or unrecognized protocol: {prot}")
+            raise Exception("Unsupported or unrecognized protocol: %s" % prot)
         return prots[prot]
 
     def get_upload_https_auth(self, user=None, password=None):
@@ -538,7 +456,7 @@ class LinuxPolicy(Policy):
         :param password: Password for `user` to use for upload
         :type password: ``str``
 
-        :returns: The user/password auth suitable for use in requests calls
+        :returns: The user/password auth suitable for use in reqests calls
         :rtype: ``requests.auth.HTTPBasicAuth()``
         """
         if not user:
@@ -548,73 +466,6 @@ class LinuxPolicy(Policy):
 
         return requests.auth.HTTPBasicAuth(user, password)
 
-    def get_upload_s3_access_key(self):
-        """Helper function to determine if we should use the policy default
-        upload access key or one provided by the user
-
-        :returns: The access_key to use for upload
-        :rtype: ``str``
-        """
-        return (os.getenv('SOSUPLOADS3ACCESSKEY', None) or
-                self.upload_s3_access_key or
-                self._upload_s3_access_key)
-
-    def get_upload_s3_endpoint(self):
-        """Helper function to determine if we should use the policy default
-        upload endpoint or one provided by the user
-
-        :returns: The S3 Endpoint to use for upload
-        :rtype: ``str``
-        """
-        if not self.upload_s3_endpoint:
-            self.prompt_for_upload_s3_endpoint()
-        return self.upload_s3_endpoint
-
-    def get_upload_s3_region(self):
-        """Helper function to determine if we should use the policy default
-        upload region or one provided by the user
-
-        :returns: The S3 region to use for upload
-        :rtype: ``str``
-        """
-        return self.upload_s3_region or self._upload_s3_region
-
-    def get_upload_s3_bucket(self):
-        """Helper function to determine if we should use the policy default
-        upload bucket or one provided by the user
-
-        :returns: The S3 bucket to use for upload
-        :rtype: ``str``
-        """
-        if self.upload_url and self.upload_url.startswith('s3://'):
-            bucket_and_prefix = self.upload_url[5:].split('/', 1)
-            self.upload_s3_bucket = bucket_and_prefix[0]
-            if len(bucket_and_prefix) > 1:
-                self.upload_s3_object_prefix = bucket_and_prefix[1]
-        if not self.upload_s3_bucket:
-            self.prompt_for_upload_s3_bucket()
-        return self.upload_s3_bucket or self._upload_s3_bucket
-
-    def get_upload_s3_object_prefix(self):
-        """Helper function to determine if we should use the policy default
-        upload object prefix or one provided by the user
-
-        :returns: The S3 object prefix to use for upload
-        :rtype: ``str``
-        """
-        return self.upload_s3_object_prefix or self._upload_s3_object_prefix
-
-    def get_upload_s3_secret_key(self):
-        """Helper function to determine if we should use the policy default
-        upload secret key or one provided by the user
-
-        :returns: The S3 secret key to use for upload
-        :rtype: ``str``
-        """
-        return (os.getenv('SOSUPLOADS3SECRETKEY', None) or
-                self.upload_s3_secret_key or
-                self._upload_s3_secret_key)
-
     def get_upload_url(self):
         """Helper function to determine if we should use the policy default
         upload url or one provided by the user
@@ -622,14 +473,6 @@ class LinuxPolicy(Policy):
         :returns: The URL to use for upload
         :rtype: ``str``
         """
-        if not self.upload_url and (
-            self.upload_s3_bucket and
-            self.upload_s3_access_key and
-            self.upload_s3_secret_key
-        ):
-            bucket = self.get_upload_s3_bucket()
-            prefix = self.get_upload_s3_object_prefix()
-            self._upload_url = f"s3://{bucket}/{prefix}"
         return self.upload_url or self._upload_url
 
     def get_upload_url_string(self):
@@ -695,7 +538,7 @@ class LinuxPolicy(Policy):
 
         # need to strip the protocol prefix here
         sftp_url = self.get_upload_url().replace('sftp://', '')
-        sftp_cmd = f"sftp -oStrictHostKeyChecking=no {user}@{sftp_url}"
+        sftp_cmd = "sftp -oStrictHostKeyChecking=no %s@%s" % (user, sftp_url)
         ret = pexpect.spawn(sftp_cmd, encoding='utf-8')
 
         sftp_expects = [
@@ -721,25 +564,25 @@ class LinuxPolicy(Policy):
             sftp_connected = ret.expect(pass_expects, timeout=10) == 0
             if not sftp_connected:
                 ret.close()
-                raise Exception("Incorrect username or password for "
-                                f"{self.get_upload_url_string()}")
+                raise Exception("Incorrect username or password for %s"
+                                % self.get_upload_url_string())
         elif idx == 2:
-            raise Exception("Connection refused by "
-                            f"{self.get_upload_url_string()}. Incorrect port?")
+            raise Exception("Connection refused by %s. Incorrect port?"
+                            % self.get_upload_url_string())
         elif idx == 3:
-            raise Exception("Timeout hit trying to connect to "
-                            f"{self.get_upload_url_string()}")
+            raise Exception("Timeout hit trying to connect to %s"
+                            % self.get_upload_url_string())
         elif idx == 4:
-            raise Exception("Unexpected error trying to connect to sftp: "
-                            f"{ret.before}")
+            raise Exception("Unexpected error trying to connect to sftp: %s"
+                            % ret.before)
 
         if not sftp_connected:
             ret.close()
-            raise Exception("Unable to connect via SFTP to "
-                            f"{self.get_upload_url_string()}")
+            raise Exception("Unable to connect via SFTP to %s"
+                            % self.get_upload_url_string())
 
-        put_cmd = (f'put {self.upload_archive_name} '
-                   f'{self._get_sftp_upload_name()}')
+        put_cmd = 'put %s %s' % (self.upload_archive_name,
+                                 self._get_sftp_upload_name())
         ret.sendline(put_cmd)
 
         put_expects = [
@@ -757,11 +600,11 @@ class LinuxPolicy(Policy):
         elif put_success == 1:
             raise Exception("Timeout expired while uploading")
         elif put_success == 2:
-            raise Exception(f"Unknown error during upload: {ret.before}")
+            raise Exception("Unknown error during upload: %s" % ret.before)
         elif put_success == 3:
             raise Exception("Unable to write archive to destination")
         else:
-            raise Exception(f"Unexpected response from server: {ret.before}")
+            raise Exception("Unexpected response from server: %s" % ret.before)
 
     def _get_sftp_upload_name(self):
         """If a specific file name pattern is required by the SFTP server,
@@ -834,8 +677,8 @@ class LinuxPolicy(Policy):
                     raise Exception(
                         "Authentication failed: invalid user credentials"
                     )
-                raise Exception(f"POST request returned {r.status_code}: "
-                                f"{r.reason}")
+                raise Exception("POST request returned %s: %s"
+                                % (r.status_code, r.reason))
             return True
 
     def upload_ftp(self, url=None, directory=None, user=None, password=None):
@@ -891,24 +734,25 @@ class LinuxPolicy(Policy):
                                 "password?")
             session.cwd(directory)
         except socket.timeout:
-            raise Exception(f"timeout hit while connecting to {url}")
+            raise Exception("timeout hit while connecting to %s" % url)
         except socket.gaierror:
-            raise Exception(f"unable to connect to {url}")
+            raise Exception("unable to connect to %s" % url)
         except ftplib.error_perm as err:
             errno = str(err).split()[0]
             if errno == '503':
-                raise Exception(f"could not login as '{user}'")
+                raise Exception("could not login as '%s'" % user)
             if errno == '530':
-                raise Exception(f"invalid password for user '{user}'")
+                raise Exception("invalid password for user '%s'" % user)
             if errno == '550':
-                raise Exception("could not set upload directory to "
-                                f"{directory}")
-            raise Exception(f"error trying to establish session: {str(err)}")
+                raise Exception("could not set upload directory to %s"
+                                % directory)
+            raise Exception("error trying to establish session: %s"
+                            % str(err))
 
         try:
             with open(self.upload_archive_name, 'rb') as _arcfile:
                 session.storbinary(
-                    f"STOR {self.upload_archive_name.split('/')[-1]}",
+                    "STOR %s" % self.upload_archive_name.split('/')[-1],
                     _arcfile
                 )
             session.quit()
@@ -916,71 +760,6 @@ class LinuxPolicy(Policy):
         except IOError:
             raise Exception("could not open archive file")
 
-    def upload_s3(self, endpoint=None, region=None, bucket=None, prefix=None,
-                  access_key=None, secret_key=None):
-        """Attempts to upload the archive to an S3 bucket.
-
-        :param endpoint: The S3 endpoint to upload to
-        :type endpoint: str
-
-        :param region: The S3 region to upload to
-        :type region: str
-
-        :param bucket: The name of the S3 bucket to upload to
-        :type bucket: str
-
-        :param prefix: The prefix for the S3 object/key
-        :type prefix: str
-
-        :param access_key: The access key for the S3 bucket
-        :type access_key: str
-
-        :param secret_key: The secret key for the S3 bucket
-        :type secret_key: str
-
-        :returns: True if upload is successful
-        :rtype: bool
-
-        :raises: Exception if upload is unsuccessful
-        """
-        if not BOTO3_LOADED:
-            raise Exception("Unable to upload due to missing python boto3 "
-                            "library")
-
-        if not endpoint:
-            endpoint = self.get_upload_s3_endpoint()
-        if not region:
-            region = self.get_upload_s3_region()
-
-        if not bucket:
-            bucket = self.get_upload_s3_bucket().strip('/')
-
-        if not prefix:
-            prefix = self.get_upload_s3_object_prefix()
-            if prefix != '' and prefix.startswith('/'):
-                prefix = prefix[1:]
-            if prefix != '' and not prefix.endswith('/'):
-                prefix = f'{prefix}/' if prefix else ''
-
-        if not access_key:
-            access_key = self.get_upload_s3_access_key()
-
-        if not secret_key:
-            secret_key = self.get_upload_s3_secret_key()
-
-        s3_client = boto3.client('s3', endpoint_url=endpoint,
-                                 region_name=region,
-                                 aws_access_key_id=access_key,
-                                 aws_secret_access_key=secret_key)
-
-        try:
-            key = prefix + self.upload_archive_name.split('/')[-1]
-            s3_client.upload_file(self.upload_archive_name,
-                                  bucket, key)
-            return True
-        except Exception as e:
-            raise Exception(f"Failed to upload to S3: {str(e)}") from e
-
     def set_sos_prefix(self):
         """If sosreport commands need to always be prefixed with something,
         for example running in a specific container image, then it should be
@@ -1020,7 +799,6 @@ class LinuxPolicy(Policy):
         return ''
 
     def restart_sos_container(self):
-        # pylint: disable=no-member
         """Restarts the container created for sos collect if it has stopped.
 
         This is called immediately after create_sos_container() as the command
@@ -1029,10 +807,10 @@ class LinuxPolicy(Policy):
         default to opening a bash shell in the container to keep it running,
         thus allowing us to exec into it again.
         """
-        return f"{self.container_runtime} start {self.sos_container_name}"
+        return "%s start %s" % (self.container_runtime,
+                                self.sos_container_name)
 
     def format_container_command(self, cmd):
-        # pylint: disable=no-member
         """Returns the command that allows us to exec into the created
         container for sos collect.
 
@@ -1043,8 +821,9 @@ class LinuxPolicy(Policy):
         :rtype: ``str``
         """
         if self.container_runtime:
-            return (f'{self.container_runtime} exec {self.sos_container_name} '
-                    f'{cmd}')
+            return '%s exec %s %s' % (self.container_runtime,
+                                      self.sos_container_name,
+                                      cmd)
         else:
             return cmd
 
