@@ -8,9 +8,8 @@
 #
 # See the LICENSE file in the source distribution for further information.
 
-import os
-from sos.report.plugins import AzurePlugin
 from sos.policies.distros.redhat import RedHatPolicy, OS_RELEASE
+import os
 
 
 class AzurePolicy(RedHatPolicy):
@@ -18,15 +17,14 @@ class AzurePolicy(RedHatPolicy):
     distro = "Azure Linux"
     vendor = "Microsoft"
     vendor_urls = [
-        ('Distribution Website', 'https://github.com/microsoft/azurelinux')
+        ('Distribution Website', 'https://github.com/microsoft/CBL-Mariner')
     ]
 
     def __init__(self, sysroot=None, init=None, probe_runtime=True,
                  remote_exec=None):
-        super().__init__(sysroot=sysroot, init=init,
-                         probe_runtime=probe_runtime,
-                         remote_exec=remote_exec)
-        self.valid_subclasses += [AzurePlugin]
+        super(AzurePolicy, self).__init__(sysroot=sysroot, init=init,
+                                          probe_runtime=probe_runtime,
+                                          remote_exec=remote_exec)
 
     @classmethod
     def check(cls, remote=''):
@@ -42,8 +40,6 @@ class AzurePolicy(RedHatPolicy):
                 if line.startswith('NAME'):
                     if 'Common Base Linux Mariner' in line:
                         return True
-                    if 'Microsoft Azure Linux' in line:
-                        return True
         return False
 
 # vim: set et ts=4 sw=4 :
