@@ -15,6 +15,8 @@ from sos.presets import PresetDefaults
 RHEL_RELEASE_STR = "Red Hat Enterprise Linux"
 
 _opts_verify = SoSOptions(verify=True)
+_opts_all_logs = SoSOptions(all_logs=True)
+_opts_all_logs_verify = SoSOptions(all_logs=True, verify=True)
 _cb_profiles = ['boot', 'storage', 'system']
 _cb_plugopts = ['boot.all-images=on', 'rpm.rpmva=on', 'rpm.rpmdb=on']
 
@@ -29,7 +31,7 @@ RHOSP = "rhosp"
 RHOSP_DESC = "Red Hat OpenStack Platform"
 RHOSP_OPTS = SoSOptions(plugopts=[
                              'process.lsof=off',
-                             'networking.ethtool-namespaces=False',
+                             'networking.ethtool_namespaces=False',
                              'networking.namespaces=200'])
 
 RHOCP = "ocp"
@@ -40,7 +42,7 @@ RHOCP_OPTS = SoSOptions(
     plugopts=[
         'crio.timeout=600',
         'networking.timeout=600',
-        'networking.ethtool-namespaces=False',
+        'networking.ethtool_namespaces=False',
         'networking.namespaces=200'
     ])
 
@@ -51,29 +53,6 @@ RH_SATELLITE = "satellite"
 RH_SATELLITE_DESC = "Red Hat Satellite"
 SAT_OPTS = SoSOptions(log_size=100, plugopts=['apache.log=on'])
 
-AAPEDA = 'aap_eda'
-AAPEDA_DESC = 'Ansible Automation Platform Event Driven Controller'
-AAPEDA_OPTS = SoSOptions(
-    enable_plugins=['containers_common'],
-    plugopts=[
-        'containers_common.rootlessusers=eda'
-    ])
-AAPEDA_NOTE = ('Collects \'eda\' user output for the containers_common plugin.'
-               ' If you need more users, do not forget to add \'eda\' '
-               'to your own list for the \'rootlessusers\' option.')
-
-AAPCONTROLLER = 'aap_controller'
-AAPCONTROLLER_DESC = 'Ansible Automation Platform Controller'
-AAPCONTROLLER_OPTS = SoSOptions(
-    enable_plugins=['containers_common'],
-    plugopts=[
-        'containers_common.rootlessusers=awx'
-    ])
-AAPCONTROLLER_NOTE = ('Collects \'awx\' user output for the containers_common'
-                      'pluging. If you need more users, do not forget to add'
-                      '\'awx\' to your own list for the \'rootlessusers\' '
-                      'option.')
-
 CB = "cantboot"
 CB_DESC = "For use when normal system startup fails"
 CB_OPTS = SoSOptions(
@@ -87,11 +66,6 @@ NOTE_TIME = "This preset may increase report run time"
 NOTE_SIZE_TIME = "This preset may increase report size and run time"
 
 RHEL_PRESETS = {
-    AAPEDA: PresetDefaults(name=AAPEDA, desc=AAPEDA_DESC, opts=AAPEDA_OPTS,
-                           note=AAPEDA_NOTE),
-    AAPCONTROLLER: PresetDefaults(name=AAPCONTROLLER, desc=AAPCONTROLLER_DESC,
-                                  opts=AAPCONTROLLER_OPTS,
-                                  note=AAPCONTROLLER_NOTE),
     RHV: PresetDefaults(name=RHV, desc=RHV_DESC, note=NOTE_TIME,
                         opts=_opts_verify),
     RHEL: PresetDefaults(name=RHEL, desc=RHEL_DESC),
@@ -106,4 +80,14 @@ RHEL_PRESETS = {
 }
 
 
+ATOMIC = "atomic"
+ATOMIC_RELEASE_STR = "Atomic"
+ATOMIC_DESC = "Red Hat Enterprise Linux Atomic Host"
+
+ATOMIC_PRESETS = {
+    ATOMIC: PresetDefaults(name=ATOMIC, desc=ATOMIC_DESC, note=NOTE_TIME,
+                           opts=_opts_verify)
+}
+
+
 # vim: set et ts=4 sw=4 :
