@@ -20,7 +20,9 @@ class Apt(Plugin, DebianPlugin, UbuntuPlugin):
 
     def setup(self):
         self.add_copy_spec([
-            "/etc/apt", "/var/log/apt"
+            "/etc/apt",
+            "/var/log/apt",
+            "/var/log/unattended-upgrades"
         ])
 
         self.add_forbidden_path("/etc/apt/auth.conf")
@@ -43,4 +45,18 @@ class Apt(Plugin, DebianPlugin, UbuntuPlugin):
             suggest_filename="apt-cache_policy_details"
         )
 
+    def postproc(self):
+        super(Apt, self).postproc()
+        self.do_file_sub(
+            "/etc/apt/sources.list",
+            r"(deb\shttp(s)?://)\S+:\S+(@.*)",
+            r"\1******:******\3"
+        )
+        self.do_path_regex_sub(
+            "/etc/apt/sources.list.d/",
+            r"(deb\shttp(s)?://)\S+:\S+(@.*)",
+            r"\1******:******\3"
+        )
+
+
 # vim: set et ts=4 sw=4 :
