@@ -22,12 +22,12 @@ class Chrony(Plugin):
         self.add_cmd_output([
             "chronyc activity",
             "chronyc tracking",
-            "chronyc -n sources",
             "chronyc sourcestats",
             "chronyc serverstats",
             "chronyc ntpdata",
             "chronyc -n clients"
         ])
+        self.add_cmd_output("chronyc -n sources", tags="chronyc_sources")
 
 
 class RedHatChrony(Chrony, RedHatPlugin):
@@ -45,6 +45,8 @@ class DebianChrony(Chrony, DebianPlugin, UbuntuPlugin):
         super(DebianChrony, self).setup()
         self.add_copy_spec([
             "/etc/chrony/chrony.conf",
+            "/etc/chrony/conf.d",
+            "/etc/chrony/sources.d",
             "/var/lib/chrony/chrony.drift",
             "/etc/default/chrony"
         ])
