@@ -13,15 +13,16 @@ class convert2rhel(Plugin, RedHatPlugin):
 
     short_desc = 'Convert2RHEL'
     plugin_name = 'convert2rhel'
-    profiles = ('system')
-    packages = ('convert2rhel')
+    profiles = ('system',)
+    packages = ('convert2rhel',)
     verify_packages = ('convert2rhel$',)
 
     def setup(self):
 
         self.add_copy_spec([
             "/var/log/convert2rhel/convert2rhel.log",
-            "/var/log/convert2rhel/rpm_va.log"
+            "/var/log/convert2rhel/archive/convert2rhel-*.log",
+            "/var/log/convert2rhel/rpm_va.log",
         ])
 
 
