@@ -15,7 +15,7 @@ class Cups(Plugin, IndependentPlugin):
 
     plugin_name = 'cups'
     profiles = ('hardware',)
-
+    services = ('cups', 'cups-browsed')
     packages = ('cups',)
 
     def setup(self):
@@ -39,6 +39,4 @@ class Cups(Plugin, IndependentPlugin):
             "lpstat -d"
         ])
 
-        self.add_journal(units="cups")
-
 # vim: set et ts=4 sw=4 :
