@@ -44,19 +44,34 @@ class Filesys(Plugin, DebianPlugin, UbuntuPlugin, CosPlugin):
             "/proc/self/mountstats",
             "/proc/[0-9]*/mountinfo",
             "/etc/mtab",
-            "/etc/fstab"
+            "/etc/fstab",
+            "/run/mount/utab",
         ])
         self.add_cmd_output("mount -l", root_symlink="mount",
                             tags="mount")
         self.add_cmd_output("df -al -x autofs", root_symlink="df",
                             tags='df__al')
         self.add_cmd_output([
-            "df -ali -x autofs",
+            "df -aliT -x autofs",
             "findmnt",
             "lslocks"
         ])
 
-        self.add_forbidden_path('/proc/fs/panfs')
+        self.add_forbidden_path([
+            # cifs plugin
+            '/proc/fs/cifs',
+            # lustre plugin
+            '/proc/fs/ldiskfs',
+            '/proc/fs/lustre',
+            # nfs plugin
+            '/proc/fs/nfsd',
+            '/proc/fs/nfsfs',
+            # panfs (from Panasas company) provides statistics which can be
+            # very large (100s of GB)
+            '/proc/fs/panfs',
+            # xfs plugin
+            '/proc/fs/xfs'
+        ])
 
         if self.get_option('lsof'):
             self.add_cmd_output("lsof -b +M -n -l -P", root_symlink="lsof",
@@ -68,11 +83,11 @@ class Filesys(Plugin, DebianPlugin, UbuntuPlugin, CosPlugin):
         mounts = '/proc/mounts'
         ext_fs_regex = r"^(/dev/\S+).+ext[234]\s+"
         for dev in self.do_regex_find_all(ext_fs_regex, mounts):
-            self.add_cmd_output("dumpe2fs %s %s" % (dumpe2fs_opts, dev),
+            self.add_cmd_output(f"dumpe2fs {dumpe2fs_opts} {dev}",
                                 tags="dumpe2fs_h")
 
             if self.get_option('frag'):
-                self.add_cmd_output("e2freefrag %s" % (dev), priority=100)
+                self.add_cmd_output(f"e2freefrag {dev}", priority=100)
 
     def postproc(self):
         self.do_file_sub(
@@ -92,7 +107,7 @@ class Filesys(Plugin, DebianPlugin, UbuntuPlugin, CosPlugin):
 class RedHatFilesys(Filesys, RedHatPlugin):
 
     def setup(self):
-        super(RedHatFilesys, self).setup()
-        self.add_cmd_output("ls -ltradZ /tmp")
+        super().setup()
+        self.add_cmd_output('ls -ldZ /tmp')
 
 # vim: set et ts=4 sw=4 :
