@@ -22,7 +22,7 @@ class ForemanProxy(Plugin):
 
     def setup(self):
         self.add_file_tags({
-            '/var/log/foreman-proxy/proxy.log.*': 'foreman_proxy_log',
+            '/var/log/foreman-proxy/proxy.log': 'foreman_proxy_log',
             '/etc/foreman-proxy/settings.yml': 'foreman_proxy_conf'
         })
 
@@ -38,6 +38,10 @@ class ForemanProxy(Plugin):
                 self.apachepkg),
             "/var/log/{}*/katello-reverse-proxy_error_ssl.log*".format(
                 self.apachepkg),
+            "/var/log/{}*/rhsm-pulpcore-https-*_access_ssl.log*".format(
+                self.apachepkg),
+            "/var/log/{}*/rhsm-pulpcore-https-*_error_ssl.log*".format(
+                self.apachepkg),
         ])
 
         # collect http[|s]_proxy env.variables
@@ -64,7 +68,7 @@ class RedHatForemanProxy(ForemanProxy, RedHatPlugin):
 
 class DebianForemanProxy(ForemanProxy, DebianPlugin, UbuntuPlugin):
 
-    apachepkg = 'apache'
+    apachepkg = 'apache2'
 
 
 # vim: set et ts=4 sw=4 :
