@@ -21,7 +21,8 @@ class Grub2(Plugin, IndependentPlugin):
 
         self.add_file_tags({
             '/boot/grub2/grub.cfg': 'grub2_cfg',
-            '/boot/efi/.*/grub.cfg': 'grub2_efi_cfg'
+            '/boot/efi/.*/grub.cfg': 'grub2_efi_cfg',
+            '/boot/grub2/grubenv': 'grubenv'
         })
 
         self.add_copy_spec([
@@ -30,9 +31,12 @@ class Grub2(Plugin, IndependentPlugin):
             "/boot/grub2/grubenv",
             "/boot/grub/grub.cfg",
             "/boot/loader/entries",
+            "/boot/grub2/custom.cfg",
+            "/boot/grub2/user.cfg",
             "/etc/default/grub",
             "/etc/grub2.cfg",
-            "/etc/grub.d"
+            "/etc/grub.d",
+            "/etc/grub2-efi.cfg"
         ])
 
         self.add_cmd_output("ls -lanR /boot", tags="ls_boot")
