@@ -8,37 +8,44 @@
 #
 # See the LICENSE file in the source distribution for further information.
 
-from sos.report.plugins import Plugin, RedHatPlugin
+from sos.report.plugins import Plugin, RedHatPlugin, DebianPlugin, UbuntuPlugin
 
 
 class Krb5(Plugin):
+    """This plugin handles the collection of kerberos authentication config
+    files and logging. Users should expect to see their krb5 config(s) in the
+    final archive, along with krb5 logging and `klist` output.
+
+    kdc configs and acls will also be collected from the distribution-spcecific
+    kdc directory.
+    """
 
     short_desc = 'Kerberos authentication'
     plugin_name = 'krb5'
     profiles = ('identity', 'system')
-    packages = ('krb5-libs', 'krb5-user')
-
-    # This is Debian's default, which is closest to upstream's
-    kdcdir = "/var/lib/krb5kdc"
 
     def setup(self):
         self.add_copy_spec([
             "/etc/krb5.conf",
             "/etc/krb5.conf.d/*",
-            "%s/kadm5.acl" % self.kdcdir,
-            "%s/kdc.conf" % self.kdcdir,
-            "/var/log/krb5kdc.log",
+            f"{self.kdcdir}/kadm5.acl",
+            f"{self.kdcdir}/kdc.conf",
             "/var/log/kadmind.log"
         ])
-        self.add_cmd_output("klist -ket %s/.k5*" % self.kdcdir)
+        self.add_copy_spec("/var/log/krb5kdc.log", tags="kerberos_kdc_log")
+        self.add_cmd_output(f"klist -ket {self.kdcdir}/.k5*")
         self.add_cmd_output("klist -ket /etc/krb5.keytab")
 
 
 class RedHatKrb5(Krb5, RedHatPlugin):
 
-    def setup(self):
-        self.kdcdir = "/var/kerberos/krb5kdc"
-        super(RedHatKrb5, self).setup()
+    packages = ('krb5-libs', 'krb5-server')
+    kdcdir = "/var/kerberos/krb5kdc"
 
 
+class UbuntuKrb5(Krb5, DebianPlugin, UbuntuPlugin):
+
+    packages = ('krb5-kdc', 'krb5-config', 'krb5-user')
+    kdcdir = "/var/lib/krb5kdc"
+
 # vim: set et ts=4 sw=4 :
