@@ -20,13 +20,13 @@ class Networking(Plugin):
 
     option_list = [
         PluginOpt("traceroute", default=False,
-                  desc="collect a traceroute to %s" % trace_host),
-        PluginOpt("namespace_pattern", default="", val_type=str,
+                  desc=f"collect a traceroute to {trace_host}"),
+        PluginOpt("namespace-pattern", default="", val_type=str,
                   desc=("Specific namespace names or patterns to collect, "
                         "whitespace delimited.")),
         PluginOpt("namespaces", default=None, val_type=int,
                   desc="Number of namespaces to collect, 0 for unlimited"),
-        PluginOpt("ethtool_namespaces", default=True,
+        PluginOpt("ethtool-namespaces", default=True,
                   desc=("Toggle if ethtool commands should be run for each "
                         "namespace")),
         PluginOpt("eepromdump", default=False,
@@ -36,16 +36,16 @@ class Networking(Plugin):
     # switch to enable netstat "wide" (non-truncated) output mode
     ns_wide = "-W"
 
+    # list of kernel modules needed by ss_cmd, this may vary by distro version
+    ss_kmods = ['tcp_diag', 'udp_diag', 'inet_diag', 'unix_diag',
+                'netlink_diag', 'af_packet_diag', 'xsk_diag']
+
     # list of ethtool short options, used in add_copy_spec and add_cmd_tags
     # do NOT add there "e" (see eepromdump plugopt)
     ethtool_shortopts = "acdgiklmPST"
 
     def setup(self):
-        super(Networking, self).setup()
-        for opt in self.ethtool_shortopts:
-            self.add_cmd_tags({
-                'ethtool -%s .*' % opt: 'ethool_%s' % opt
-            })
+        super().setup()
 
         self.add_file_tags({
             '/proc/net/bonding/bond.*': 'bond',
@@ -53,20 +53,20 @@ class Networking(Plugin):
         })
 
         self.add_copy_spec([
-            "/proc/net/",
-            "/etc/nsswitch.conf",
-            "/etc/yp.conf",
+            "/etc/dnsmasq*",
+            "/etc/host*",
             "/etc/inetd.conf",
+            "/etc/iproute2",
+            "/etc/network*",
+            "/etc/nsswitch.conf",
+            "/etc/resolv.conf",
             "/etc/xinetd.conf",
             "/etc/xinetd.d",
-            "/etc/host*",
-            "/etc/resolv.conf",
-            "/etc/network*",
-            "/etc/dnsmasq*",
+            "/etc/yp.conf",
+            "/proc/net/",
             "/sys/class/net/*/device/numa_node",
             "/sys/class/net/*/flags",
             "/sys/class/net/*/statistics/",
-            "/etc/iproute2"
         ])
 
         self.add_forbidden_path([
@@ -86,13 +86,13 @@ class Networking(Plugin):
                             tags=['ip_route', 'iproute_show_table_all'])
         self.add_cmd_output("plotnetcfg")
 
-        self.add_cmd_output("netstat %s -neopa" % self.ns_wide,
+        self.add_cmd_output(f"netstat {self.ns_wide} -neopa",
                             root_symlink="netstat")
 
         self.add_cmd_output([
             "nstat -zas",
             "netstat -s",
-            "netstat %s -agn" % self.ns_wide,
+            f"netstat {self.ns_wide} -agn",
             "networkctl status -a",
             "ip -6 route show table all",
             "ip -d route show cache",
@@ -122,8 +122,7 @@ class Networking(Plugin):
             if devlinks['status'] == 0:
                 devlinks_list = devlinks['output'].splitlines()
                 for devlink in devlinks_list:
-                    self.add_cmd_output("devlink dev eswitch show %s" %
-                                        devlink)
+                    self.add_cmd_output(f"devlink dev eswitch show {devlink}")
 
         # below commands require some kernel module(s) to be loaded
         # run them only if the modules are loaded, or if explicitly requested
@@ -132,16 +131,47 @@ class Networking(Plugin):
         macsec_pred = SoSPredicate(self, kmods=['macsec'])
         self.add_cmd_output(ip_macsec_show_cmd, pred=macsec_pred, changes=True)
 
+        self.collect_ss_ip_ethtool_info()
+        self.collect_bridge_info()
+
+    def add_command_tags(self):
+        """ Command tags for ip/ethtool/netstat """
+        for opt in self.ethtool_shortopts:
+            self.add_cmd_tags({
+                f'ethtool -{opt} .*': f'ethool_{opt}'
+            })
+
+        self.add_cmd_tags({
+            "ethtool [^-].*": "ethtool",
+            "ip -d address": "ip_addr",
+            "ip -s -s neigh show": "ip_neigh_show",
+            "ip -s -d link": "ip_s_link",
+            "netstat.*-neopa": "netstat",
+            "netstat.*-agn": "netstat_agn",
+            "netstat -s": "netstat_s"
+        })
+
+    def collect_bridge_info(self):
+        """ Collect information about bridges (some data already collected via
+        "ip .." commands)
+        """
+        self.add_cmd_output([
+            "bridge -s -s -d link show",
+            "bridge -s -s -d -t fdb show",
+            "bridge -s -s -d -t mdb show",
+            "bridge -d vlan show"
+        ])
+
+    def collect_ss_ip_ethtool_info(self):
+        """ Collect ss, ip and ethtool cmd outputs """
         ss_cmd = "ss -peaonmi"
-        ss_pred = SoSPredicate(self, kmods=[
-            'tcp_diag', 'udp_diag', 'inet_diag', 'unix_diag', 'netlink_diag',
-            'af_packet_diag', 'xsk_diag'
-        ], required={'kmods': 'all'})
+        ss_pred = SoSPredicate(self, kmods=self.ss_kmods,
+                               required={'kmods': 'all'})
         self.add_cmd_output(ss_cmd, pred=ss_pred, changes=True)
 
         # Get ethtool output for every device that does not exist in a
         # namespace.
-        _ecmds = ["ethtool -%s" % opt for opt in self.ethtool_shortopts]
+        _ecmds = [f"ethtool -{opt}" for opt in self.ethtool_shortopts]
         self.add_device_cmd([
             _cmd + " %(dev)s" for _cmd in _ecmds
         ], devices='ethernet')
@@ -164,17 +194,8 @@ class Networking(Plugin):
                            "interrupt device operation")
             self.add_device_cmd(cmd, devices="ethernet")
 
-        # Collect information about bridges (some data already collected via
-        # "ip .." commands)
-        self.add_cmd_output([
-            "bridge -s -s -d link show",
-            "bridge -s -s -d -t fdb show",
-            "bridge -s -s -d -t mdb show",
-            "bridge -d vlan show"
-        ])
-
         if self.get_option("traceroute"):
-            self.add_cmd_output("/bin/traceroute -n %s" % self.trace_host,
+            self.add_cmd_output(f"/bin/traceroute -n {self.trace_host}",
                                 priority=100)
 
         # Capture additional data from namespaces; each command is run
@@ -182,17 +203,17 @@ class Networking(Plugin):
         self.add_cmd_output("ip netns")
         cmd_prefix = "ip netns exec "
         namespaces = self.get_network_namespaces(
-                self.get_option("namespace_pattern"),
+                self.get_option("namespace-pattern"),
                 self.get_option("namespaces"))
         if namespaces:
             # 'ip netns exec <foo> iptables-save' must be guarded by nf_tables
             # kmod, if 'iptables -V' output contains 'nf_tables'
             # analogously for ip6tables
-            co = {'cmd': 'iptables -V', 'output': 'nf_tables'}
+            cout = {'cmd': 'iptables -V', 'output': 'nf_tables'}
             co6 = {'cmd': 'ip6tables -V', 'output': 'nf_tables'}
             iptables_with_nft = (SoSPredicate(self, kmods=['nf_tables'])
                                  if self.test_predicate(self,
-                                 pred=SoSPredicate(self, cmd_outputs=co))
+                                 pred=SoSPredicate(self, cmd_outputs=cout))
                                  else None)
             ip6tables_with_nft = (SoSPredicate(self, kmods=['nf_tables'])
                                   if self.test_predicate(self,
@@ -201,7 +222,7 @@ class Networking(Plugin):
 
             for namespace in namespaces:
                 _devs = self.devices['namespaced_network'][namespace]
-                _subdir = "namespaces/%s" % namespace
+                _subdir = f"namespaces/{namespace}"
                 ns_cmd_prefix = cmd_prefix + namespace + " "
                 self.add_cmd_output([
                     ns_cmd_prefix + "ip -d address show",
@@ -211,9 +232,9 @@ class Networking(Plugin):
                     ns_cmd_prefix + "ip -6 rule list",
                     ns_cmd_prefix + "ip vrf show",
                     ns_cmd_prefix + "sysctl -a",
-                    ns_cmd_prefix + "netstat %s -neopa" % self.ns_wide,
+                    ns_cmd_prefix + f"netstat {self.ns_wide} -neopa",
                     ns_cmd_prefix + "netstat -s",
-                    ns_cmd_prefix + "netstat %s -agn" % self.ns_wide,
+                    ns_cmd_prefix + f"netstat {self.ns_wide} -agn",
                     ns_cmd_prefix + "nstat -zas",
                 ], priority=50, subdir=_subdir)
                 self.add_cmd_output([ns_cmd_prefix + "iptables-save"],
@@ -233,7 +254,7 @@ class Networking(Plugin):
 
                 # Collect ethtool commands only when ethtool_namespaces
                 # is set to true.
-                if self.get_option("ethtool_namespaces"):
+                if self.get_option("ethtool-namespaces"):
                     # Devices that exist in a namespace use less ethtool
                     # parameters. Run this per namespace.
                     self.add_device_cmd([
@@ -243,22 +264,7 @@ class Networking(Plugin):
                         ns_cmd_prefix + "ethtool -S %(dev)s"
                     ], devices=_devs['ethernet'], priority=50, subdir=_subdir)
 
-        self.add_cmd_tags({
-            "ethtool [^-].*": "ethtool",
-            "ethtool -S.*": "ethtool_S",
-            "ethtool -T.*": "ethtool_T",
-            "ethtool -a.*": "ethtool_a",
-            "ethtool -c.*": "ethtool_c",
-            "ethtool -g.*": "ethtool_g",
-            "ethtool -i.*": "ethtool_i",
-            "ethtool -k.*": "ethtool_k",
-            "ip -d address": "ip_addr",
-            "ip -s -s neigh show": "ip_neigh_show",
-            "ip -s -d link": "ip_s_link",
-            "netstat.*-neopa": "netstat",
-            "netstat.*-agn": "netstat_agn",
-            "netstat -s": "netstat_s"
-        })
+        self.add_command_tags()
 
 
 class RedHatNetworking(Networking, RedHatPlugin):
@@ -271,34 +277,40 @@ class RedHatNetworking(Networking, RedHatPlugin):
             # major version
             if int(netstat_pkg['version'][0]) < 2:
                 self.ns_wide = "-T"
-        except Exception:
+        except Exception:  # pylint: disable=broad-except
             # default to upstream option
             pass
 
-        super(RedHatNetworking, self).setup()
+        super().setup()
 
 
 class UbuntuNetworking(Networking, UbuntuPlugin, DebianPlugin):
     trace_host = "archive.ubuntu.com"
 
     def setup(self):
-        super(UbuntuNetworking, self).setup()
+
+        ubuntu_jammy_and_after_ss_kmods = ['tcp_diag', 'udp_diag',
+                                           'inet_diag', 'unix_diag',
+                                           'netlink_diag', 'af_packet_diag',
+                                           'xsk_diag', 'mptcp_diag',
+                                           'raw_diag']
+
+        if self.policy.dist_version() >= 22.04:
+            self.ss_kmods = ubuntu_jammy_and_after_ss_kmods
+
+        super().setup()
 
         self.add_copy_spec([
-            "/etc/resolvconf",
+            "/etc/netplan/*.yaml",
             "/etc/network/interfaces",
             "/etc/network/interfaces.d",
             "/etc/resolv.conf",
-            "/run/netplan/*.yaml",
-            "/etc/netplan/*.yaml",
+            "/etc/resolvconf",
             "/lib/netplan/*.yaml",
+            "/run/netplan/*.yaml",
             "/run/systemd/network"
         ])
 
-        if self.get_option("traceroute"):
-            self.add_cmd_output("/usr/sbin/traceroute -n %s" % self.trace_host,
-                                priority=100)
-
     def postproc(self):
 
         self.do_path_regex_sub(
