@@ -10,7 +10,6 @@
 
 from sos.report.plugins import Plugin, RedHatPlugin, DebianPlugin, UbuntuPlugin
 
-from os.path import join as path_join
 from os import environ
 
 import re
@@ -65,7 +64,9 @@ class OpenVSwitch(Plugin):
             log_dirs.append(environ.get('OVS_LOGDIR'))
 
         if not all_logs:
-            self.add_copy_spec([path_join(ld, '*.log') for ld in log_dirs])
+            self.add_copy_spec([
+                self.path_join(ld, '*.log') for ld in log_dirs
+            ])
         else:
             self.add_copy_spec(log_dirs)
 
@@ -75,12 +76,23 @@ class OpenVSwitch(Plugin):
             "/run/openvswitch/ovs-monitor-ipsec.pid"
         ])
 
+        self.add_copy_spec([
+            self.path_join('/usr/local/etc/openvswitch', 'conf.db'),
+            self.path_join('/etc/openvswitch', 'conf.db'),
+            self.path_join('/var/lib/openvswitch', 'conf.db'),
+        ])
+        ovs_dbdir = environ.get('OVS_DBDIR')
+        if ovs_dbdir:
+            self.add_copy_spec(self.path_join(ovs_dbdir, 'conf.db'))
+
+        self.add_file_tags({
+            "/var/log/openvswitch/ovs-vswitchd.log":
+                "openvswitch_daemon_log",
+            "/var/log/openvswitch/ovsdb-server.log":
+                "openvswitch_server_log"
+        })
+
         self.add_cmd_output([
-            # The '-t 5' adds an upper bound on how long to wait to connect
-            # to the Open vSwitch server, avoiding hangs when running sos.
-            "ovs-vsctl -t 5 show",
-            # Gather the database.
-            "ovsdb-client -f list dump",
             # List the contents of important runtime directories
             "ls -laZ /run/openvswitch",
             "ls -laZ /dev/hugepages/",
@@ -108,14 +120,14 @@ class OpenVSwitch(Plugin):
             "ovs-appctl tnl/ports/show -v",
             # Capture upcall information
             "ovs-appctl upcall/show",
-            # Capture DPDK and other parameters
-            "ovs-vsctl -t 5 get Open_vSwitch . other_config",
             # Capture OVS list
             "ovs-vsctl -t 5 list Open_vSwitch",
             # Capture OVS interface list
             "ovs-vsctl -t 5 list interface",
             # Capture OVS detailed information from all the bridges
             "ovs-vsctl -t 5 list bridge",
+            # Capture OVS datapath list
+            "ovs-vsctl -t 5 list datapath",
             # Capture DPDK queue to pmd mapping
             "ovs-appctl dpif-netdev/pmd-rxq-show",
             # Capture DPDK pmd stats
@@ -139,6 +151,13 @@ class OpenVSwitch(Plugin):
             # Capture miniflow extract implementations
             "ovs-appctl dpif-netdev/miniflow-parser-get"
         ])
+        # Capture DPDK and other parameters
+        self.add_cmd_output("ovs-vsctl -t 5 get Open_vSwitch . other_config",
+                            tags="openvswitch_other_config")
+        # The '-t 5' adds an upper bound on how long to wait to connect
+        # to the Open vSwitch server, avoiding hangs when running sos.
+        self.add_cmd_output("ovs-vsctl -t 5 show",
+                            tags="ovs_vsctl_show")
 
         # Gather systemd services logs
         self.add_journal(units="openvswitch")
@@ -206,6 +225,7 @@ class OpenVSwitch(Plugin):
 
         # Gather additional output for each OVS bridge on the host.
         br_list_result = self.collect_cmd_output("ovs-vsctl -t 5 list-br")
+        ofp_ver_result = self.collect_cmd_output("ovs-ofctl -t 5 --version")
         if br_list_result['status'] == 0:
             for br in br_list_result['output'].splitlines():
                 self.add_cmd_output([
@@ -220,6 +240,7 @@ class OpenVSwitch(Plugin):
                     "ovs-ofctl queue-get-config %s" % br,
                     "ovs-ofctl queue-stats %s" % br,
                     "ovs-ofctl show %s" % br,
+                    "ovs-ofctl dump-groups %s" % br,
                 ])
 
                 # Flow protocols currently supported
@@ -232,6 +253,16 @@ class OpenVSwitch(Plugin):
                     "OpenFlow15"
                 ]
 
+                # Flow protocol hex identifiers
+                ofp_versions = {
+                    0x01: "OpenFlow10",
+                    0x02: "OpenFlow11",
+                    0x03: "OpenFlow12",
+                    0x04: "OpenFlow13",
+                    0x05: "OpenFlow14",
+                    0x06: "OpenFlow15",
+                }
+
                 # List protocols currently in use, if any
                 ovs_list_bridge_cmd = "ovs-vsctl -t 5 list bridge %s" % br
                 br_info = self.collect_cmd_output(ovs_list_bridge_cmd)
@@ -242,6 +273,21 @@ class OpenVSwitch(Plugin):
                         br_protos_ln = line[line.find("[")+1:line.find("]")]
                         br_protos = br_protos_ln.replace('"', '').split(", ")
 
+                # If 'list bridge' yeilded no protocols, use the range of
+                # protocols enabled by default on this version of ovs.
+                if br_protos == [''] and ofp_ver_result['output']:
+                    ofp_version_range = ofp_ver_result['output'].splitlines()
+                    ver_range = []
+
+                    for line in ofp_version_range:
+                        if "OpenFlow versions" in line:
+                            v = line.split("OpenFlow versions ")[1].split(":")
+                            ver_range = range(int(v[0], 16), int(v[1], 16)+1)
+
+                    for protocol in ver_range:
+                        if protocol in ofp_versions:
+                            br_protos.append(ofp_versions[protocol])
+
                 # Collect flow information for relevant protocol versions only
                 for flow in flow_versions:
                     if flow in br_protos:
@@ -250,6 +296,7 @@ class OpenVSwitch(Plugin):
                             "ovs-ofctl -O %s dump-groups %s" % (flow, br),
                             "ovs-ofctl -O %s dump-group-stats %s" % (flow, br),
                             "ovs-ofctl -O %s dump-flows %s" % (flow, br),
+                            "ovs-ofctl -O %s dump-tlv-map %s" % (flow, br),
                             "ovs-ofctl -O %s dump-ports-desc %s" % (flow, br)
                         ])
 
@@ -327,7 +374,7 @@ class OpenVSwitch(Plugin):
 
 class RedHatOpenVSwitch(OpenVSwitch, RedHatPlugin):
 
-    packages = ('openvswitch', 'openvswitch2.*',
+    packages = ('openvswitch', 'openvswitch[2-9].*',
                 'openvswitch-dpdk', 'nuage-openvswitch'
                 '6windgate-fp')
 
