@@ -16,7 +16,7 @@ import glob
 class Peripety(Plugin, RedHatPlugin):
 
     short_desc = 'Peripety Storage Event Monitor'
-
+    plugin_name = 'peripety'
     packages = ('peripety',)
     services = ('peripetyd',)
 
@@ -38,6 +38,5 @@ class Peripety(Plugin, RedHatPlugin):
                 "prpt info %s" % disk,
                 "prpt query --blk %s" % disk
             ])
-        self.add_journal('peripetyd')
 
 # vim: set et ts=4 sw=4 :
