@@ -14,7 +14,8 @@
 
 import os
 
-from sos.report.plugins import (Plugin, UbuntuPlugin, DebianPlugin, SCLPlugin)
+from sos.report.plugins import (Plugin, UbuntuPlugin, DebianPlugin, SCLPlugin,
+                                PluginOpt)
 from sos.utilities import find
 
 
@@ -30,12 +31,18 @@ class PostgreSQL(Plugin):
     password_warn_text = " (password visible in process listings)"
 
     option_list = [
-        ('pghome', 'PostgreSQL server home directory.', '', '/var/lib/pgsql'),
-        ('username', 'username for pg_dump', '', 'postgres'),
-        ('password', 'password for pg_dump' + password_warn_text, '', ''),
-        ('dbname', 'database name to dump for pg_dump', '', ''),
-        ('dbhost', 'database hostname/IP (do not use unix socket)', '', ''),
-        ('dbport', 'database server port number', '', '5432')
+        PluginOpt('pghome', default='/var/lib/pgsql',
+                  desc='psql server home directory'),
+        PluginOpt('username', default='postgres', val_type=str,
+                  desc='username for pg_dump'),
+        PluginOpt('password', default='', val_type=str,
+                  desc='password for pg_dump' + password_warn_text),
+        PluginOpt('dbname', default='', val_type=str,
+                  desc='database name to dump with pg_dump'),
+        PluginOpt('dbhost', default='', val_type=str,
+                  desc='database hostname/IP address (no unix sockets)'),
+        PluginOpt('dbport', default=5432, val_type=int,
+                  desc='database server listening port')
     ]
 
     def do_pg_dump(self, scl=None, filename="pgdump.tar"):
@@ -100,7 +107,7 @@ class RedHatPostgreSQL(PostgreSQL, SCLPlugin):
             scl = pkg.split('-postgresql-')[0]
             _dir = self.convert_copyspec_scl(scl, pghome)
             dirs.append(_dir)
-            if os.path.isdir(_dir):
+            if self.path_isdir(_dir):
                 self.add_cmd_output("du -sh %s" % _dir)
             if (self.is_service_running(pkg.replace('-server', '')) and
                     scl in self.scls_matched):
@@ -117,7 +124,7 @@ class RedHatPostgreSQL(PostgreSQL, SCLPlugin):
 
             # copy PG_VERSION and postmaster.opts
             for f in ["PG_VERSION", "postmaster.opts"]:
-                self.add_copy_spec(os.path.join(_dir, "data", f))
+                self.add_copy_spec(self.path_join(_dir, "data", f))
 
 
 class DebianPostgreSQL(PostgreSQL, DebianPlugin, UbuntuPlugin):
