@@ -8,9 +8,9 @@
 #
 # See the LICENSE file in the source distribution for further information.
 
-from sos.report.plugins import Plugin, IndependentPlugin, PluginOpt
-from pipes import quote
 from re import match
+from shlex import quote
+from sos.report.plugins import Plugin, IndependentPlugin, PluginOpt
 
 
 class PulpCore(Plugin, IndependentPlugin):
@@ -24,16 +24,17 @@ class PulpCore(Plugin, IndependentPlugin):
         PluginOpt('task-days', default=7, desc='days of task history')
     ]
 
+    dbhost = "localhost"
+    dbport = 5432
+    dbname = "pulpcore"
+    dbpasswd = ""
+    staticroot = "/var/lib/pulp/assets"
+    uploaddir = "/var/lib/pulp/media/upload"
+    env = {"PGPASSWORD": dbpasswd}
+
     def parse_settings_config(self):
+        """ Parse pulp settings """
         databases_scope = False
-        self.dbhost = "localhost"
-        self.dbport = 5432
-        self.dbname = "pulpcore"
-        self.dbpasswd = ""
-        # TODO: read also redis config (we dont expect much customisations)
-        # TODO: read also db user (pulp)
-        self.staticroot = "/var/lib/pulp/assets"
-        self.uploaddir = "/var/lib/pulp/media/upload"
 
         def separate_value(line, sep=':'):
             # an auxiliary method to parse values from lines like:
@@ -45,34 +46,35 @@ class PulpCore(Plugin, IndependentPlugin):
             return val
 
         try:
-            # split the lines to "one option per line" format
-            for line in open("/etc/pulp/settings.py").read() \
-                    .replace(',', ',\n').replace('{', '{\n') \
-                    .replace('}', '\n}').splitlines():
-                # skip empty lines and lines with comments
-                if not line or line[0] == '#':
-                    continue
-                if line.startswith("DATABASES"):
-                    databases_scope = True
-                    continue
-                # example HOST line to parse:
-                #         'HOST': 'localhost',
-                pattern = r"\s*['|\"]%s['|\"]\s*:\s*\S+"
-                if databases_scope and match(pattern % 'HOST', line):
-                    self.dbhost = separate_value(line)
-                if databases_scope and match(pattern % 'PORT', line):
-                    self.dbport = separate_value(line)
-                if databases_scope and match(pattern % 'NAME', line):
-                    self.dbname = separate_value(line)
-                if databases_scope and match(pattern % 'PASSWORD', line):
-                    self.dbpasswd = separate_value(line)
-                # if line contains closing '}' database_scope end
-                if databases_scope and '}' in line:
-                    databases_scope = False
-                if line.startswith("STATIC_ROOT = "):
-                    self.staticroot = separate_value(line, sep='=')
-                if line.startswith("CHUNKED_UPLOAD_DIR = "):
-                    self.uploaddir = separate_value(line, sep='=')
+            with open("/etc/pulp/settings.py", 'r', encoding='UTF-8') as file:
+                # split the lines to "one option per line" format
+                for line in file.read() \
+                        .replace(',', ',\n').replace('{', '{\n') \
+                        .replace('}', '\n}').splitlines():
+                    # skip empty lines and lines with comments
+                    if not line or line[0] == '#':
+                        continue
+                    if line.startswith("DATABASES"):
+                        databases_scope = True
+                        continue
+                    # example HOST line to parse:
+                    #         'HOST': 'localhost',
+                    pattern = r"\s*['|\"]%s['|\"]\s*:\s*\S+"
+                    if databases_scope and match(pattern % 'HOST', line):
+                        self.dbhost = separate_value(line)
+                    if databases_scope and match(pattern % 'PORT', line):
+                        self.dbport = separate_value(line)
+                    if databases_scope and match(pattern % 'NAME', line):
+                        self.dbname = separate_value(line)
+                    if databases_scope and match(pattern % 'PASSWORD', line):
+                        self.dbpasswd = separate_value(line)
+                    # if line contains closing '}' database_scope end
+                    if databases_scope and '}' in line:
+                        databases_scope = False
+                    if line.startswith("STATIC_ROOT = "):
+                        self.staticroot = separate_value(line, sep='=')
+                    if line.startswith("CHUNKED_UPLOAD_DIR = "):
+                        self.uploaddir = separate_value(line, sep='=')
         except IOError:
             # fallback when the cfg file is not accessible
             pass
@@ -98,14 +100,21 @@ class PulpCore(Plugin, IndependentPlugin):
                         "DJANGO_SETTINGS_MODULE": "pulpcore.app.settings"}
         self.add_cmd_output("dynaconf list", env=dynaconf_env)
         for _dir in [self.staticroot, self.uploaddir]:
-            self.add_cmd_output("ls -l %s" % _dir)
+            self.add_dir_listing(_dir)
 
         task_days = self.get_option('task-days')
         for table in ['core_task', 'core_taskgroup',
                       'core_groupprogressreport', 'core_progressreport']:
-            _query = "select * from %s where pulp_last_updated > NOW() - " \
-                     "interval '%s days' order by pulp_last_updated" % \
-                     (table, task_days)
+            _query = ("COPY (SELECT STRING_AGG(column_name, ', ') FROM "
+                      f"information_schema.columns WHERE table_name='{table}'"
+                      "AND table_schema = 'public' AND column_name NOT IN"
+                      " ('args', 'kwargs', 'enc_args', 'enc_kwargs'))"
+                      " TO STDOUT;")
+            col_out = self.exec_cmd(self.build_query_cmd(_query), env=self.env)
+            columns = col_out['output'] if col_out['status'] == 0 else '*'
+            _query = (f"select {columns} from {table} where pulp_last_updated"
+                      f"> NOW() - interval '{task_days} days' order by"
+                      " pulp_last_updated")
             _cmd = self.build_query_cmd(_query)
             self.add_cmd_output(_cmd, env=self.env, suggest_filename=table)
 
@@ -137,33 +146,24 @@ class PulpCore(Plugin, IndependentPlugin):
         a large amount of quoting in sos logs referencing the command being run
         """
         if csv:
-            query = "COPY (%s) TO STDOUT " \
-                    "WITH (FORMAT 'csv', DELIMITER ',', HEADER)" % query
+            query = f"COPY ({query}) TO STDOUT " \
+                    "WITH (FORMAT 'csv', DELIMITER ',', HEADER)"
         _dbcmd = "psql --no-password -h %s -p %s -U pulp -d %s -c %s"
         return _dbcmd % (self.dbhost, self.dbport, self.dbname, quote(query))
 
     def postproc(self):
-        # TODO obfuscate from /etc/pulp/settings.py :
+        # obfuscate from /etc/pulp/settings.py and "dynaconf list":
         # SECRET_KEY = "eKfeDkTnvss7p5WFqYdGPWxXfHnsbDBx"
         # 'PASSWORD': 'tGrag2DmtLqKLTWTQ6U68f6MAhbqZVQj',
-        self.do_path_regex_sub(
-            "/etc/pulp/settings.py",
-            r"(SECRET_KEY\s*=\s*)(.*)",
-            r"\1********")
-        self.do_path_regex_sub(
-            "/etc/pulp/settings.py",
-            r"(PASSWORD\S*\s*:\s*)(.*)",
-            r"\1********")
-        # apply the same for "dynaconf list" output that prints settings.py
-        # in a pythonic format
-        self.do_cmd_output_sub(
-            "dynaconf list",
-            r"(SECRET_KEY<str>\s*)'(.*)'",
-            r"\1********")
-        self.do_cmd_output_sub(
-            "dynaconf list",
-            r"(PASSWORD\S*\s*:\s*)(.*)",
-            r"\1********")
+        # AUTH_LDAP_BIND_PASSWORD = 'ouch-a-secret'
+        # the PASSWORD can be also in an one-liner list, so detect its value
+        # in non-greedy manner till first ',' or '}'
+        key_pass_re = r"((?:SECRET_KEY|AUTH_LDAP_BIND_PASSWORD)" \
+                      r"(?:\<.+\>)?(\s*=)?|(password|PASSWORD)" \
+                      r"(\"|'|:)+)\s*(\S*)"
+        repl = r"\1 ********"
+        self.do_path_regex_sub("/etc/pulp/settings.py", key_pass_re, repl)
+        self.do_cmd_output_sub("dynaconf list", key_pass_re, repl)
 
 
 # vim: set et ts=4 sw=4 :
