@@ -22,13 +22,14 @@ class Rpmostree(Plugin, RedHatPlugin):
         self.add_copy_spec('/etc/ostree/remotes.d/')
 
         subcmds = [
-            'status',
+            'status --json',
+            'kargs',
             'db list',
             'db diff',
             '--version'
         ]
 
-        self.add_cmd_output(["rpm-ostree %s" % subcmd for subcmd in subcmds])
+        self.add_cmd_output([f"rpm-ostree {subcmd}" for subcmd in subcmds])
 
         units = [
             'rpm-ostreed',
