@@ -6,7 +6,7 @@
 #
 # See the LICENSE file in the source distribution for further information.
 
-from sos.report.plugins import Plugin, UbuntuPlugin
+from sos.report.plugins import Plugin, UbuntuPlugin, SoSPredicate
 from sos.utilities import is_executable
 
 
@@ -26,15 +26,25 @@ class Ubuntu(Plugin, UbuntuPlugin):
         if self.is_installed('ubuntu-advantage-tools'):
             if is_executable('ua'):
                 ua_tools_status = 'ua status'
+            elif is_executable('pro'):
+                ua_tools_status = 'pro status'
             else:
                 ua_tools_status = 'ubuntu-advantage status'
-            self.add_cmd_output(ua_tools_status)
+            ua_pred = SoSPredicate(self, kmods=['tls'])
+            self.add_cmd_output(ua_tools_status,
+                                pred=ua_pred, changes=True)
+            self.add_cmd_output(f"{ua_tools_status} --format json",
+                                pred=ua_pred, changes=True)
 
             if not self.get_option("all_logs"):
                 self.add_copy_spec([
                     "/var/log/ubuntu-advantage.log",
                     "/var/log/ubuntu-advantage.log.1",
                     "/var/log/ubuntu-advantage.log.2*",
+                    "/var/log/ubuntu-advantage-timer.log",
+                    "/var/log/ubuntu-advantage-timer.log.1",
+                    "/var/log/ubuntu-advantage-timer.log.2*",
                 ])
             else:
                 self.add_copy_spec("/var/log/ubuntu-advantage.log*")
+                self.add_copy_spec("/var/log/ubuntu-advantage-timer.log*")
