@@ -8,11 +8,10 @@
 #
 # See the LICENSE file in the source distribution for further information.
 
-from sos.report.plugins import Plugin, RedHatPlugin
-
 import glob
 import json
 import re
+from sos.report.plugins import Plugin, RedHatPlugin
 
 
 # This configuration is based on vdsm.storage.lvm.LVM_CONF_TEMPLATE.
@@ -95,7 +94,7 @@ class Vdsm(Plugin, RedHatPlugin):
         if qemu_pids:
             files = ["cmdline", "status", "mountstats"]
             self.add_copy_spec([
-                "/proc/%s/%s" % (pid, name)
+                f"/proc/{pid}/{name}"
                 for pid in qemu_pids
                 for name in files
             ])
@@ -105,9 +104,9 @@ class Vdsm(Plugin, RedHatPlugin):
             "su vdsm -s /bin/sh -c 'ls -lR /rhev/data-center'"
         ])
         self.add_cmd_output([
-            "lvm vgs -v -o +tags --config \'%s\'" % LVM_CONFIG,
-            "lvm lvs -v -o +tags --config \'%s\'" % LVM_CONFIG,
-            "lvm pvs -v -o +all --config \'%s\'" % LVM_CONFIG
+            f"lvm vgs -v -o +tags --config \'{LVM_CONFIG}\'",
+            f"lvm lvs -v -o +tags --config \'{LVM_CONFIG}\'",
+            f"lvm pvs -v -o +all --config \'{LVM_CONFIG}\'"
         ])
 
         self.add_cmd_output([
@@ -129,12 +128,12 @@ class Vdsm(Plugin, RedHatPlugin):
                 pools = json.loads(res['output'])
                 for pool in pools:
                     self.add_cmd_output(
-                        'vdsm-client StoragePool getSpmStatus'
-                        ' storagepoolID={}'.format(pool)
+                        f"vdsm-client StoragePool getSpmStatus "
+                        f"storagepoolID={pool}"
                     )
-        except ValueError as e:
+        except ValueError as err:
             self._log_error(
-                'vdsm-client Host getConnectedStoragePools: %s' % (e)
+                f'vdsm-client Host getConnectedStoragePools: {err}'
             )
 
         try:
@@ -145,9 +144,9 @@ class Vdsm(Plugin, RedHatPlugin):
                 self.add_cmd_output([
                     dump_volume_chains_cmd % uuid for uuid in sd_uuids
                 ])
-        except ValueError as e:
+        except ValueError as err:
             self._log_error(
-                'vdsm-client Host getStorageDomains: %s' % (e)
+                f'vdsm-client Host getStorageDomains: {err}'
             )
 
     def _add_vdsm_forbidden_paths(self):
