@@ -7,6 +7,7 @@
 # See the LICENSE file in the source distribution for further information.
 
 import os
+import pwd
 import re
 import inspect
 from subprocess import Popen, PIPE, STDOUT
@@ -19,12 +20,85 @@ import tempfile
 import threading
 import time
 import io
-
 from contextlib import closing
 from collections import deque
 
+try:
+    from packaging.version import parse as parse_version
+except ImportError:
+    from pkg_resources import parse_version
+
+# try loading magic>=0.4.20 which implements detect_from_filename method
+magic_mod = False
+try:
+    import magic
+    magic.detect_from_filename(__file__)
+    magic_mod = True
+except (ImportError, AttributeError):
+    log = logging.getLogger('sos')
+    from textwrap import fill
+    msg = ("""\
+WARNING: Failed to load 'magic' module version >= 0.4.20 which sos aims to \
+use for detecting binary files. A less effective method will be used. It is \
+recommended to install proper python3-magic package with the module.
+""")
+    log.warning('\n' + fill(msg, 72, replace_whitespace=False) + '\n')
+
+
 TIMEOUT_DEFAULT = 300
 
+__all__ = [
+    'TIMEOUT_DEFAULT',
+    'ImporterHelper',
+    'SoSTimeoutError',
+    'TempFileUtil',
+    'bold',
+    'file_is_binary',
+    'fileobj',
+    'find',
+    'get_human_readable',
+    'grep',
+    'import_module',
+    'is_executable',
+    'listdir',
+    'parse_version',
+    'path_exists',
+    'path_isdir',
+    'path_isfile',
+    'path_islink',
+    'path_join',
+    'recursive_dict_values_by_key',
+    'shell_out',
+    'sos_get_command_output',
+    'tail',
+]
+
+
+def format_version_to_pep440(ver):
+    """ Convert the version into a PEP440 compliant version scheme."""
+    public_version_re = re.compile(
+            r"^([0-9][0-9.]*(?:(?:a|b|rc|.post|.dev)[0-9]+)*)\+?"
+            )
+    try:
+        _, public, local = public_version_re.split(ver, maxsplit=1)
+        if not local:
+            return ver
+        sanitized_local = re.sub("[+~]+", ".", local).strip("-")
+        pep440_version = f"{public}+{sanitized_local}"
+        return pep440_version
+    except Exception as err:
+        log.debug(f"Unable to format {ver} to pep440 format: {err}")
+        return ver
+
+
+def sos_parse_version(ver, pep440=True):
+    """ Converts the version to PEP440 format before parsing """
+    if pep440:
+        ver_pep440 = format_version_to_pep440(ver)
+        return parse_version(ver_pep440)
+
+    return parse_version(ver)
+
 
 def tail(filename, number_of_bytes):
     """Returns the last number_of_bytes of filename"""
@@ -41,7 +115,7 @@ def fileobj(path_or_file, mode='r'):
             return open(path_or_file, mode)
         except IOError:
             log = logging.getLogger('sos')
-            log.debug("fileobj: %s could not be opened" % path_or_file)
+            log.debug(f"fileobj: {path_or_file} could not be opened")
             return closing(io.StringIO())
     else:
         return closing(path_or_file)
@@ -51,15 +125,49 @@ def convert_bytes(bytes_, K=1 << 10, M=1 << 20, G=1 << 30, T=1 << 40):
     """Converts a number of bytes to a shorter, more human friendly format"""
     fn = float(bytes_)
     if bytes_ >= T:
-        return '%.1fT' % (fn / T)
+        return f'{(fn / T):.1fT}'
     elif bytes_ >= G:
-        return '%.1fG' % (fn / G)
+        return f'{(fn / G):.1fG}'
     elif bytes_ >= M:
-        return '%.1fM' % (fn / M)
+        return f'{(fn / M):.1fM}'
     elif bytes_ >= K:
-        return '%.1fK' % (fn / K)
+        return f'{(fn / K):.1fK}'
     else:
-        return '%d' % bytes_
+        return f'{bytes_}'
+
+
+def file_is_binary(fname):
+    """Helper to determine if a given file contains binary content or not.
+
+    This is especially helpful for `sos clean`, which cannot obfuscate binary
+    data and instead, by default, will remove binary files.
+
+    :param fname:   The full path of the file to check binaryness of
+    :type fname:    ``str``
+
+    :returns:   True if binary, else False
+    :rtype:     ``bool``
+    """
+    if magic_mod:
+        try:
+            _ftup = magic.detect_from_filename(fname)
+            _mimes = ['text/', 'inode/']
+            return (
+                _ftup.encoding == 'binary' and not
+                any(_ftup.mime_type.startswith(_mt) for _mt in _mimes)
+            )
+        except Exception:
+            pass
+    # if for some reason the above check fails or magic>=0.4.20 is not present,
+    # fail over to checking the very first byte of the file content
+    with open(fname, 'tr') as tfile:
+        try:
+            # when opened as above (tr), reading binary content will raise
+            # an exception
+            tfile.read(1)
+            return False
+        except UnicodeDecodeError:
+            return True
 
 
 def find(file_pattern, top_dir, max_depth=None, path_pattern=None):
@@ -111,7 +219,8 @@ def is_executable(command, sysroot=None):
 def sos_get_command_output(command, timeout=TIMEOUT_DEFAULT, stderr=False,
                            chroot=None, chdir=None, env=None, foreground=False,
                            binary=False, sizelimit=None, poller=None,
-                           to_file=False):
+                           to_file=False, runas=None):
+    # pylint: disable=too-many-locals,too-many-branches
     """Execute a command and return a dictionary of status and output,
     optionally changing root or current working directory before
     executing command.
@@ -122,18 +231,31 @@ def sos_get_command_output(command, timeout=TIMEOUT_DEFAULT, stderr=False,
     def _child_prep_fn():
         if chroot and chroot != '/':
             os.chroot(chroot)
+        if runas:
+            os.setgid(pwd.getpwnam(runas).pw_gid)
+            os.setuid(pwd.getpwnam(runas).pw_uid)
+            os.chdir(pwd.getpwnam(runas).pw_dir)
         if (chdir):
             os.chdir(chdir)
 
     def _check_poller(proc):
-        if poller():
+        if poller() or proc.poll() == 124:
             proc.terminate()
             raise SoSTimeoutError
         time.sleep(0.01)
 
+    if runas:
+        pwd_user = pwd.getpwnam(runas)
+        env.update({
+            'HOME': pwd_user.pw_dir,
+            'LOGNAME': runas,
+            'PWD': pwd_user.pw_dir,
+            'USER': runas
+        })
+
     cmd_env = os.environ.copy()
     # ensure consistent locale for collected command output
-    cmd_env['LC_ALL'] = 'C'
+    cmd_env['LC_ALL'] = 'C.UTF-8'
     # optionally add an environment change for the command
     if env:
         for key, value in env.items():
@@ -143,11 +265,8 @@ def sos_get_command_output(command, timeout=TIMEOUT_DEFAULT, stderr=False,
                 cmd_env.pop(key, None)
     # use /usr/bin/timeout to implement a timeout
     if timeout and is_executable("timeout"):
-        command = "timeout %s %ds %s" % (
-            '--foreground' if foreground else '',
-            timeout,
-            command
-        )
+        command = (f"timeout {'--foreground' if foreground else ''} {timeout}s"
+                   f" {command}")
 
     args = shlex.split(command)
     # Expand arguments that are wildcard root paths.
@@ -186,9 +305,11 @@ def sos_get_command_output(command, timeout=TIMEOUT_DEFAULT, stderr=False,
                 p.wait(timeout if timeout else None)
             except Exception:
                 p.terminate()
-                _output.close()
+                if to_file:
+                    _output.close()
                 # until we separate timeouts from the `timeout` command
                 # handle per-cmd timeouts via Plugin status checks
+                reader.running = False
                 return {'status': 124, 'output': reader.get_contents(),
                         'truncated': reader.is_full}
         if to_file:
@@ -202,6 +323,8 @@ def sos_get_command_output(command, timeout=TIMEOUT_DEFAULT, stderr=False,
         truncated = reader.is_full
 
     except OSError as e:
+        if to_file:
+            _output.close()
         if e.errno == errno.ENOENT:
             return {'status': 127, 'output': "", 'truncated': ''}
         else:
@@ -223,7 +346,12 @@ def import_module(module_fqname, superclasses=None):
     be subclasses of the specified superclass or superclasses. If superclasses
     is plural it must be a tuple of classes."""
     module_name = module_fqname.rpartition(".")[-1]
-    module = __import__(module_fqname, globals(), locals(), [module_name])
+    try:
+        module = __import__(module_fqname, globals(), locals(), [module_name])
+    except ImportError as e:
+        print(f'Error while trying to load module {module_fqname}: '
+              f' {e.__class__.__name__}')
+        raise e
     modules = [class_ for cname, class_ in
                inspect.getmembers(module, inspect.isclass)
                if class_.__module__ == module_fqname]
@@ -248,7 +376,7 @@ def get_human_readable(size, precision=2):
     while size > 1024 and suffixindex < 4:
         suffixindex += 1
         size = size/1024.0
-    return "%.*f%s" % (precision, size, suffixes[suffixindex])
+    return f"{size:.{precision}f}{suffixes[suffixindex]}"
 
 
 def _os_wrapper(path, sysroot, method, module=os.path):
@@ -260,6 +388,8 @@ def _os_wrapper(path, sysroot, method, module=os.path):
 
 
 def path_exists(path, sysroot):
+    if '*' in path:
+        return _os_wrapper(path, sysroot, 'glob', module=glob)
     return _os_wrapper(path, sysroot, 'exists')
 
 
@@ -298,6 +428,60 @@ def bold(text):
     return '\033[1m' + text + '\033[0m'
 
 
+def recursive_dict_values_by_key(dobj, keys=[]):
+    """Recursively compile all elements of a potentially nested dict by a set
+    of keys. If a given key is a dict within ``dobj``, then _all_ elements
+    within that dict, regardless of child keys, will be returned.
+
+    For example, if a Plugin searches the devices dict for the 'storage' key,
+    then all storage devices under the that dict (e.g. block, fibre, etc...)
+    will be returned. However, if the Plugin specifies 'block' via ``keys``,
+    then only the block devices within the devices['storage'] dict will be
+    returned.
+
+    Any elements passed here that are _not_ keys within the dict or any nested
+    dicts will also be returned.
+
+    :param dobj:    The 'top-level' dict to intially search by
+    :type dobj:     ``dict``
+
+    :param keys:    Which keys to compile elements from within ``dobj``. If no
+                    keys are given, all nested elements are returned
+    :param keys:    ``list`` of ``str``
+
+    :returns:       All elements within the dict and any nested dicts
+    :rtype:         ``list``
+    """
+    _items = []
+    _filt = []
+    _items.extend(keys)
+    if isinstance(dobj, dict):
+        for k, v in dobj.items():
+            _filt.append(k)
+            # get everything below this key, including nested dicts
+            if not keys or k in keys:
+                _items.extend(recursive_dict_values_by_key(v))
+            # recurse into this dict only for dict keys that match what
+            # we're looking for
+            elif isinstance(v, dict):
+                try:
+                    # this will return a nested list, extract it
+                    _items.extend(
+                        recursive_dict_values_by_key(
+                            v[key] for key in keys if key in v
+                        )[0]
+                    )
+                except IndexError:
+                    # none of the keys given exist in the nested dict
+                    pass
+                _filt.extend(v.keys())
+
+    else:
+        _items.extend(dobj)
+
+    return [d for d in _items if d not in _filt]
+
+
 class FakeReader():
     """Used as a replacement AsyncReader for when we are writing directly to
     disk, and allows us to keep more simplified flows for executing,
@@ -329,7 +513,7 @@ class AsyncReader(threading.Thread):
     """
 
     def __init__(self, channel, sizelimit, binary):
-        super(AsyncReader, self).__init__()
+        super().__init__()
         self.chan = channel
         self.binary = binary
         self.chunksize = 2048
@@ -412,8 +596,7 @@ class ImporterHelper(object):
             pnames = self._get_plugins_from_list(py_files)
             if pnames:
                 return pnames
-            else:
-                return []
+        return []
 
     def get_modules(self):
         """Returns the list of importable modules in the configured python
@@ -445,10 +628,12 @@ class TempFileUtil():
                 f.flush()
                 f.close()
             except Exception:
+                # file already closed or potentially already removed, ignore
                 pass
             try:
                 os.unlink(fname)
             except Exception:
+                # if the above failed, this is also likely to fail, ignore
                 pass
         self.files = []
 
