@@ -11,6 +11,9 @@ from uaclient.exceptions import (
     IncompatibleServiceStopsEnable,
     InvalidProImage,
     LockHeldError,
+    MagicAttachTokenAlreadyActivated,
+    MagicAttachTokenError,
+    MagicAttachUnavailable,
     NonAutoAttachImageError,
     NonRootUserError,
     RequiredServiceStopsEnable,
@@ -27,6 +30,9 @@ __all__ = [
     "EntitlementNotFoundError",
     "InvalidProImage",
     "LockHeldError",
+    "MagicAttachTokenAlreadyActivated",
+    "MagicAttachTokenError",
+    "MagicAttachUnavailable",
     "NonAutoAttachImageError",
     "NonRootUserError",
     "UbuntuProError",
