@@ -1,6 +1,7 @@
 import enum
 import re
 from collections import defaultdict
+from datetime import datetime, timezone
 from typing import Any, Dict, List, NamedTuple, Optional, Tuple
 
 from uaclient import apt, exceptions, messages
@@ -21,9 +22,11 @@ from uaclient.api.u.pro.security.fix._common import (
     query_installed_source_pkg_versions,
 )
 from uaclient.api.u.pro.status.enabled_services.v1 import _enabled_services
-from uaclient.api.u.pro.status.is_attached.v1 import _is_attached
+from uaclient.api.u.pro.status.is_attached.v1 import (
+    ContractExpiryStatus,
+    _is_attached,
+)
 from uaclient.config import UAConfig
-from uaclient.contract import ContractExpiryStatus, get_contract_expiry_status
 from uaclient.data_types import (
     DataObject,
     Field,
@@ -71,12 +74,30 @@ class FixPlanAttachReason(enum.Enum):
 class FixWarningType(enum.Enum):
     PACKAGE_CANNOT_BE_INSTALLED = "package-cannot-be-installed"
     SECURITY_ISSUE_NOT_FIXED = "security-issue-not-fixed"
+    FAIL_UPDATING_ESM_CACHE = "fail-updating-esm-cache"
 
 
 class FixPlanStep(DataObject):
     fields = [
-        Field("operation", StringDataValue),
-        Field("order", IntDataValue),
+        Field(
+            "operation",
+            StringDataValue,
+            doc=(
+                "The operation that would be performed to fix the issue. This"
+                " can be either an attach, enable, apt-upgrade or a no-op type"
+            ),
+        ),
+        Field(
+            "order", IntDataValue, doc="The execution order of the operation"
+        ),
+        Field(
+            "data",
+            DataObject,
+            doc=(
+                "A data object that can be either an ``AptUpgradeData``,"
+                " ``AttachData``, ``EnableData``, ``NoOpData``"
+            ),
+        ),
     ]
 
     def __init__(self, *, operation: str, order: int):
@@ -86,9 +107,21 @@ class FixPlanStep(DataObject):
 
 class AptUpgradeData(DataObject):
     fields = [
-        Field("binary_packages", data_list(StringDataValue)),
-        Field("source_packages", data_list(StringDataValue)),
-        Field("pocket", StringDataValue),
+        Field(
+            "binary_packages",
+            data_list(StringDataValue),
+            doc="A list of binary packages that need to be upgraded",
+        ),
+        Field(
+            "source_packages",
+            data_list(StringDataValue),
+            doc="A list of source packages that need to be upgraded",
+        ),
+        Field(
+            "pocket",
+            StringDataValue,
+            doc="The pocket where the packages will be installed from",
+        ),
     ]
 
     def __init__(
@@ -117,9 +150,21 @@ class FixPlanAptUpgradeStep(FixPlanStep):
 
 class AttachData(DataObject):
     fields = [
-        Field("reason", StringDataValue),
-        Field("required_service", StringDataValue),
-        Field("source_packages", data_list(StringDataValue)),
+        Field(
+            "reason",
+            StringDataValue,
+            doc="The reason why an attach operation is needed",
+        ),
+        Field(
+            "required_service",
+            StringDataValue,
+            doc="The required service that requires the attach operation",
+        ),
+        Field(
+            "source_packages",
+            data_list(StringDataValue),
+            doc="The source packages that require the attach operation",
+        ),
     ]
 
     def __init__(
@@ -144,8 +189,16 @@ class FixPlanAttachStep(FixPlanStep):
 
 class EnableData(DataObject):
     fields = [
-        Field("service", StringDataValue),
-        Field("source_packages", data_list(StringDataValue)),
+        Field(
+            "service",
+            StringDataValue,
+            doc="The service that needs to be enabled",
+        ),
+        Field(
+            "source_packages",
+            data_list(StringDataValue),
+            doc="The source packages that require the service to be enabled",
+        ),
     ]
 
     def __init__(self, *, service: str, source_packages: List[str]):
@@ -167,7 +220,11 @@ class FixPlanEnableStep(FixPlanStep):
 
 class NoOpData(DataObject):
     fields = [
-        Field("status", StringDataValue),
+        Field(
+            "status",
+            StringDataValue,
+            doc="The status of the issue when no operation can be performed",
+        ),
     ]
 
     def __init__(self, *, status: str):
@@ -188,8 +245,16 @@ class FixPlanNoOpStep(FixPlanStep):
 
 class NoOpLivepatchFixData(NoOpData):
     fields = [
-        Field("status", StringDataValue),
-        Field("patch_version", StringDataValue),
+        Field(
+            "status",
+            StringDataValue,
+            doc="The status of the CVE when no operation can be performed",
+        ),
+        Field(
+            "patch_version",
+            StringDataValue,
+            doc="Version of the patch from Livepatch that fixed the CVE",
+        ),
     ]
 
     def __init__(self, *, status: str, patch_version: str):
@@ -210,9 +275,21 @@ class FixPlanNoOpLivepatchFixStep(FixPlanNoOpStep):
 
 class NoOpAlreadyFixedData(NoOpData):
     fields = [
-        Field("status", StringDataValue),
-        Field("source_packages", data_list(StringDataValue)),
-        Field("pocket", StringDataValue),
+        Field(
+            "status",
+            StringDataValue,
+            doc="The status of the issue when no operation can be performed",
+        ),
+        Field(
+            "source_packages",
+            data_list(StringDataValue),
+            doc="The source packages that are already fixed",
+        ),
+        Field(
+            "pocket",
+            StringDataValue,
+            doc="The pocket where the packages would have been installed from",
+        ),
     ]
 
     def __init__(
@@ -236,8 +313,17 @@ class FixPlanNoOpAlreadyFixedStep(FixPlanNoOpStep):
 
 class FixPlanWarning(DataObject):
     fields = [
-        Field("warning_type", StringDataValue),
-        Field("order", IntDataValue),
+        Field("warning_type", StringDataValue, doc="The type of warning"),
+        Field(
+            "order", IntDataValue, doc="The execution order of the operation"
+        ),
+        Field(
+            "data",
+            DataObject,
+            doc="A data object that represents either a"
+            " ``PackageCannotBeInstalledData`` or a"
+            " ``SecurityIssueNotFixedData``",
+        ),
     ]
 
     def __init__(self, *, warning_type: str, order: int):
@@ -247,8 +333,16 @@ class FixPlanWarning(DataObject):
 
 class SecurityIssueNotFixedData(DataObject):
     fields = [
-        Field("source_packages", data_list(StringDataValue)),
-        Field("status", StringDataValue),
+        Field(
+            "source_packages",
+            data_list(StringDataValue),
+            doc="A list of source packages that cannot be fixed at the moment",
+        ),
+        Field(
+            "status",
+            StringDataValue,
+            doc="The status of the CVE regarding those packages",
+        ),
     ]
 
     def __init__(self, *, source_packages: List[str], status: str):
@@ -273,11 +367,37 @@ class FixPlanWarningSecurityIssueNotFixed(FixPlanWarning):
 
 class PackageCannotBeInstalledData(DataObject):
     fields = [
-        Field("binary_package", StringDataValue),
-        Field("binary_package_version", StringDataValue),
-        Field("source_package", StringDataValue),
-        Field("related_source_packages", data_list(StringDataValue)),
-        Field("pocket", StringDataValue),
+        Field(
+            "binary_package",
+            StringDataValue,
+            doc="The binary package that cannot be installed",
+        ),
+        Field(
+            "binary_package_version",
+            StringDataValue,
+            doc="The version of the binary package that cannot be installed",
+        ),
+        Field(
+            "source_package",
+            StringDataValue,
+            doc="The source package associated with the binary package",
+        ),
+        Field(
+            "related_source_packages",
+            data_list(StringDataValue),
+            doc=(
+                "A list of source packages that come from the same pocket as"
+                " the affected package"
+            ),
+        ),
+        Field(
+            "pocket",
+            StringDataValue,
+            doc=(
+                "The pocket where the affected package should be installed"
+                " from"
+            ),
+        ),
     ]
 
     def __init__(
@@ -311,10 +431,36 @@ class FixPlanWarningPackageCannotBeInstalled(FixPlanWarning):
         self.data = data
 
 
+class FailUpdatingESMCacheData(DataObject):
+    fields = [
+        Field("title", StringDataValue),
+        Field("code", StringDataValue),
+    ]
+
+    def __init__(self, *, title: str, code: str):
+        self.title = title
+        self.code = code
+
+
+class FixPlanWarningFailUpdatingESMCache(FixPlanWarning):
+    fields = [
+        Field("warning_type", StringDataValue),
+        Field("order", IntDataValue),
+        Field("data", FailUpdatingESMCacheData),
+    ]
+
+    def __init__(self, *, order: int, data: FailUpdatingESMCacheData):
+        super().__init__(
+            warning_type=FixWarningType.FAIL_UPDATING_ESM_CACHE.value,
+            order=order,
+        )
+        self.data = data
+
+
 class FixPlanError(DataObject):
     fields = [
-        Field("msg", StringDataValue),
-        Field("code", StringDataValue, required=False),
+        Field("msg", StringDataValue, doc="The error message"),
+        Field("code", StringDataValue, required=False, doc="The message code"),
     ]
 
     def __init__(self, *, msg: str, code: Optional[str]):
@@ -329,8 +475,16 @@ class AdditionalData(DataObject):
 class USNAdditionalData(AdditionalData):
 
     fields = [
-        Field("associated_cves", data_list(StringDataValue)),
-        Field("associated_launchpad_bugs", data_list(StringDataValue)),
+        Field(
+            "associated_cves",
+            data_list(StringDataValue),
+            doc="The associated CVEs for the USN",
+        ),
+        Field(
+            "associated_launchpad_bugs",
+            data_list(StringDataValue),
+            doc="The associated Launchpad bugs for the USN",
+        ),
     ]
 
     def __init__(
@@ -345,14 +499,53 @@ class USNAdditionalData(AdditionalData):
 
 class FixPlanResult(DataObject):
     fields = [
-        Field("title", StringDataValue),
-        Field("description", StringDataValue, required=False),
-        Field("expected_status", StringDataValue),
-        Field("affected_packages", data_list(StringDataValue), required=False),
-        Field("plan", data_list(FixPlanStep)),
-        Field("warnings", data_list(FixPlanWarning), required=False),
-        Field("error", FixPlanError, required=False),
-        Field("additional_data", AdditionalData, required=False),
+        Field("title", StringDataValue, doc="The title of the issue"),
+        Field(
+            "description",
+            StringDataValue,
+            required=False,
+            doc="The description of the issue",
+        ),
+        Field(
+            "current_status",
+            StringDataValue,
+            required=False,
+            doc="The current status of the issue on the system",
+        ),
+        Field(
+            "expected_status",
+            StringDataValue,
+            doc="The expected status of fixing the issue",
+        ),
+        Field(
+            "affected_packages",
+            data_list(StringDataValue),
+            required=False,
+            doc="A list of package names affected by the issue",
+        ),
+        Field(
+            "plan",
+            data_list(FixPlanStep),
+            doc="A list of ``FixPlanStep`` objects",
+        ),
+        Field(
+            "warnings",
+            data_list(FixPlanWarning),
+            required=False,
+            doc="A list of ``FixPlanWarning`` objects",
+        ),
+        Field(
+            "error",
+            FixPlanError,
+            required=False,
+            doc="A ``FixPlanError`` object, if an error occurred.",
+        ),
+        Field(
+            "additional_data",
+            AdditionalData,
+            required=False,
+            doc="Additional data for the issue",
+        ),
     ]
 
     def __init__(
@@ -365,10 +558,12 @@ class FixPlanResult(DataObject):
         error: Optional[FixPlanError],
         additional_data: AdditionalData,
         description: Optional[str] = None,
-        affected_packages: Optional[List[str]] = None
+        affected_packages: Optional[List[str]] = None,
+        current_status: Optional[str] = None
     ):
         self.title = title
         self.description = description
+        self.current_status = current_status
         self.expected_status = expected_status
         self.affected_packages = affected_packages
         self.plan = plan
@@ -379,8 +574,17 @@ class FixPlanResult(DataObject):
 
 class FixPlanUSNResult(DataObject):
     fields = [
-        Field("target_usn_plan", FixPlanResult),
-        Field("related_usns_plan", data_list(FixPlanResult), required=False),
+        Field(
+            "target_usn_plan",
+            FixPlanResult,
+            doc="A ``FixPlanResult`` object for the target USN",
+        ),
+        Field(
+            "related_usns_plan",
+            data_list(FixPlanResult),
+            required=False,
+            doc="A list of ``FixPlanResult`` objects for the related USNs",
+        ),
     ]
 
     def __init__(
@@ -399,11 +603,13 @@ class FixPlan:
         title: str,
         description: Optional[str],
         affected_packages: Optional[List[str]] = None,
+        current_status: Optional[str] = None,
     ):
         self.order = 1
         self.title = title
         self.description = description
         self.affected_packages = affected_packages
+        self.current_status = current_status
         self.fix_steps = []  # type: List[FixPlanStep]
         self.fix_warnings = []  # type: List[FixPlanWarning]
         self.error = None  # type: Optional[FixPlanError]
@@ -456,11 +662,16 @@ class FixPlan:
                 order=self.order,
                 data=SecurityIssueNotFixedData.from_dict(data),
             )
-        else:
+        elif warning_type == FixWarningType.PACKAGE_CANNOT_BE_INSTALLED:
             fix_warning = FixPlanWarningPackageCannotBeInstalled(
                 order=self.order,
                 data=PackageCannotBeInstalledData.from_dict(data),
             )
+        else:
+            fix_warning = FixPlanWarningFailUpdatingESMCache(
+                order=self.order,
+                data=FailUpdatingESMCacheData.from_dict(data),
+            )
 
         self.fix_warnings.append(fix_warning)
         self.order += 1
@@ -471,7 +682,7 @@ class FixPlan:
     def register_additional_data(self, additional_data: Dict[str, Any]):
         self.additional_data = AdditionalData(**additional_data)
 
-    def _get_status(self) -> str:
+    def _get_expected_status(self) -> str:
         if self.error:
             return "error"
 
@@ -480,18 +691,19 @@ class FixPlan:
             and isinstance(self.fix_steps[0], FixPlanNoOpStep)
             and self.fix_steps[0].data.status == "system-not-affected"
         ):
-            return str(FixStatus.SYSTEM_NOT_AFFECTED)
+            return FixStatus.SYSTEM_NOT_AFFECTED.value.msg
         elif self.fix_warnings:
-            return str(FixStatus.SYSTEM_STILL_VULNERABLE)
+            return FixStatus.SYSTEM_STILL_VULNERABLE.value.msg
         else:
-            return str(FixStatus.SYSTEM_NON_VULNERABLE)
+            return FixStatus.SYSTEM_NON_VULNERABLE.value.msg
 
     @property
     def fix_plan(self):
         return FixPlanResult(
             title=self.title,
             description=self.description,
-            expected_status=self._get_status(),
+            expected_status=self._get_expected_status(),
+            current_status=self.current_status,
             affected_packages=self.affected_packages,
             plan=self.fix_steps,
             warnings=self.fix_warnings,
@@ -530,7 +742,7 @@ def _get_cve_data(
 ) -> Tuple[CVE, List[USN]]:
     try:
         cve = client.get_cve(cve_id=issue_id)
-        usns = client.get_notices(details=issue_id)
+        usns = client.get_notices(cves=issue_id)
     except exceptions.SecurityAPIError as e:
         if e.code == 404:
             raise exceptions.SecurityIssueNotFound(issue_id=issue_id)
@@ -564,15 +776,12 @@ def _get_usn_data(
 
 def _get_upgradable_pkgs(
     binary_pkgs: List[BinaryPackageFix],
-    pocket: str,
+    check_esm_cache: bool,
 ) -> Tuple[List[str], List[UnfixedPackage]]:
     upgrade_pkgs = []
     unfixed_pkgs = []
 
     for binary_pkg in sorted(binary_pkgs):
-        check_esm_cache = (
-            pocket != messages.SECURITY_UBUNTU_STANDARD_UPDATES_POCKET
-        )
         candidate_version = apt.get_pkg_candidate_version(
             binary_pkg.binary_pkg, check_esm_cache=check_esm_cache
         )
@@ -649,6 +858,7 @@ def _fix_plan_cve(issue_id: str, cfg: UAConfig) -> FixPlanResult:
 
     if livepatch_cve_status:
         fix_plan = get_fix_plan(title=issue_id)
+        fix_plan.current_status = FixStatus.SYSTEM_NON_VULNERABLE.value.msg
         fix_plan.register_step(
             operation=FixStepType.NOOP,
             data={
@@ -715,9 +925,9 @@ def _fix_plan_usn(issue_id: str, cfg: UAConfig) -> FixPlanUSNResult:
     )
     additional_data = {
         "associated_cves": [] if not usn.cves_ids else usn.cves_ids,
-        "associated_launchpad_bugs": []
-        if not usn.references
-        else usn.references,
+        "associated_launchpad_bugs": (
+            [] if not usn.references else usn.references
+        ),
     }
 
     target_usn_plan = _generate_fix_plan(
@@ -740,9 +950,9 @@ def _fix_plan_usn(issue_id: str, cfg: UAConfig) -> FixPlanUSNResult:
         )
         additional_data = {
             "associated_cves": [] if not usn.cves_ids else usn.cves_ids,
-            "associated_launchpad_bugs": []
-            if not usn.references
-            else usn.references,
+            "associated_launchpad_bugs": (
+                [] if not usn.references else usn.references
+            ),
         }
 
         related_usns_plan.append(
@@ -799,6 +1009,28 @@ def get_pocket_short_name(pocket: str):
         return pocket
 
 
+def _should_update_esm_cache(
+    check_esm_cache: bool,
+    esm_cache_updated: bool,
+    cfg: UAConfig,
+) -> bool:
+    if (
+        check_esm_cache
+        and not esm_cache_updated
+        and not _is_attached(cfg).is_attached
+    ):
+        last_apt_update = apt.get_apt_cache_datetime()
+        if last_apt_update is None:
+            return True
+
+        now = datetime.now(timezone.utc)
+        time_since_update = now - last_apt_update
+        if time_since_update.days > 2:
+            return True
+
+    return False
+
+
 def _generate_fix_plan(
     *,
     issue_id: str,
@@ -811,6 +1043,7 @@ def _generate_fix_plan(
 ) -> FixPlanResult:
     count = len(affected_pkg_status)
     src_pocket_pkgs = defaultdict(list)
+    esm_cache_updated = False
 
     fix_plan = get_fix_plan(
         title=issue_id,
@@ -822,11 +1055,14 @@ def _generate_fix_plan(
         fix_plan.register_additional_data(additional_data)
 
     if count == 0:
+        fix_plan.current_status = FixStatus.SYSTEM_NOT_AFFECTED.value.msg
         fix_plan.register_step(
             operation=FixStepType.NOOP,
             data={"status": FixPlanNoOpStatus.NOT_AFFECTED.value},
         )
         return fix_plan.fix_plan
+    else:
+        fix_plan.current_status = FixStatus.SYSTEM_STILL_VULNERABLE.value.msg
 
     pkg_status_groups = group_by_usn_package_status(
         affected_pkg_status, usn_released_pkgs
@@ -843,6 +1079,9 @@ def _generate_fix_plan(
                     "status": status_value,
                 },
             )
+            fix_plan.current_status = (
+                FixStatus.SYSTEM_STILL_VULNERABLE.value.msg
+            )
         else:
             (
                 src_pocket_pkgs,
@@ -868,6 +1107,9 @@ def _generate_fix_plan(
 
         if not binary_pkgs:
             if source_pkgs:
+                fix_plan.current_status = (
+                    FixStatus.SYSTEM_NON_VULNERABLE.value.msg
+                )
                 fix_plan.register_step(
                     operation=FixStepType.NOOP,
                     data={
@@ -878,9 +1120,35 @@ def _generate_fix_plan(
                 )
             continue
 
-        upgrade_pkgs, unfixed_pkgs = _get_upgradable_pkgs(binary_pkgs, pocket)
+        check_esm_cache = (
+            pocket != messages.SECURITY_UBUNTU_STANDARD_UPDATES_POCKET
+        )
+
+        if _should_update_esm_cache(check_esm_cache, esm_cache_updated, cfg):
+            try:
+                apt.update_esm_caches(cfg)
+                esm_cache_updated = True
+            except Exception as e:
+                error_msg = messages.E_UPDATING_ESM_CACHE.format(
+                    error=getattr(e, "msg", str(e))
+                )
+
+                fix_plan.register_warning(
+                    warning_type=FixWarningType.FAIL_UPDATING_ESM_CACHE,
+                    data={
+                        "title": error_msg.msg,
+                        "code": error_msg.name,
+                    },
+                )
+
+        upgrade_pkgs, unfixed_pkgs = _get_upgradable_pkgs(
+            binary_pkgs, check_esm_cache
+        )
 
         if unfixed_pkgs:
+            fix_plan.current_status = (
+                FixStatus.SYSTEM_STILL_VULNERABLE.value.msg
+            )
             for unfixed_pkg in unfixed_pkgs:
                 fix_plan.register_warning(
                     warning_type=FixWarningType.PACKAGE_CANNOT_BE_INSTALLED,
@@ -909,8 +1177,8 @@ def _generate_fix_plan(
                     },
                 )
             else:
-                contract_expiry_status, _ = get_contract_expiry_status(cfg)
-                if contract_expiry_status != ContractExpiryStatus.ACTIVE:
+                contract_expiry_status = _is_attached(cfg).contract_status
+                if contract_expiry_status != ContractExpiryStatus.ACTIVE.value:
                     fix_plan.register_step(
                         operation=FixStepType.ATTACH,
                         data={
