@@ -11,7 +11,7 @@ from uaclient import (
 )
 from uaclient.config import UAConfig
 from uaclient.defaults import ATTACH_FAIL_DATE_FORMAT
-from uaclient.status import UserFacingStatus
+from uaclient.entitlements.entitlement_status import UserFacingStatus
 
 API_V1_CONTEXT_MACHINE_TOKEN = "/v1/context/machines/token"
 API_V1_TMPL_CONTEXT_MACHINE_TOKEN_RESOURCE = (
@@ -61,14 +61,8 @@ class UAContractClient(serviceclient.UAServiceClient):
 
     def request_resources(self) -> Dict[str, Any]:
         """Requests list of entitlements available to this machine type."""
-        platform = util.get_platform_info()
-        query_params = {
-            "architecture": platform["arch"],
-            "series": platform["series"],
-            "kernel": platform["kernel"],
-        }
         resource_response, headers = self.request_url(
-            API_V1_RESOURCES, query_params=query_params
+            API_V1_RESOURCES, query_params=self._get_platform_basic_info()
         )
         return resource_response
 
@@ -177,6 +171,40 @@ class UAContractClient(serviceclient.UAServiceClient):
             machine_token["activityInfo"] = response
             self.cfg.write_cache("machine-token", machine_token)
 
+    def get_updated_contract_info(
+        self,
+        machine_token: str,
+        contract_id: str,
+        machine_id: Optional[str] = None,
+    ) -> Dict[str, Any]:
+        """Get the updated machine token from the contract server.
+
+        @param machine_token: The machine token needed to talk to
+            this contract service endpoint.
+        @param contract_id: Unique contract id provided by contract service
+        @param machine_id: Optional unique system machine id. When absent,
+            contents of /etc/machine-id will be used.
+        """
+        if not machine_id:
+            machine_id = self._get_platform_data(machine_id).get(
+                "machineId", None
+            )
+        headers = self.headers()
+        headers.update({"Authorization": "Bearer {}".format(machine_token)})
+        url = API_V1_TMPL_CONTEXT_MACHINE_TOKEN_RESOURCE.format(
+            contract=contract_id,
+            machine=machine_id,
+        )
+        response, headers = self.request_url(
+            url,
+            method="GET",
+            headers=headers,
+            query_params=self._get_platform_basic_info(),
+        )
+        if headers.get("expires"):
+            response["expires"] = headers["expires"]
+        return response
+
     def _request_machine_token_update(
         self,
         machine_token: str,
@@ -234,6 +262,15 @@ class UAContractClient(serviceclient.UAServiceClient):
             "os": platform_os,
         }
 
+    def _get_platform_basic_info(self):
+        """Return a dict of platform basic info for some contract requests"""
+        platform = util.get_platform_info()
+        return {
+            "architecture": platform["arch"],
+            "series": platform["series"],
+            "kernel": platform["kernel"],
+        }
+
     def _get_activity_info(self, machine_id: Optional[str] = None):
         """Return a dict of activity info data for contract requests"""
         from uaclient.entitlements import ENTITLEMENT_CLASSES
@@ -259,6 +296,7 @@ class UAContractClient(serviceclient.UAServiceClient):
 
 
 def process_entitlements_delta(
+    cfg: UAConfig,
     past_entitlements: Dict[str, Any],
     new_entitlements: Dict[str, Any],
     allow_enable: bool,
@@ -267,6 +305,7 @@ def process_entitlements_delta(
     """Iterate over all entitlements in new_entitlement and apply any delta
     found according to past_entitlements.
 
+    :param cfg: UAConfig instance
     :param past_entitlements: dict containing the last valid information
         regarding service entitlements.
     :param new_entitlements: dict containing the current information regarding
@@ -277,18 +316,27 @@ def process_entitlements_delta(
     :param series_overrides: Boolean set True if series overrides should be
         applied to the new_access dict.
     """
+    from uaclient.entitlements import entitlements_enable_order
+
     delta_error = False
     unexpected_error = False
-    for name, new_entitlement in sorted(new_entitlements.items()):
+
+    # We need to sort our entitlements because some of them
+    # depend on other service to be enable first.
+    for name in entitlements_enable_order(cfg):
+        try:
+            new_entitlement = new_entitlements[name]
+        except KeyError:
+            continue
+
         try:
             deltas, service_enabled = process_entitlement_delta(
-                past_entitlements.get(name, {}),
-                new_entitlement,
+                cfg=cfg,
+                orig_access=past_entitlements.get(name, {}),
+                new_access=new_entitlement,
                 allow_enable=allow_enable,
                 series_overrides=series_overrides,
             )
-        except exceptions.EntitlementNotFoundError:
-            continue
         except exceptions.UserFacingError:
             delta_error = True
             event.service_failed(name)
@@ -323,6 +371,7 @@ def process_entitlements_delta(
 
 
 def process_entitlement_delta(
+    cfg: UAConfig,
     orig_access: Dict[str, Any],
     new_access: Dict[str, Any],
     allow_enable: bool = False,
@@ -330,6 +379,7 @@ def process_entitlement_delta(
 ) -> Tuple[Dict, bool]:
     """Process a entitlement access dictionary deltas if they exist.
 
+    :param cfg: UAConfig instance
     :param orig_access: Dict with original entitlement access details before
         contract refresh deltas
     :param new_access: Dict with updated entitlement access details after
@@ -361,14 +411,14 @@ def process_entitlement_delta(
             )
             raise exceptions.UserFacingError(msg=msg.msg, msg_code=msg.name)
         try:
-            ent_cls = entitlement_factory(name)
+            ent_cls = entitlement_factory(cfg=cfg, name=name)
         except exceptions.EntitlementNotFoundError as exc:
             logging.debug(
                 'Skipping entitlement deltas for "%s". No such class', name
             )
             raise exc
 
-        entitlement = ent_cls(assume_yes=allow_enable)
+        entitlement = ent_cls(cfg=cfg, assume_yes=allow_enable)
         ret = entitlement.process_contract_deltas(
             orig_access, deltas, allow_enable=allow_enable
         )
@@ -376,7 +426,7 @@ def process_entitlement_delta(
 
 
 def _create_attach_forbidden_message(
-    e: exceptions.ContractAPIError
+    e: exceptions.ContractAPIError,
 ) -> messages.NamedMessage:
     msg = messages.ATTACH_EXPIRED_TOKEN
     if (
@@ -391,14 +441,24 @@ def _create_attach_forbidden_message(
 
         if reason == "no-longer-effective":
             date = info["time"].strftime(ATTACH_FAIL_DATE_FORMAT)
+            additional_info = {
+                "contract_expiry_date": info["time"].strftime("%m-%d-%Y"),
+                "contract_id": contract_id,
+            }
             reason_msg = messages.ATTACH_FORBIDDEN_EXPIRED.format(
                 contract_id=contract_id, date=date
             )
+            reason_msg.additional_info = additional_info
         elif reason == "not-effective-yet":
             date = info["time"].strftime(ATTACH_FAIL_DATE_FORMAT)
+            additional_info = {
+                "contract_effective_date": info["time"].strftime("%m-%d-%Y"),
+                "contract_id": contract_id,
+            }
             reason_msg = messages.ATTACH_FORBIDDEN_NOT_YET.format(
                 contract_id=contract_id, date=date
             )
+            reason_msg.additional_info = additional_info
         elif reason == "never-effective":
             reason_msg = messages.ATTACH_FORBIDDEN_NEVER.format(
                 contract_id=contract_id
@@ -407,6 +467,7 @@ def _create_attach_forbidden_message(
         if reason_msg:
             msg = messages.ATTACH_FORBIDDEN.format(reason=reason_msg.msg)
             msg.name = reason_msg.name
+            msg.additional_info = reason_msg.additional_info
 
     return msg
 
@@ -447,7 +508,9 @@ def request_updated_contract(
                     elif e.code == 403:
                         msg = _create_attach_forbidden_message(e)
                         raise exceptions.UserFacingError(
-                            msg=msg.msg, msg_code=msg.name
+                            msg=msg.msg,
+                            msg_code=msg.name,
+                            additional_info=msg.additional_info,
                         )
                 raise e
             with util.disable_log_to_console():
@@ -464,7 +527,7 @@ def request_updated_contract(
         )
 
     process_entitlements_delta(
-        orig_entitlements, cfg.entitlements, allow_enable
+        cfg, orig_entitlements, cfg.entitlements, allow_enable
     )
 
 
@@ -479,3 +542,38 @@ def get_contract_information(cfg: UAConfig, token: str) -> Dict[str, Any]:
     """Query contract information for a specific token"""
     client = UAContractClient(cfg)
     return client.request_contract_information(token)
+
+
+def is_contract_changed(cfg: UAConfig) -> bool:
+    orig_token = cfg.machine_token
+    orig_entitlements = cfg.entitlements
+    machine_token = orig_token.get("machineToken", "")
+    contract_id = (
+        orig_token.get("machineTokenInfo", {})
+        .get("contractInfo", {})
+        .get("id", "")
+    )
+    contract_client = UAContractClient(cfg)
+    resp = contract_client.get_updated_contract_info(
+        machine_token, contract_id
+    )
+    resp_expiry = (
+        resp.get("machineTokenInfo", {})
+        .get("contractInfo", {})
+        .get("effectiveTo", None)
+    )
+    new_expiry = (
+        util.parse_rfc3339_date(resp_expiry)
+        if resp_expiry
+        else cfg.contract_expiry_datetime
+    )
+    if cfg.contract_expiry_datetime != new_expiry:
+        return True
+    curr_entitlements = cfg.get_entitlements_from_token(resp)
+    for name, new_entitlement in sorted(curr_entitlements.items()):
+        deltas = util.get_dict_deltas(
+            orig_entitlements.get(name, {}), new_entitlement
+        )
+        if deltas:
+            return True
+    return False
