@@ -8,11 +8,18 @@ from typing import Any, Dict, List, Optional, Tuple, Type, Union
 
 import yaml
 
-from uaclient import config, contract, event_logger, exceptions, messages, util
+from uaclient import (
+    config,
+    contract,
+    event_logger,
+    exceptions,
+    messages,
+    status,
+    util,
+)
 from uaclient.defaults import DEFAULT_HELP_FILE
-from uaclient.entitlements.entitlement_status import (
+from uaclient.status import (
     ApplicabilityStatus,
-    ApplicationStatus,
     CanDisableFailure,
     CanDisableFailureReason,
     CanEnableFailure,
@@ -155,7 +162,7 @@ class UAEntitlement(metaclass=abc.ABCMeta):
     # Any custom messages to emit to the console or callables which are
     # handled at pre_enable, pre_disable, pre_install or post_enable stages
     @property
-    def messaging(self) -> MessagingOperationsDict:
+    def messaging(self,) -> MessagingOperationsDict:
         return {}
 
     def __init__(
@@ -267,7 +274,7 @@ class UAEntitlement(metaclass=abc.ABCMeta):
         """
         application_status, _ = self.application_status()
 
-        if application_status == ApplicationStatus.DISABLED:
+        if application_status == status.ApplicationStatus.DISABLED:
             return (
                 False,
                 CanDisableFailure(
@@ -312,7 +319,7 @@ class UAEntitlement(metaclass=abc.ABCMeta):
             )
 
         application_status, _ = self.application_status()
-        if application_status != ApplicationStatus.DISABLED:
+        if application_status != status.ApplicationStatus.DISABLED:
             return (
                 False,
                 CanEnableFailure(
@@ -325,7 +332,7 @@ class UAEntitlement(metaclass=abc.ABCMeta):
             return (False, CanEnableFailure(CanEnableFailureReason.IS_BETA))
 
         applicability_status, details = self.applicability_status()
-        if applicability_status == ApplicabilityStatus.INAPPLICABLE:
+        if applicability_status == status.ApplicabilityStatus.INAPPLICABLE:
             return (
                 False,
                 CanEnableFailure(
@@ -361,11 +368,11 @@ class UAEntitlement(metaclass=abc.ABCMeta):
 
         for service in services:
             try:
-                ent_cls = entitlement_factory(cfg=self.cfg, name=service)
+                ent_cls = entitlement_factory(service)
             except EntitlementNotFoundError:
                 continue
             ent_status, _ = ent_cls(self.cfg).application_status()
-            if ent_status == ApplicationStatus.ENABLED:
+            if ent_status == status.ApplicationStatus.ENABLED:
                 return True
 
         return False
@@ -394,11 +401,9 @@ class UAEntitlement(metaclass=abc.ABCMeta):
 
         for required_service in self.required_services:
             try:
-                ent_cls = entitlement_factory(
-                    cfg=self.cfg, name=required_service
-                )
+                ent_cls = entitlement_factory(required_service)
                 ent_status, _ = ent_cls(self.cfg).application_status()
-                if ent_status != ApplicationStatus.ENABLED:
+                if ent_status != status.ApplicationStatus.ENABLED:
                     return False
             except exceptions.EntitlementNotFoundError:
                 pass
@@ -412,7 +417,7 @@ class UAEntitlement(metaclass=abc.ABCMeta):
         ret = []
         for service in self.incompatible_services:
             ent_status, _ = service.entitlement(self.cfg).application_status()
-            if ent_status == ApplicationStatus.ENABLED:
+            if ent_status == status.ApplicationStatus.ENABLED:
                 ret.append(service)
 
         return ret
@@ -428,7 +433,7 @@ class UAEntitlement(metaclass=abc.ABCMeta):
         return len(self.blocking_incompatible_services()) > 0
 
     def handle_incompatible_services(
-        self,
+        self
     ) -> Tuple[bool, Optional[messages.NamedMessage]]:
         """
         Prompt user when incompatible services are found during enable.
@@ -481,7 +486,7 @@ class UAEntitlement(metaclass=abc.ABCMeta):
         return True, None
 
     def _enable_required_services(
-        self,
+        self
     ) -> Tuple[bool, Optional[messages.NamedMessage]]:
         """
         Prompt user when required services are found during enable.
@@ -494,9 +499,7 @@ class UAEntitlement(metaclass=abc.ABCMeta):
 
         for required_service in self.required_services:
             try:
-                ent_cls = entitlement_factory(
-                    cfg=self.cfg, name=required_service
-                )
+                ent_cls = entitlement_factory(required_service)
             except exceptions.EntitlementNotFoundError:
                 msg = messages.REQUIRED_SERVICE_NOT_FOUND.format(
                     service=required_service
@@ -506,7 +509,8 @@ class UAEntitlement(metaclass=abc.ABCMeta):
             ent = ent_cls(self.cfg, allow_beta=True)
 
             is_service_disabled = (
-                ent.application_status()[0] == ApplicationStatus.DISABLED
+                ent.application_status()[0]
+                == status.ApplicationStatus.DISABLED
             )
 
             if is_service_disabled:
@@ -540,7 +544,7 @@ class UAEntitlement(metaclass=abc.ABCMeta):
         return True, None
 
     def applicability_status(
-        self,
+        self
     ) -> Tuple[ApplicabilityStatus, Optional[messages.NamedMessage]]:
         """Check all contract affordances to vet current platform
 
@@ -564,11 +568,8 @@ class UAEntitlement(metaclass=abc.ABCMeta):
                 return ApplicabilityStatus.INAPPLICABLE, error_message
         affordances = entitlement_cfg["entitlement"].get("affordances", {})
         platform = util.get_platform_info()
-        affordance_arches = affordances.get("architectures", None)
-        if (
-            affordance_arches is not None
-            and platform["arch"] not in affordance_arches
-        ):
+        affordance_arches = affordances.get("architectures", [])
+        if affordance_arches and platform["arch"] not in affordance_arches:
             return (
                 ApplicabilityStatus.INAPPLICABLE,
                 messages.INAPPLICABLE_ARCH.format(
@@ -577,11 +578,8 @@ class UAEntitlement(metaclass=abc.ABCMeta):
                     supported_arches=", ".join(affordance_arches),
                 ),
             )
-        affordance_series = affordances.get("series", None)
-        if (
-            affordance_series is not None
-            and platform["series"] not in affordance_series
-        ):
+        affordance_series = affordances.get("series", [])
+        if affordance_series and platform["series"] not in affordance_series:
             return (
                 ApplicabilityStatus.INAPPLICABLE,
                 messages.INAPPLICABLE_SERIES.format(
@@ -589,10 +587,10 @@ class UAEntitlement(metaclass=abc.ABCMeta):
                 ),
             )
         kernel = platform["kernel"]
-        affordance_kernels = affordances.get("kernelFlavors", None)
+        affordance_kernels = affordances.get("kernelFlavors", [])
         affordance_min_kernel = affordances.get("minKernelVersion")
         match = re.match(RE_KERNEL_UNAME, kernel)
-        if affordance_kernels is not None:
+        if affordance_kernels:
             if not match or match.group("flavor") not in affordance_kernels:
                 return (
                     ApplicabilityStatus.INAPPLICABLE,
@@ -663,9 +661,7 @@ class UAEntitlement(metaclass=abc.ABCMeta):
 
         for dependent_service in self.dependent_services:
             try:
-                ent_cls = entitlement_factory(
-                    cfg=self.cfg, name=dependent_service
-                )
+                ent_cls = entitlement_factory(dependent_service)
             except exceptions.EntitlementNotFoundError:
                 msg = messages.DEPENDENT_SERVICE_NOT_FOUND.format(
                     service=dependent_service
@@ -676,7 +672,7 @@ class UAEntitlement(metaclass=abc.ABCMeta):
             ent = ent_cls(cfg=self.cfg, assume_yes=True)
 
             is_service_enabled = (
-                ent.application_status()[0] == ApplicationStatus.ENABLED
+                ent.application_status()[0] == status.ApplicationStatus.ENABLED
             )
 
             if is_service_enabled:
@@ -801,12 +797,12 @@ class UAEntitlement(metaclass=abc.ABCMeta):
             return False
         return True
 
-    def _check_application_status_on_cache(self) -> ApplicationStatus:
+    def _check_application_status_on_cache(self) -> status.ApplicationStatus:
         """Check on the state of application on the status cache."""
         status_cache = self.cfg.read_cache("status-cache")
 
         if status_cache is None:
-            return ApplicationStatus.DISABLED
+            return status.ApplicationStatus.DISABLED
 
         services_status_list = status_cache.get("services", [])
 
@@ -815,11 +811,11 @@ class UAEntitlement(metaclass=abc.ABCMeta):
                 service_status = service.get("status")
 
                 if service_status == "enabled":
-                    return ApplicationStatus.ENABLED
+                    return status.ApplicationStatus.ENABLED
                 else:
-                    return ApplicationStatus.DISABLED
+                    return status.ApplicationStatus.DISABLED
 
-        return ApplicationStatus.DISABLED
+        return status.ApplicationStatus.DISABLED
 
     def process_contract_deltas(
         self,
@@ -863,7 +859,7 @@ class UAEntitlement(metaclass=abc.ABCMeta):
             else:
                 application_status, _ = self.application_status()
 
-            if application_status != ApplicationStatus.DISABLED:
+            if application_status != status.ApplicationStatus.DISABLED:
                 if self.can_disable():
                     self.disable()
                     logging.info(
@@ -911,7 +907,7 @@ class UAEntitlement(metaclass=abc.ABCMeta):
         return False
 
     def user_facing_status(
-        self,
+        self
     ) -> Tuple[UserFacingStatus, Optional[messages.NamedMessage]]:
         """Return (user-facing status, details) for entitlement"""
         applicability, details = self.applicability_status()
@@ -931,15 +927,15 @@ class UAEntitlement(metaclass=abc.ABCMeta):
 
         application_status, explanation = self.application_status()
         user_facing_status = {
-            ApplicationStatus.ENABLED: UserFacingStatus.ACTIVE,
-            ApplicationStatus.DISABLED: UserFacingStatus.INACTIVE,
+            status.ApplicationStatus.ENABLED: UserFacingStatus.ACTIVE,
+            status.ApplicationStatus.DISABLED: UserFacingStatus.INACTIVE,
         }[application_status]
         return user_facing_status, explanation
 
     @abc.abstractmethod
     def application_status(
-        self,
-    ) -> Tuple[ApplicationStatus, Optional[messages.NamedMessage]]:
+        self
+    ) -> Tuple[status.ApplicationStatus, Optional[messages.NamedMessage]]:
         """
         The current status of application of this entitlement
 
