@@ -4,11 +4,10 @@ import re
 from itertools import groupby
 from typing import List, Optional, Tuple  # noqa: F401
 
-from uaclient import apt, event_logger, exceptions, messages, util
+from uaclient import apt, event_logger, exceptions, messages, status, util
 from uaclient.clouds.identity import NoCloudTypeReason, get_cloud_type
 from uaclient.entitlements import repo
 from uaclient.entitlements.base import IncompatibleService
-from uaclient.entitlements.entitlement_status import ApplicationStatus
 from uaclient.types import (  # noqa: F401
     MessagingOperations,
     MessagingOperationsDict,
@@ -206,7 +205,7 @@ class FIPSCommonEntitlement(repo.RepoEntitlement):
     ) -> bool:
         """Return False when FIPS is allowed on this cloud and series.
 
-        On Xenial GCP there will be no cloud-optimized kernel so
+        On Xenial Azure and GCP there will be no cloud-optimized kernel so
         block default ubuntu-fips enable. This can be overridden in
         config with features.allow_xenial_fips_on_cloud.
 
@@ -217,6 +216,9 @@ class FIPSCommonEntitlement(repo.RepoEntitlement):
 
         :return: False when this cloud, series or config override allows FIPS.
         """
+        if cloud_id not in ("azure", "gce"):
+            return True
+
         if cloud_id == "gce":
             if util.is_config_value_true(
                 config=self.cfg.cfg,
@@ -230,6 +232,16 @@ class FIPSCommonEntitlement(repo.RepoEntitlement):
 
             return bool("ubuntu-gcp-fips" in super().packages)
 
+        # Azure FIPS cloud support
+        if series == "xenial":
+            if util.is_config_value_true(
+                config=self.cfg.cfg,
+                path_to_value="features.allow_xenial_fips_on_cloud",
+            ):
+                return True
+            else:
+                return False
+
         return True
 
     @property
@@ -300,8 +312,8 @@ class FIPSCommonEntitlement(repo.RepoEntitlement):
         return self._replace_metapackage_on_cloud_instance(packages)
 
     def application_status(
-        self,
-    ) -> Tuple[ApplicationStatus, Optional[messages.NamedMessage]]:
+        self
+    ) -> Tuple[status.ApplicationStatus, Optional[messages.NamedMessage]]:
         super_status, super_msg = super().application_status()
 
         if util.is_container() and not util.should_reboot():
@@ -314,21 +326,18 @@ class FIPSCommonEntitlement(repo.RepoEntitlement):
             self.cfg.remove_notice(
                 "", messages.FIPS_SYSTEM_REBOOT_REQUIRED.msg
             )
-            self.cfg.remove_notice("", messages.FIPS_REBOOT_REQUIRED_MSG)
             if util.load_file(self.FIPS_PROC_FILE).strip() == "1":
                 self.cfg.remove_notice(
-                    "", messages.NOTICE_FIPS_MANUAL_DISABLE_URL
+                    "", status.NOTICE_FIPS_MANUAL_DISABLE_URL
                 )
                 return super_status, super_msg
             else:
                 self.cfg.remove_notice(
                     "", messages.FIPS_DISABLE_REBOOT_REQUIRED
                 )
-                self.cfg.add_notice(
-                    "", messages.NOTICE_FIPS_MANUAL_DISABLE_URL
-                )
+                self.cfg.add_notice("", status.NOTICE_FIPS_MANUAL_DISABLE_URL)
                 return (
-                    ApplicationStatus.DISABLED,
+                    status.ApplicationStatus.DISABLED,
                     messages.FIPS_PROC_FILE_ERROR.format(
                         file_name=self.FIPS_PROC_FILE
                     ),
@@ -336,10 +345,10 @@ class FIPSCommonEntitlement(repo.RepoEntitlement):
         else:
             self.cfg.remove_notice("", messages.FIPS_DISABLE_REBOOT_REQUIRED)
 
-        if super_status != ApplicationStatus.ENABLED:
+        if super_status != status.ApplicationStatus.ENABLED:
             return super_status, super_msg
         return (
-            ApplicationStatus.ENABLED,
+            status.ApplicationStatus.ENABLED,
             messages.FIPS_REBOOT_REQUIRED,
         )
 
@@ -371,9 +380,8 @@ class FIPSCommonEntitlement(repo.RepoEntitlement):
     def _perform_enable(self, silent: bool = False) -> bool:
         if super()._perform_enable(silent=silent):
             self.cfg.remove_notice(
-                "", messages.NOTICE_WRONG_FIPS_METAPACKAGE_ON_CLOUD
+                "", status.NOTICE_WRONG_FIPS_METAPACKAGE_ON_CLOUD
             )
-            self.cfg.remove_notice("", messages.FIPS_REBOOT_REQUIRED_MSG)
             return True
 
         return False
@@ -429,7 +437,7 @@ class FIPSEntitlement(FIPSCommonEntitlement):
         static_affordances = super().static_affordances
 
         fips_update = FIPSUpdatesEntitlement(self.cfg)
-        enabled_status = ApplicationStatus.ENABLED
+        enabled_status = status.ApplicationStatus.ENABLED
         is_fips_update_enabled = bool(
             fips_update.application_status()[0] == enabled_status
         )
@@ -459,17 +467,15 @@ class FIPSEntitlement(FIPSCommonEntitlement):
         )
 
     @property
-    def messaging(self) -> MessagingOperationsDict:
+    def messaging(self,) -> MessagingOperationsDict:
         post_enable = None  # type: Optional[MessagingOperations]
         if util.is_container():
-            pre_enable_prompt = (
-                messages.PROMPT_FIPS_CONTAINER_PRE_ENABLE.format(
-                    title=self.title
-                )
+            pre_enable_prompt = status.PROMPT_FIPS_CONTAINER_PRE_ENABLE.format(
+                title=self.title
             )
             post_enable = [messages.FIPS_RUN_APT_UPGRADE]
         else:
-            pre_enable_prompt = messages.PROMPT_FIPS_PRE_ENABLE
+            pre_enable_prompt = status.PROMPT_FIPS_PRE_ENABLE
 
         return {
             "pre_enable": [
@@ -483,7 +489,7 @@ class FIPSEntitlement(FIPSCommonEntitlement):
                 (
                     util.prompt_for_confirmation,
                     {
-                        "msg": messages.PROMPT_FIPS_PRE_DISABLE,
+                        "msg": status.PROMPT_FIPS_PRE_DISABLE,
                         "assume_yes": self.assume_yes,
                     },
                 )
@@ -526,17 +532,15 @@ class FIPSUpdatesEntitlement(FIPSCommonEntitlement):
         )
 
     @property
-    def messaging(self) -> MessagingOperationsDict:
+    def messaging(self,) -> MessagingOperationsDict:
         post_enable = None  # type: Optional[MessagingOperations]
         if util.is_container():
-            pre_enable_prompt = (
-                messages.PROMPT_FIPS_CONTAINER_PRE_ENABLE.format(
-                    title=self.title
-                )
+            pre_enable_prompt = status.PROMPT_FIPS_CONTAINER_PRE_ENABLE.format(
+                title=self.title
             )
             post_enable = [messages.FIPS_RUN_APT_UPGRADE]
         else:
-            pre_enable_prompt = messages.PROMPT_FIPS_UPDATES_PRE_ENABLE
+            pre_enable_prompt = status.PROMPT_FIPS_UPDATES_PRE_ENABLE
 
         return {
             "pre_enable": [
@@ -550,7 +554,7 @@ class FIPSUpdatesEntitlement(FIPSCommonEntitlement):
                 (
                     util.prompt_for_confirmation,
                     {
-                        "msg": messages.PROMPT_FIPS_PRE_DISABLE,
+                        "msg": status.PROMPT_FIPS_PRE_DISABLE,
                         "assume_yes": self.assume_yes,
                     },
                 )
