@@ -2,9 +2,7 @@ import logging
 import os
 import shutil
 
-from uaclient import exceptions, util
-
-LOG = logging.getLogger(util.replace_top_level_logger_name(__name__))
+from uaclient import exceptions
 
 
 def export_gpg_key(source_keyfile: str, destination_keyfile: str) -> None:
@@ -17,7 +15,7 @@ def export_gpg_key(source_keyfile: str, destination_keyfile: str) -> None:
     :raise UserFacingError: Any GPG errors or if specific key does not exist in
         the source_keyring_file.
     """
-    LOG.debug("Exporting GPG key %s", source_keyfile)
+    logging.debug("Exporting GPG key %s", source_keyfile)
     if not os.path.exists(source_keyfile):
         raise exceptions.UserFacingError(
             "GPG key '{}' not found.".format(source_keyfile)
