@@ -1,6 +1,6 @@
 import sys
 from gettext import NullTranslations, translation
-from typing import Callable, Dict, Optional
+from typing import Callable, Dict, Optional  # noqa: F401
 
 from uaclient.messages import urls
 
@@ -81,11 +81,6 @@ RELEASE_UPGRADE_SUCCESS = t.gettext(
     "Finished upgrade of Ubuntu Pro service configuration"
 )
 
-PRO_ONLY_ALLOWED_FOR_RELEASE = t.gettext(
-    "Detaching Ubuntu Pro. Previously attached subscription \
-was only valid for Ubuntu {release} ({series_codename}) release."
-)
-
 MISSING_YAML_MODULE = t.gettext(
     """\
 Couldn't import the YAML module.
@@ -109,12 +104,11 @@ WARN_NEW_VERSION_AVAILABLE_CLI = (
         """\
  A new version is available: {version}
 Please run:
-    sudo apt install ubuntu-pro-client
+    sudo apt-get install ubuntu-advantage-tools
 to get the latest bug fixes and new features."""
     )
 )
 
-UNKNOWN_ERROR = t.gettext("an unknown error")
 
 ###############################################################################
 #                      GENERIC SYSTEM OPERATIONS                              #
@@ -167,12 +161,12 @@ CONTRACT_EXPIRED_WITH_PKGS = P(
     lambda n: t.ngettext(
         """\
 *Your Ubuntu Pro subscription has EXPIRED*
-{{pkg_num}} additional security update requires Ubuntu Pro with '{{service}}' enabled.
-Renew your subscription at {url}""",  # noqa: E501
+{{pkg_num}} additional security update(s) require Ubuntu Pro with '{{service}}' enabled.
+Renew your service at {url}""",  # noqa: E501
         """\
 *Your Ubuntu Pro subscription has EXPIRED*
-{{pkg_num}} additional security updates require Ubuntu Pro with '{{service}}' enabled.
-Renew your subscription at {url}""",  # noqa: E501
+{{pkg_num}} additional security update(s) require Ubuntu Pro with '{{service}}' enabled.
+Renew your service at {url}""",  # noqa: E501
         n,
     ).format(url=urls.PRO_DASHBOARD)
 )
@@ -207,7 +201,7 @@ Your grace period will expire in {{remaining_days}} days.""",
 CONTRACT_EXPIRED = t.gettext(
     """\
 *Your Ubuntu Pro subscription has EXPIRED*
-Renew your subscription at {url}"""
+Renew your service at {url}"""
 ).format(url=urls.PRO_DASHBOARD)
 
 
@@ -276,7 +270,7 @@ Warning: Failed to migrate /etc/ubuntu-advantage/uaclient.conf
 # ATTACH
 AUTO_ATTACH_RUNNING = t.gettext(
     "Currently attempting to automatically attach this machine to "
-    "an Ubuntu Pro subscription"
+    "Ubuntu Pro services"
 )
 ATTACH_SUCCESS_TMPL = t.gettext(
     """\
@@ -307,10 +301,6 @@ And provide the following code: {bold}{{user_code}}{end_bold}"""
 )
 CLI_MAGIC_ATTACH_PROCESSING = t.gettext("Attaching the machine...")
 
-LIMITED_TO_RELEASE = t.gettext(
-    "Limited to release: Ubuntu {release} ({series_codename})."
-)
-
 # DETACH
 DETACH_WILL_DISABLE = P(
     lambda n: t.ngettext(
@@ -325,7 +315,6 @@ DETACH_SUCCESS = t.gettext("This machine is now detached.")
 REFRESH_CONTRACT_ENABLE = t.gettext(
     "One moment, checking your subscription first"
 )
-ENABLING_TMPL = t.gettext("Enabling {title}")
 ENABLED_TMPL = t.gettext("{title} enabled")
 ACCESS_ENABLED_TMPL = t.gettext("{title} access enabled")
 ENABLE_FAILED = t.gettext("Could not enable {title}.")
@@ -349,15 +338,8 @@ ENABLE_REBOOT_REQUIRED_TMPL = t.gettext(
     """\
 A reboot is required to complete {operation}."""
 )
-CONFIGURING_APT_ACCESS = t.gettext("Configuring APT access to {service}")
-AUTO_SELECTING_VARIANT = t.gettext(
-    """\
-No variant specified. To specify a variant, use the variant option.
-Auto-selecting {variant} variant. Proceed? (y/N) """
-)
 
 # DISABLE
-REMOVING_APT_CONFIGURATION = t.gettext("Removing APT access to {title}")
 DISABLE_FAILED_TMPL = t.gettext("Could not disable {title}.")
 DEPENDENT_SERVICE = t.gettext(
     """\
@@ -373,9 +355,6 @@ APT_REMOVING_SOURCE_FILE = t.gettext("Removing apt source file: {filename}")
 APT_REMOVING_PREFERENCES_FILE = t.gettext(
     "Removing apt preferences file: {filename}"
 )
-PURGING_PACKAGES = t.gettext(
-    "Uninstalling all packages installed from {title}"
-)
 
 # Kernel checks for Purge
 PURGE_EXPERIMENTAL = t.gettext(
@@ -405,15 +384,15 @@ working properly, *do not proceed*. You may end up with an unbootable system.
 # These are for the retry-auto-attach functionality
 AUTO_ATTACH_RETRY_NOTICE = t.gettext(
     """\
-Failed to automatically attach to an Ubuntu Pro subscription {num_attempts} time(s).
+Failed to automatically attach to Ubuntu Pro services {num_attempts} time(s).
 The failure was due to: {reason}.
 The next attempt is scheduled for {next_run_datestring}.
-You can try manually with `sudo pro auto-attach`."""  # noqa: E501
+You can try manually with `sudo pro auto-attach`."""
 )
 
 AUTO_ATTACH_RETRY_TOTAL_FAILURE_NOTICE = t.gettext(
     """\
-Failed to automatically attach to an Ubuntu Pro subscription {num_attempts} time(s).
+Failed to automatically attach to Ubuntu Pro services {num_attempts} time(s).
 The most recent failure was due to: {reason}.
 Try re-launching the instance or report this issue by running `ubuntu-bug ubuntu-advantage-tools`
 You can try manually with `sudo pro auto-attach`."""  # noqa: E501
@@ -432,6 +411,7 @@ RETRY_ERROR_DETAIL_CONTRACT_API_ERROR = t.gettext(
 )
 RETRY_ERROR_DETAIL_CONNECTIVITY_ERROR = t.gettext("a connectivity error")
 RETRY_ERROR_DETAIL_URL_ERROR_URL = t.gettext("an error while reaching {url}")
+RETRY_ERROR_DETAIL_UNKNOWN = t.gettext("an unknown error")
 
 # These are related messages but actually occur during a "refresh"
 DISABLE_DURING_CONTRACT_REFRESH = t.gettext(
@@ -544,17 +524,16 @@ this service.
 SECURITY_DRY_RUN_UA_NOT_ATTACHED = t.gettext(
     """\
 {bold}The machine is not attached to an Ubuntu Pro subscription.
-To proceed with the fix, a prompt would ask to attach
-the machine to a subscription or use an existing token.
-{{ pro attach }}{end_bold}"""
+To proceed with the fix, a prompt would ask for a valid Ubuntu Pro token.
+{{ pro attach TOKEN }}{end_bold}"""
 ).format(bold=TxtColor.BOLD, end_bold=TxtColor.ENDC)
 SECURITY_DRY_RUN_UA_EXPIRED_SUBSCRIPTION = t.gettext(
     """\
 {bold}The machine has an expired subscription.
-To proceed with the fix, a prompt would ask to attach the machine to a
-new subscription or use a new Ubuntu Pro subscription token.
+To proceed with the fix, a prompt would ask for a new Ubuntu Pro
+token to renew the subscription.
 {{ pro detach --assume-yes }}
-{{ pro attach }}{end_bold}"""
+{{ pro attach NEW_TOKEN }}{end_bold}"""
 ).format(bold=TxtColor.BOLD, end_bold=TxtColor.ENDC)
 SECURITY_DRY_RUN_WARNING = t.gettext(
     """\
@@ -617,7 +596,7 @@ SECURITY_CVE_STATUS_UNKNOWN = t.gettext(
 UNKNOWN: {status}"""
 )
 
-SECURITY_FOUND_CVES = t.gettext("Associated CVEs:")
+SECURITY_FOUND_CVES = t.gettext("Found CVEs:")
 SECURITY_FOUND_LAUNCHPAD_BUGS = t.gettext("Found Launchpad bugs:")
 SECURITY_FIXING_REQUESTED_USN = t.gettext(
     """\
@@ -703,7 +682,7 @@ for a list of available options."""
 SS_UPDATE_CALL = t.gettext(
     """\
  Make sure to run
-    sudo apt update
+    sudo apt-get update
 to get the latest package information from apt."""
 )
 SS_UPDATE_DAYS = (
@@ -882,21 +861,16 @@ Please run `sudo pro refresh`."""
 # Also, any generic strings about the CLI itself go here.
 
 
+CLI_TRY_HELP = t.gettext("Try 'pro --help' for more information.")
 CLI_HELP_EPILOG = t.gettext(
     "Use {name} {command} --help for more information about a command."
 )
-
-CLI_HELP_FLAG_DESC = t.gettext(
-    "Displays help on {name} and command line options"
+PRO_HELP_SERVICE_INFO = t.gettext(
+    "Use pro help <service> to get more details about each service"
 )
 
-CLI_HELP_HEADER_QUICK_START = t.gettext("Quick start commands")
-CLI_HELP_HEADER_SECURITY = t.gettext("Security-related commands")
-CLI_HELP_HEADER_TROUBLESHOOT = t.gettext("Troubleshooting-related commands")
-CLI_HELP_HEADER_OTHER = t.gettext("Other commands")
-
-
 CLI_HELP_VARIANTS_HEADER = t.gettext("Variants:")
+CLI_ARGS = t.gettext("Arguments")
 CLI_FLAGS = t.gettext("Flags")
 CLI_AVAILABLE_COMMANDS = t.gettext("Available Commands")
 CLI_FORMAT_DESC = t.gettext(
@@ -908,10 +882,6 @@ CLI_ASSUME_YES = t.gettext(
 
 CLI_API_DESC = t.gettext("Calls the Client API endpoints.")
 CLI_API_ENDPOINT = t.gettext("API endpoint to call")
-CLI_API_SHOW_PROGRESS = t.gettext(
-    "For endpoints that support progress updates, show each progress update "
-    "on a new line in JSON format"
-)
 CLI_API_ARGS = t.gettext(
     "Options to pass to the API endpoint, formatted as key=value"
 )
@@ -925,10 +895,10 @@ CLI_COLLECT_LOGS_DESC = t.gettext(
     "Collect logs and relevant system information into a tarball."
 )
 CLI_COLLECT_LOGS_OUTPUT = t.gettext(
-    "tarball where the logs will be stored. (Defaults to " "./pro_logs.tar.gz)"
+    "tarball where the logs will be stored. (Defaults to " "./ua_logs.tar.gz)"
 )
 
-CLI_CONFIG_SHOW_DESC = t.gettext("Show customizable configuration settings")
+CLI_CONFIG_SHOW_DESC = t.gettext("Show customisable configuration settings")
 CLI_CONFIG_SHOW_KEY = t.gettext(
     "Optional key or key(s) to show configuration settings."
 )
@@ -947,7 +917,7 @@ CLI_CONFIG_DESC = t.gettext("Manage Ubuntu Pro configuration")
 
 CLI_ATTACH_DESC = t.gettext(
     """\
-Attach this machine to an Ubuntu Pro subscription with a token obtained from:
+Attach this machine to Ubuntu Pro with a token obtained from:
 {url}
 
 When running this command without a token, it will generate a short code
@@ -981,16 +951,6 @@ CLI_FIX_NO_RELATED = t.gettext(
     " also fix related USNs to the target USN."
 )
 
-CLI_FIX_FAIL_UPDATING_ESM_CACHE = t.gettext(
-    "WARNING: Failed to update ESM cache - package availability may be inaccurate"  # noqa
-)
-
-CLI_FIX_FAIL_UPDATING_ESM_CACHE_NON_ROOT = t.gettext(
-    "{bold}WARNING: Unable to update ESM cache when running as non-root,\n"
-    "please run sudo apt update and try again "
-    "if packages cannot be found.{end_bold}"
-).format(bold=TxtColor.BOLD, end_bold=TxtColor.ENDC)
-
 CLI_SS_DESC = t.gettext(
     """\
 Show security updates for packages in the system, including all
@@ -1033,15 +993,13 @@ Refresh three distinct Ubuntu Pro related artifacts in the system:
 * messages: Update APT and MOTD messages related to UA.
 
 You can individually target any of the three specific actions,
-by passing the target name to the command.  If no `target`
+by passing it's target to nome to the command.  If no `target`
 is specified, all targets are refreshed.
 """
 )
 CLI_REFRESH_TARGET = t.gettext("Target to refresh.")
 
-CLI_DETACH_DESC = t.gettext(
-    "Detach this machine from an Ubuntu Pro subscription."
-)
+CLI_DETACH_DESC = t.gettext("Detach this machine from Ubuntu Pro services.")
 
 CLI_HELP_DESC = t.gettext(
     "Provide detailed information about Ubuntu Pro services."
@@ -1109,7 +1067,7 @@ The attached status output has four columns:
   entitles use of this service. Possible values are: yes or no
 * STATUS: whether the service is enabled on this machine. Possible
   values are: enabled, disabled, n/a (if your contract entitles
-  you to the service, but it isn't available for this machine) or - (if
+  you to the service, but it isn't available for this machine) or — (if
   you aren't entitled to this service)
 * DESCRIPTION: a brief description of the service
 
@@ -1240,7 +1198,7 @@ CIS_USG_POST_ENABLE = t.gettext("Visit {url} for the next steps").format(
 )
 CIS_IS_NOW_USG = t.gettext(
     """\
-From Ubuntu 20.04 onward 'pro enable cis' has been
+From Ubuntu 20.04 and onwards 'pro enable cis' has been
 replaced by 'pro enable usg'. See more information at:
 {url}"""
 ).format(url=urls.USG_DOCS)
@@ -1297,7 +1255,7 @@ Warning: FIPS kernel is not optimized for your specific cloud.
 To fix it, run the following commands:
 
     1. sudo pro disable fips
-    2. sudo apt remove ubuntu-fips
+    2. sudo apt-get remove ubuntu-fips
     3. sudo pro enable fips --assume-yes
     4. sudo reboot
 """
@@ -1340,25 +1298,6 @@ This will disable the {title} entitlement but the {title} packages will remain i
     )
     + PROMPT_YES_NO
 )
-KERNEL_DOWNGRADE_WARNING = t.gettext(
-    """\
-This will downgrade the kernel from {current_version} to {new_version}.
-Warning: Downgrading the kernel may cause hardware failures.  Please ensure the
-         hardware is compatible with the new kernel version before proceeding.
-"""
-)
-KERNEL_FLAVOR_CHANGE_WARNING_PROMPT = t.gettext(
-    """\
-The "{variant}" variant of {service} is based on the "{base_flavor}" Ubuntu
-kernel but this machine is running the "{current_flavor}" kernel.
-The "{current_flavor}" kernel may have significant hardware support
-differences from "{variant}" {service}.
-
-Warning: Installing {variant} {service} may result in lost hardware support
-         and may prevent the system from booting.
-
-Do you accept the risk and wish to continue? (y/N) """
-)
 FIPS_SYSTEM_REBOOT_REQUIRED = t.gettext(
     "FIPS support requires system reboot to complete configuration."
 )
@@ -1378,13 +1317,6 @@ Please run `apt upgrade` to ensure all FIPS packages are updated to the correct
 version.
 """
 )
-FIPS_PACKAGES_UPGRADE_FAILURE = (
-    t.gettext(
-        "Failure occurred while upgrading packages to {service} versions."
-    )
-    + "\n"
-    + FIPS_RUN_APT_UPGRADE
-)
 
 FIPS_UPDATES_TITLE = t.gettext("FIPS Updates")
 FIPS_UPDATES_DESCRIPTION = t.gettext(
@@ -1435,13 +1367,6 @@ more. Find out more about Landscape at {home_url}"""
     install_url=urls.LANDSCAPE_DOCS_INSTALL,
     home_url=urls.LANDSCAPE_HOME_PAGE,
 )
-LANDSCAPE_CONFIG_REMAINS = t.gettext(
-    """\
-/etc/landscape/client.conf contains your landscape-client configuration.
-To re-enable Landscape with the same configuration, run:
-    sudo pro enable landscape --assume-yes
-"""
-)
 
 LIVEPATCH_TITLE = t.gettext("Livepatch")
 LIVEPATCH_DESCRIPTION = t.gettext("Canonical Livepatch service")
@@ -1455,10 +1380,10 @@ on fips-enabled systems. You can find out more about Ubuntu Kernel Livepatch
 service at {url}"""
 ).format(url=urls.LIVEPATCH_HOME_PAGE)
 LIVEPATCH_KERNEL_NOT_SUPPORTED_DESCRIPTION = t.gettext(
-    "Current kernel is not covered by livepatch"
+    "Current kernel is not supported"
 )
 LIVEPATCH_KERNEL_NOT_SUPPORTED_UNATTACHED = t.gettext(
-    "Kernels covered by livepatch are listed here: {url}"
+    "Supported livepatch kernels are listed here: {url}"
 ).format(url=urls.LIVEPATCH_SUPPORTED_KERNELS)
 LIVEPATCH_UNABLE_TO_CONFIGURE = t.gettext(
     "Unable to configure livepatch: {error_msg}"
@@ -1468,10 +1393,8 @@ LIVEPATCH_DISABLE_REATTACH = t.gettext(
     "Disabling Livepatch prior to re-attach with new token"
 )
 LIVEPATCH_LTS_REBOOT_REQUIRED = t.gettext(
-    "Livepatch coverage requires a system reboot across LTS upgrade."
+    "Livepatch support requires a system reboot across LTS upgrade."
 )
-INSTALLING_LIVEPATCH = t.gettext("Installing Livepatch")
-SETTING_UP_LIVEPATCH = t.gettext("Setting up Livepatch")
 
 REALTIME_TITLE = t.gettext("Real-time kernel")
 REALTIME_DESCRIPTION = t.gettext(
@@ -1494,10 +1417,6 @@ REALTIME_NVIDIA_TITLE = t.gettext("Real-time NVIDIA Tegra Kernel")
 REALTIME_NVIDIA_DESCRIPTION = t.gettext(
     "RT kernel optimized for NVIDIA Tegra platform"
 )
-REALTIME_RASPI_TITLE = t.gettext("Raspberry Pi Real-time for Pi5/Pi4")
-REALTIME_RASPI_DESCRIPTION = t.gettext(
-    "24.04 Real-time kernel optimised for Raspberry Pi"
-)
 REALTIME_INTEL_TITLE = t.gettext("Real-time Intel IOTG Kernel")
 REALTIME_INTEL_DESCRIPTION = t.gettext(
     "RT kernel optimized for Intel IOTG platform"
@@ -1615,21 +1534,13 @@ It is only possible to enable Anbox Cloud on a container using
 the --access-only flag.""",
 )
 
-ATTACH_FAILURE_RESTRICTED_RELEASE = FormattedNamedMessage(
-    "attach-failure-restricted-release",
-    t.gettext(
-        "Attach failed. Attaching to this contract \
-is only allowed on the Ubuntu {release} ({series_codename}) release."
-    ),
-)
-
-UNEXPECTED_ERROR = FormattedNamedMessage(
+UNEXPECTED_ERROR = NamedMessage(
     "unexpected-error",
     t.gettext(
         """\
-An unexpected error occurred: {error_msg}
-For more details, see the log: {log_path}
-If you think this is a bug, please run: ubuntu-bug ubuntu-advantage-tools"""
+Unexpected error(s) occurred.
+For more details, see the log: /var/log/ubuntu-advantage.log
+To file a bug run: ubuntu-bug ubuntu-advantage-tools"""
     ),
 )
 
@@ -1683,6 +1594,14 @@ FAILED_DISABLING_DEPENDENT_SERVICE = FormattedNamedMessage(
 Cannot disable dependent service: {required_service}{error}"""
     ),
 )
+DEPENDENT_SERVICE_STOPS_DISABLE = FormattedNamedMessage(
+    "depedent-service-stops-disable",
+    t.gettext(
+        """\
+Cannot disable {service_being_disabled} when {dependent_service} is enabled.
+"""
+    ),
+)
 REPO_PURGE_FAIL_NO_ORIGIN = FormattedNamedMessage(
     "repo-purge-fail-no-origin",
     t.gettext(
@@ -1695,6 +1614,22 @@ ERROR_ENABLING_REQUIRED_SERVICE = FormattedNamedMessage(
     "error-enabling-required-service",
     t.gettext("Cannot enable required service: {service}{error}"),
 )
+REQUIRED_SERVICE_STOPS_ENABLE = FormattedNamedMessage(
+    "required-service-stops-enable",
+    t.gettext(
+        """\
+Cannot enable {service_being_enabled} when {required_service} is disabled.
+"""
+    ),
+)
+INCOMPATIBLE_SERVICE_STOPS_ENABLE = FormattedNamedMessage(
+    "incompatible-service-stops-enable",
+    t.gettext(
+        """\
+Cannot enable {service_being_enabled} when \
+{incompatible_service} is enabled."""
+    ),
+)
 
 SERVICE_ERROR_INSTALL_ON_CONTAINER = FormattedNamedMessage(
     "service-error-install-on-container",
@@ -1718,31 +1653,18 @@ NO_APT_URL_FOR_SERVICE = FormattedNamedMessage(
     "no-apt-url-for-service",
     t.gettext("{title} does not have an aptURL directive"),
 )
-NO_SUITES_FOR_SERVICE = FormattedNamedMessage(
-    "no-suites-for-service",
-    t.gettext("{title} does not have a suites directive"),
-)
 ALREADY_DISABLED = FormattedNamedMessage(
     "service-already-disabled",
     t.gettext(
         """\
-{title} is not currently enabled - nothing to do.
-See: sudo pro status"""
-    ),
-)
-CANNOT_DISABLE_NOT_APPLICABLE = FormattedNamedMessage(
-    "cannot-disable-not-applicable",
-    t.gettext(
-        """\
-Disabling {title} with pro is not supported.\nSee: sudo pro status"""
+{title} is not currently enabled\nSee: sudo pro status"""
     ),
 )
 ALREADY_ENABLED = FormattedNamedMessage(
     "service-already-enabled",
     t.gettext(
         """\
-{title} is already enabled - nothing to do.
-See: sudo pro status"""
+{title} is already enabled.\nSee: sudo pro status"""
     ),
 )
 UNENTITLED = FormattedNamedMessage(
@@ -1756,9 +1678,6 @@ View your subscription at: {url}"""
 SERVICE_NOT_ENTITLED = FormattedNamedMessage(
     "service-not-entitled", t.gettext("{title} is not entitled")
 )
-AUTO_SELECTED_VARIANT_WARNING = FormattedNamedMessage(
-    "auto-selected-variant", t.gettext("Auto-selected {variant_name} variant")
-)
 
 INAPPLICABLE_KERNEL_VER = FormattedNamedMessage(
     "inapplicable-kernel-version",
@@ -1865,25 +1784,25 @@ LIVEPATCH_KERNEL_UPGRADE_REQUIRED = NamedMessage(
     msg=t.gettext(
         """\
 The running kernel has reached the end of its active livepatch window.
-Please upgrade the kernel with apt and reboot for continued livepatch coverage."""  # noqa: E501
+Please upgrade the kernel with apt and reboot for continued livepatch support."""  # noqa: E501
     ),
 )
 LIVEPATCH_KERNEL_EOL = FormattedNamedMessage(
     name="livepatch-kernel-eol",
     msg=t.gettext(
         """\
-The current kernel ({{version}}, {{arch}}) has reached the end of its livepatch coverage.
-Covered kernels are listed here: {url}
-Either switch to a covered kernel or `sudo pro disable livepatch` to dismiss this warning."""  # noqa: E501
+The current kernel ({{version}}, {{arch}}) has reached the end of its livepatch support.
+Supported kernels are listed here: {url}
+Either switch to a supported kernel or `pro disable livepatch` to dismiss this warning."""  # noqa: E501
     ).format(url=urls.LIVEPATCH_SUPPORTED_KERNELS),
 )
 LIVEPATCH_KERNEL_NOT_SUPPORTED = FormattedNamedMessage(
     name="livepatch-kernel-not-supported",
     msg=t.gettext(
         """\
-The current kernel ({{version}}, {{arch}}) is not covered by livepatch.
-Covered kernels are listed here: {url}
-Either switch to a covered kernel or `sudo pro disable livepatch` to dismiss this warning."""  # noqa: E501
+The current kernel ({{version}}, {{arch}}) is not supported by livepatch.
+Supported kernels are listed here: {url}
+Either switch to a supported kernel or `pro disable livepatch` to dismiss this warning."""  # noqa: E501
     ).format(
         url=urls.LIVEPATCH_SUPPORTED_KERNELS
     ),  # noqa: E501
@@ -1893,15 +1812,6 @@ LIVEPATCH_APPLICATION_STATUS_CLIENT_FAILURE = NamedMessage(
     t.gettext("canonical-livepatch status didn't finish successfully"),
 )
 
-LIVEPATCH_CLIENT_FAILURE_WARNING = FormattedNamedMessage(
-    "livepatch-client-failure-warning",
-    t.gettext(
-        """\
-Error running canonical-livepatch status:
-{livepatch_error}"""
-    ),
-)
-
 REALTIME_FIPS_INCOMPATIBLE = NamedMessage(
     "realtime-fips-incompatible",
     t.gettext(
@@ -1918,7 +1828,9 @@ REALTIME_FIPS_UPDATES_INCOMPATIBLE = NamedMessage(
 )
 REALTIME_LIVEPATCH_INCOMPATIBLE = NamedMessage(
     "realtime-livepatch-incompatible",
-    t.gettext("Livepatch does not currently cover the Real-time kernel."),
+    t.gettext(
+        "Livepatch is not currently supported for the Real-time kernel."
+    ),
 )
 REALTIME_VARIANT_INCOMPATIBLE = FormattedNamedMessage(
     "realtime-variant-incompatible",
@@ -1929,17 +1841,6 @@ REALTIME_ERROR_INSTALL_ON_CONTAINER = NamedMessage(
     t.gettext("Cannot install Real-time kernel on a container."),
 )
 
-ROS_REQUIRES_ESM = NamedMessage(
-    "ros-requires-esm",
-    t.gettext("ROS packages assume ESM updates are enabled."),
-)
-ROS_UPDATES_REQUIRES_ROS = NamedMessage(
-    "ros-updates-requires-ros",
-    t.gettext(
-        "ROS bug-fix updates assume ROS security fix updates are enabled."
-    ),
-)
-
 UNATTENDED_UPGRADES_SYSTEMD_JOB_DISABLED = NamedMessage(
     "unattended-upgrades-systemd-job-disabled",
     t.gettext("apt-daily.timer jobs are not running"),
@@ -1957,6 +1858,19 @@ UNATTENDED_UPGRADES_UNINSTALLED = NamedMessage(
     t.gettext("unattended-upgrades package is not installed"),
 )
 
+LANDSCAPE_CLIENT_NOT_INSTALLED = NamedMessage(
+    "landscape-client-not-installed",
+    t.gettext("lanscape-client is not installed"),
+)
+LANDSCAPE_NOT_CONFIGURED = NamedMessage(
+    "landscape-not-configured",
+    t.gettext(
+        """\
+Landscape is installed but not configured.
+Run `sudo landscape-config` to set it up, or run `sudo pro disable landscape`\
+"""
+    ),
+)
 LANDSCAPE_NOT_REGISTERED = NamedMessage(
     "landscape-not-registered",
     t.gettext(
@@ -1969,9 +1883,16 @@ Run `sudo landscape-config` to register, or run `sudo pro disable landscape`\
 LANDSCAPE_SERVICE_NOT_ACTIVE = NamedMessage(
     "landscape-service-not-active",
     t.gettext(
-        "landscape-client is either not installed or installed but disabled."
+        """\
+Landscape is installed and configured and registered but not running.
+Run `sudo landscape-config` to start it, or run `sudo pro disable landscape`\
+"""
     ),
 )
+LANDSCAPE_CONFIG_FAILED = NamedMessage(
+    "landscape-config-failed",
+    t.gettext("""landscape-config command failed"""),
+)
 
 INVALID_SECURITY_ISSUE = FormattedNamedMessage(
     "invalid-security-issue",
@@ -1984,11 +1905,6 @@ USNs should follow the pattern USN-nnnn."""
 )
 
 
-GENERIC_UNKNOWN_ISSUE = NamedMessage(
-    "unknown-issue",
-    UNKNOWN_ERROR,
-)
-
 ###############################################################################
 #                              ERROR MESSAGES                                 #
 ###############################################################################
@@ -2106,13 +2022,12 @@ E_PROXY_AUTH_FAIL = NamedMessage(
     "proxy-auth-fail", t.gettext("Proxy authentication failed")
 )
 
-E_CONNECTIVITY_ERROR = FormattedNamedMessage(
+E_CONNECTIVITY_ERROR = NamedMessage(
     "connectivity-error",
     t.gettext(
         """\
-Failed to connect to {url}
-{cause_error}
-"""
+Failed to connect to authentication server
+Check your Internet connection and try again."""
     ),
 )
 
@@ -2235,9 +2150,8 @@ E_VALID_SERVICE_FAILURE_UNATTACHED = FormattedNamedMessage(
     "valid-service-failure-unattached",
     t.gettext(
         """\
-Cannot {{operation}} services when unattached - nothing to do.
-To use '{{valid_service}}' you need an Ubuntu Pro subscription.
-Personal and community subscriptions are available at no charge.
+To use '{{valid_service}}' you need an Ubuntu Pro subscription
+Personal and community subscriptions are available at no charge
 See {url}"""
     ).format(url=urls.PRO_HOME_PAGE),
 )
@@ -2259,16 +2173,6 @@ E_ENTITLEMENTS_NOT_ENABLED_ERROR = NamedMessage(
     t.gettext("failed to enable some services"),
 )
 
-E_ENTITLEMENT_NOT_ENABLED_ERROR = FormattedNamedMessage(
-    "entitlement-not-enabled",
-    t.gettext("failed to enable {service}"),
-)
-
-E_ENTITLEMENT_NOT_DISABLED_ERROR = FormattedNamedMessage(
-    "entitlement-not-disabled",
-    t.gettext("failed to disable {service}"),
-)
-
 E_ATTACH_FAILURE_DEFAULT_SERVICES = NamedMessage(
     "attach-failure-default-service",
     t.gettext(
@@ -2314,39 +2218,8 @@ E_INVALID_CONTRACT_DELTAS_SERVICE_TYPE = FormattedNamedMessage(
     t.gettext("Could not determine contract delta service type {orig} {new}"),
 )
 
-E_REQUIRED_SERVICE_STOPS_ENABLE = FormattedNamedMessage(
-    "required-service-stops-enable",
-    t.gettext(
-        """\
-Cannot enable {service_being_enabled} when {required_service} is disabled.
-"""
-    ),
-)
-E_INCOMPATIBLE_SERVICE_STOPS_ENABLE = FormattedNamedMessage(
-    "incompatible-service-stops-enable",
-    t.gettext(
-        """\
-Cannot enable {service_being_enabled} when \
-{incompatible_service} is enabled."""
-    ),
-)
-E_DEPENDENT_SERVICE_STOPS_DISABLE = FormattedNamedMessage(
-    "depedent-service-stops-disable",
-    t.gettext(
-        """\
-Cannot disable {service_being_disabled} when {dependent_service} is enabled.
-"""
-    ),
-)
-
 E_INVALID_PRO_IMAGE = FormattedNamedMessage(
-    name="invalid-pro-image",
-    msg=t.gettext(
-        """\
-Failed to identify this image as a valid Ubuntu Pro image.
-Details:
-{error_msg}"""
-    ),
+    name="invalid-pro-image", msg=t.gettext("Error on Pro Image:\n{error_msg}")
 )
 
 E_CLOUD_METADATA_ERROR = FormattedNamedMessage(
@@ -2404,11 +2277,6 @@ E_INVALID_FILE_FORMAT = FormattedNamedMessage(
     msg=t.gettext("{file_name} is not valid {file_format}"),
 )
 
-E_INVALID_FILE_ENCODING = FormattedNamedMessage(
-    name="invalid-file-encoding",
-    msg=t.gettext("{file_name} is not encoded as {file_encoding}"),
-)
-
 E_ERROR_PARSING_VERSION_OS_RELEASE = FormattedNamedMessage(
     "error-parsing-version-os-release",
     t.gettext(
@@ -2511,11 +2379,6 @@ E_CLI_VALID_CHOICES = FormattedNamedMessage(
     "invalid-arg-choice", "\n" + t.gettext("{arg} must be one of: {choices}")
 )
 
-E_CLI_EMPTY_CONFIG_VALUE = FormattedNamedMessage(
-    "empty-value",
-    t.gettext("Empty value provided for {arg}."),
-)
-
 E_CLI_EXPECTED_FORMAT = FormattedNamedMessage(
     "generic-invalid-format",
     "\n" + t.gettext("Expected {expected} but found: {actual}"),
@@ -2558,11 +2421,6 @@ E_API_ERROR_ARGS_AND_DATA_TOGETHER = NamedMessage(
     t.gettext("Cannot provide both --args and --data at the same time"),
 )
 
-E_PROMPT_DENIED = NamedMessage(
-    "prompt-denied",
-    t.gettext("Operation cancelled by user"),
-)
-
 E_LOCK_HELD_ERROR = FormattedNamedMessage(
     "lock-held-error",
     t.gettext(
@@ -2664,46 +2522,3 @@ E_INCORRECT_ENUM_VALUE_ERROR_MESSAGE = FormattedNamedMessage(
 E_PYCURL_CA_CERTIFICATES = NamedMessage(
     "pycurl-ca-certificates-error", "Problem reading SSL CA certificates"
 )
-
-E_UPDATING_ESM_CACHE = FormattedNamedMessage(
-    "error-updating-esm-cache",
-    t.gettext("Error updating ESM services cache: {error}"),
-)
-
-E_ENTITLEMENTS_APT_DIRECTIVES_ARE_NOT_UNIQUE = FormattedNamedMessage(
-    "entitlements-apt-directives-are-not-unique",
-    t.gettext(
-        "There is a problem with the resource directives provided by {url}\n"
-        "These entitlements: {names} are sharing the following directives\n"
-        " - APT url: {apt_url}\n - Suite: {suite}\n"
-        "These directives need to be unique for every entitlement."
-    ),
-)
-
-E_LANDSCAPE_CONFIG_FAILED = NamedMessage(
-    "landscape-config-failed",
-    t.gettext("landscape-config command failed"),
-)
-
-E_NON_INTERACTIVE_KERNEL_PURGE_DISALLOWED = NamedMessage(
-    "non-interactive-kernel-purge-disallowed",
-    t.gettext(
-        "You must use the pro command to purge a service that has installed a "
-        "kernel"
-    ),
-)
-
-E_NOT_SUPPORTED = NamedMessage(
-    "not-supported",
-    t.gettext("The operation is not supported"),
-)
-
-E_CONTRACT_EXPIRED = NamedMessage(
-    "contract-expired",
-    CONTRACT_EXPIRED,
-)
-
-E_INVALID_URL = FormattedNamedMessage(
-    "invalid-url",
-    t.gettext("Invalid URL: {url}"),
-)
