@@ -5,7 +5,7 @@ import os
 import pathlib
 import re
 import stat
-import subprocess  # nosec B404
+import subprocess
 import tempfile
 import time
 import uuid
@@ -44,14 +44,7 @@ RE_KERNEL_UNAME = (
 )
 
 DistroInfo = NamedTuple(
-    "DistroInfo",
-    [
-        ("eol", datetime.date),
-        ("eol_esm", datetime.date),
-        ("series", str),
-        ("release", str),
-        ("series_codename", str),
-    ],
+    "DistroInfo", [("eol", datetime.date), ("eol_esm", datetime.date)]
 )
 
 KernelInfo = NamedTuple(
@@ -78,7 +71,6 @@ ReleaseInfo = NamedTuple(
         ("pretty_version", str),
     ],
 )
-
 CpuInfo = NamedTuple(
     "CpuInfo",
     [
@@ -282,16 +274,14 @@ def get_machine_id(cfg) -> str:
     We first check for the machine-id in machine-token.json before
     looking at the system file.
     """
-    from uaclient.files import machine_token
     from uaclient.files.state_files import machine_id_file
 
-    machine_token_file = machine_token.get_machine_token_file()
-    if machine_token_file.machine_token:
-        machine_id = machine_token_file.machine_token.get(
-            "machineTokenInfo", {}
-        ).get("machineId")
-        if machine_id:
-            return machine_id
+    if cfg.machine_token:
+        cfg_machine_id = cfg.machine_token.get("machineTokenInfo", {}).get(
+            "machineId"
+        )
+        if cfg_machine_id:
+            return cfg_machine_id
 
     fallback_machine_id = machine_id_file.read()
 
@@ -443,9 +433,6 @@ def get_distro_info(series: str) -> DistroInfo:
             else:
                 eol_esm = values[7] if "LTS" in values[0] else values[5]
             return DistroInfo(
-                release=values[0],
-                series_codename=values[1],
-                series=values[2],
                 eol=datetime.datetime.strptime(values[5], "%Y-%m-%d").date(),
                 eol_esm=datetime.datetime.strptime(eol_esm, "%Y-%m-%d").date(),
             )
@@ -629,23 +616,22 @@ def _subp(
 
     stdout = None
     stderr = None
-    set_lang = {}
-
     if pipe_stdouterr:
         stdout = subprocess.PIPE
         stderr = subprocess.PIPE
-        # Set LANG to avoid non-utf8 when we pipe the handlers
-        set_lang = {"LANG": "C.UTF-8", "LC_ALL": "C.UTF-8"}
 
-    if override_env_vars is None:
-        override_env_vars = {}
-    merged_env = {**os.environ, **set_lang, **override_env_vars}
+    # If env is None, subprocess.Popen will use the process environment
+    # variables by default, as stated here:
+    # https://docs.python.org/3.5/library/subprocess.html?highlight=subprocess#popen-constructor
+    merged_env = None
+    if override_env_vars:
+        merged_env = {**os.environ, **override_env_vars}
 
     if rcs is None:
         rcs = [0]
     redacted_cmd = util.redact_sensitive_logs(" ".join(args))
     try:
-        proc = subprocess.Popen(  # nosec B603
+        proc = subprocess.Popen(
             bytes_args,
             stdout=stdout,
             stderr=stderr,
@@ -654,8 +640,8 @@ def _subp(
         (out, err) = proc.communicate(timeout=timeout)
     except OSError:
         try:
-            out_result = out.decode("utf-8", errors="ignore") if out else ""
-            err_result = err.decode("utf-8", errors="ignore") if err else ""
+            out_result = out.decode("utf-8") if out else ""
+            err_result = err.decode("utf-8") if err else ""
             raise exceptions.ProcessExecutionError(
                 cmd=redacted_cmd,
                 exit_code=proc.returncode,
@@ -665,8 +651,8 @@ def _subp(
         except UnboundLocalError:
             raise exceptions.ProcessExecutionError(cmd=redacted_cmd)
 
-    out_result = out.decode("utf-8", errors="ignore") if out else ""
-    err_result = err.decode("utf-8", errors="ignore") if err else ""
+    out_result = out.decode("utf-8") if out else ""
+    err_result = err.decode("utf-8") if err else ""
     if proc.returncode not in rcs:
         raise exceptions.ProcessExecutionError(
             cmd=redacted_cmd,
