@@ -14,7 +14,6 @@ from functools import lru_cache, wraps
 from http.client import HTTPMessage
 from typing import (
     Any,
-    Callable,
     Dict,
     List,
     Mapping,
@@ -27,7 +26,8 @@ from typing import (
 from urllib import error, request
 from urllib.parse import urlparse
 
-from uaclient import exceptions, status
+from uaclient import event_logger, exceptions, messages, status
+from uaclient.types import MessagingOperations
 
 REBOOT_FILE_CHECK_PATH = "/var/run/reboot-required"
 REBOOT_PKGS_FILE_PATH = "/var/run/reboot-required.pkgs"
@@ -44,6 +44,8 @@ PROXY_VALIDATION_APT_HTTPS_URL = "https://esm.ubuntu.com"
 PROXY_VALIDATION_SNAP_HTTP_URL = "http://api.snapcraft.io"
 PROXY_VALIDATION_SNAP_HTTPS_URL = "https://api.snapcraft.io"
 
+event = event_logger.get_event_logger()
+
 
 class LogFormatter(logging.Formatter):
 
@@ -57,49 +59,6 @@ class LogFormatter(logging.Formatter):
         return logging.Formatter(log_fmt).format(record)
 
 
-class UrlError(IOError):
-    def __init__(
-        self,
-        cause: error.URLError,
-        code: Optional[int] = None,
-        headers: Optional[Dict[str, str]] = None,
-        url: Optional[str] = None,
-    ):
-        if getattr(cause, "reason", None):
-            cause_error = str(cause.reason)
-        else:
-            cause_error = str(cause)
-        super().__init__(cause_error)
-        self.code = code
-        self.headers = headers
-        if self.headers is None:
-            self.headers = {}
-        self.url = url
-
-
-class ProcessExecutionError(IOError):
-    def __init__(
-        self,
-        cmd: str,
-        exit_code: Optional[int] = None,
-        stdout: str = "",
-        stderr: str = "",
-    ) -> None:
-        self.stdout = stdout
-        self.stderr = stderr
-        self.exit_code = exit_code
-        if not exit_code:
-            message_tmpl = "Invalid command specified '{cmd}'."
-        else:
-            message_tmpl = (
-                "Failed running command '{cmd}' [exit({exit_code})]."
-                " Message: {stderr}"
-            )
-        super().__init__(
-            message_tmpl.format(cmd=cmd, stderr=stderr, exit_code=exit_code)
-        )
-
-
 class DatetimeAwareJSONEncoder(json.JSONEncoder):
     """A json.JSONEncoder subclass that writes out isoformat'd datetimes."""
 
@@ -391,7 +350,7 @@ def is_container(run_path: str = "/run") -> bool:
     try:
         subp(["ischroot"])
         return False
-    except ProcessExecutionError:
+    except exceptions.ProcessExecutionError:
         pass
 
     try:
@@ -464,11 +423,11 @@ def prompt_choices(msg: str = "", valid_choices: List[str] = []) -> str:
         value, ", ".join([choice.upper() for choice in valid_choices])
     )
     while True:
-        print(msg)
+        event.info(msg)
         value = input("> ").lower()
         if value in valid_choices:
             break
-        print(error_msg)
+        event.info(error_msg)
     return value
 
 
@@ -618,16 +577,16 @@ def _subp(
         (out, err) = proc.communicate(timeout=timeout)
     except OSError:
         try:
-            raise ProcessExecutionError(
+            raise exceptions.ProcessExecutionError(
                 cmd=redacted_cmd,
                 exit_code=proc.returncode,
                 stdout=out.decode("utf-8"),
                 stderr=err.decode("utf-8"),
             )
         except UnboundLocalError:
-            raise ProcessExecutionError(cmd=redacted_cmd)
+            raise exceptions.ProcessExecutionError(cmd=redacted_cmd)
     if proc.returncode not in rcs:
-        raise ProcessExecutionError(
+        raise exceptions.ProcessExecutionError(
             cmd=redacted_cmd,
             exit_code=proc.returncode,
             stdout=out.decode("utf-8"),
@@ -676,7 +635,7 @@ def subp(
         try:
             out, err = _subp(args, rcs, capture, timeout, env=env)
             break
-        except ProcessExecutionError as e:
+        except exceptions.ProcessExecutionError as e:
             if capture:
                 logging.debug(redact_sensitive_logs(str(e)))
                 msg = "Stderr: {}\nStdout: {}".format(e.stderr, e.stdout)
@@ -760,7 +719,7 @@ def is_config_value_true(config: Dict[str, Any], path_to_value: str):
         return False
     else:
         raise exceptions.UserFacingError(
-            status.ERROR_INVALID_CONFIG_VALUE.format(
+            messages.ERROR_INVALID_CONFIG_VALUE.format(
                 path_to_value=path_to_value,
                 expected_value="boolean string: true or false",
                 value=value_str,
@@ -833,13 +792,11 @@ def is_installed(package_name: str) -> bool:
     try:
         out, _ = subp(["dpkg", "-l", package_name])
         return "ii  {} ".format(package_name) in out
-    except ProcessExecutionError:
+    except exceptions.ProcessExecutionError:
         return False
 
 
-def handle_message_operations(
-    msg_ops: List[Union[str, Tuple[Callable, Dict]]],
-) -> bool:
+def handle_message_operations(msg_ops: Optional[MessagingOperations],) -> bool:
     """Emit messages to the console for user interaction
 
     :param msg_op: A list of strings or tuples. Any string items are printed.
@@ -849,9 +806,12 @@ def handle_message_operations(
 
     :return: True upon success, False on failure.
     """
+    if not msg_ops:
+        return True
+
     for msg_op in msg_ops:
         if isinstance(msg_op, str):
-            print(msg_op)
+            event.info(msg_op)
         else:  # Then we are a callable and dict of args
             functor, args = msg_op
             if not functor(**args):
@@ -921,9 +881,7 @@ def validate_proxy(
         return None
 
     if not is_service_url(proxy):
-        raise exceptions.UserFacingError(
-            status.MESSAGE_NOT_SETTING_PROXY_INVALID_URL.format(proxy=proxy)
-        )
+        raise exceptions.ProxyInvalidUrl(proxy)
 
     req = request.Request(test_url, method="HEAD")
     proxy_handler = request.ProxyHandler({protocol: proxy})
@@ -936,13 +894,11 @@ def validate_proxy(
         with disable_log_to_console():
             msg = getattr(e, "reason", str(e))
             logging.error(
-                status.MESSAGE_ERROR_USING_PROXY.format(
+                messages.ERROR_USING_PROXY.format(
                     proxy=proxy, test_url=test_url, error=msg
                 )
             )
-        raise exceptions.UserFacingError(
-            status.MESSAGE_NOT_SETTING_PROXY_NOT_WORKING.format(proxy=proxy)
-        )
+        raise exceptions.ProxyNotWorkingError(proxy)
 
 
 def handle_unicode_characters(message: str) -> str:
@@ -962,7 +918,7 @@ def handle_unicode_characters(message: str) -> str:
         # Remove our unicode success/failure marks if we aren't going to be
         # writing to a utf-8 output; see
         # https://github.com/CanonicalLtd/ubuntu-advantage-client/issues/1463
-        message = message.replace(status.OKGREEN_CHECK + " ", "")
-        message = message.replace(status.FAIL_X + " ", "")
+        message = message.replace(messages.OKGREEN_CHECK + " ", "")
+        message = message.replace(messages.FAIL_X + " ", "")
 
     return message
