@@ -1,12 +1,15 @@
 import logging
 import sys
 
+from uaclient import util
 from uaclient.messages import BROKEN_YAML_MODULE, MISSING_YAML_MODULE
 
+LOG = logging.getLogger(util.replace_top_level_logger_name(__name__))
+
 try:
     import yaml
 except ImportError:
-    logging.error(MISSING_YAML_MODULE.msg)
+    LOG.error(MISSING_YAML_MODULE.msg)
     sys.exit(1)
 
 
@@ -14,7 +17,7 @@ def safe_load(stream):
     try:
         return yaml.safe_load(stream)
     except AttributeError:
-        logging.error(BROKEN_YAML_MODULE.format(path=yaml.__path__).msg)
+        LOG.error(BROKEN_YAML_MODULE.format(path=yaml.__path__).msg)
         sys.exit(1)
 
 
@@ -22,7 +25,7 @@ def safe_dump(data, stream=None, **kwargs):
     try:
         return yaml.safe_dump(data, stream, **kwargs)
     except AttributeError:
-        logging.error(BROKEN_YAML_MODULE.format(path=yaml.__path__).msg)
+        LOG.error(BROKEN_YAML_MODULE.format(path=yaml.__path__).msg)
         sys.exit(1)
 
 
