@@ -3,7 +3,7 @@
 import logging
 from datetime import datetime, timedelta, timezone
 
-from uaclient import apt, defaults
+from uaclient import apt
 from uaclient.apt_news import update_apt_news
 from uaclient.config import UAConfig
 from uaclient.daemon import setup_logging
@@ -22,13 +22,7 @@ def main(cfg: UAConfig):
 
 
 if __name__ == "__main__":
-    setup_logging(
-        logging.INFO,
-        logging.DEBUG,
-        defaults.CONFIG_DEFAULTS["log_file"],
-        logger=logging.getLogger(),
-    )
-    cfg = UAConfig()
+    cfg = UAConfig(root_mode=True)
     setup_logging(
         logging.INFO,
         logging.DEBUG,
