@@ -11,7 +11,6 @@ from optparse import OptionParser
 import re
 import gettext
 import distro_info
-import subprocess
 
 
 SYNAPTIC_PINFILE = "/var/lib/synaptic/preferences"
@@ -42,22 +41,9 @@ def _get_info_from_os_release(key):
         )
 
 
-def _get_output_from_lsb_release(lsb_option):
-    " get info from lsb_release output "
-    return subprocess.check_output(
-        ["lsb_release", lsb_option, "-s"], universal_newlines=True
-    ).strip()
-
-
 def get_distro():
     " get distro name "
-    try:
-        return _get_info_from_os_release(key="UBUNTU_CODENAME")
-    except Exception:
-        # If the system does not have os-release file or does not have the
-        # required entry in it, we will get the distro name from lsb_release
-        # command
-        return _get_output_from_lsb_release("-c")
+    return _get_info_from_os_release(key="UBUNTU_CODENAME")
 
 
 DISTRO = get_distro()
@@ -78,13 +64,7 @@ def _handleException(type, value, tb):
 
 def get_distro_version():
     " get distro version "
-    try:
-        return _get_info_from_os_release(key="VERSION_ID").replace('"', "")
-    except Exception:
-        # If the system does not have os-release file or does not have the
-        # required entry in it, we will get the distro name from lsb_release
-        # command
-        return _get_output_from_lsb_release("-r")
+    return _get_info_from_os_release(key="VERSION_ID").replace('"', "")
 
 
 def clean(cache, depcache):
@@ -126,10 +106,12 @@ def _isESMUpgrade(ver, esm_origin):
 
 
 def isESMAppsUpgrade(ver):
+    " check if the given version is a ESM Apps upgrade "
     return _isESMUpgrade(ver, esm_origin=ESM_APPS_ORIGIN)
 
 
 def isESMInfraUpgrade(ver):
+    " check if the given version is a ESM Infra upgrade "
     return _isESMUpgrade(ver, esm_origin=ESM_INFRA_ORIGIN)
 
 
@@ -141,6 +123,7 @@ def write_package_names(outstream, cache, depcache):
 
 
 def is_esm_distro():
+    " check if the current distro is ESM or not "
     ubuntu_distro = distro_info.UbuntuDistroInfo()
 
     is_esm_supported = bool(
@@ -155,10 +138,12 @@ def is_esm_distro():
 
 
 def is_lts_distro():
+    " check if the current distro is LTS or not"
     return distro_info.UbuntuDistroInfo().is_lts(DISTRO)
 
 
 def _output_esm_package_count(outstream, service_type, esm_pkg_count):
+    " output the number of packages upgrades related to esm service "
     if esm_pkg_count > 0:
         outstream.write("\n")
         outstream.write(gettext.dngettext("update-notifier",
@@ -175,6 +160,7 @@ def _output_esm_package_count(outstream, service_type, esm_pkg_count):
 def _output_esm_package_alert(
     outstream, service_type, disabled_pkg_count, is_esm=False
 ):
+    " output the number of upgradable packages if esm service was enabled "
     outstream.write("\n")
     if disabled_pkg_count > 0:
         outstream.write("\n")
@@ -239,7 +225,7 @@ def write_human_readable_summary(outstream, upgrades, security_updates,
                                  disabled_esm_infra_updates,
                                  disabled_esm_apps_updates):
 
-    " write out human summary to outstream "
+    " write out human summary summary to outstream "
     esm_distro = is_esm_distro()
     lts_distro = is_lts_distro()
 
@@ -321,14 +307,17 @@ def has_disabled_esm_security_update(depcache, pkg, esm_origin):
 
 
 def has_disabled_esm_apps_security_update(depcache, pkg):
+    " check if we have a disabled ESM Apps security update "
     return has_disabled_esm_security_update(depcache, pkg, ESM_APPS_ORIGIN)
 
 
 def has_disabled_esm_infra_security_update(depcache, pkg):
+    " check if we have a disabled ESM Infra security update "
     return has_disabled_esm_security_update(depcache, pkg, ESM_INFRA_ORIGIN)
 
 
 def has_esm_service(cache, depcache, esm_origin):
+    " check if we have an enabled ESM service in the machine "
     have_esm = None
     for file in cache.file_list:
         origin = file.origin
