@@ -260,18 +260,6 @@ def check_attachment_for_errors(report, attachment):
                 # File these reports on the initramfs-tools package instead
                 report['SourcePackage'] = 'initramfs-tools'
 
-        if report['Package'] in ['emacs22', 'emacs23', 'emacs-snapshot', 'xemacs21']:
-            # emacs add-on packages trigger byte compilation, which might fail
-            # we are very interested in reading the compilation log to determine
-            # where to reassign this report to
-            regex = r'^!! Byte-compilation for x?emacs\S+ failed!'
-            if attachment in report and re.search(regex, log_file, re.MULTILINE):
-                for line in log_file.split('\n'):
-                    m = re.search(r'^!! and attach the file (\S+)', line)
-                    if m:
-                        path = m.group(1)
-                        apport.hookutils.attach_file_if_exists(report, path)
-
         if report['Package'].startswith('linux-image-') and attachment in report:
             # /etc/kernel/*.d failures from kernel package postinst
             m = re.search(r'^run-parts: (/etc/kernel/\S+\.d/\S+) exited with return code \d+', log_file, re.MULTILINE)
