@@ -1,8 +1,9 @@
 # Generated file
 NETPLAN_FEATURE_FLAGS = [
+    "dbus-config",
     "generate-just-in-time",
     "generated-supplicant",
-    "dbus-config",
+    "default-routes",
     "auth-phase2",
     "dhcp-use-domains",
     "ipv6-mtu",
@@ -10,4 +11,5 @@ NETPLAN_FEATURE_FLAGS = [
     "sriov",
     "openvswitch",
     "activation-mode",
+    "eswitch-mode",
 ]
