@@ -25,13 +25,18 @@ import sys
 import glob
 import subprocess
 import shutil
+import netifaces
+import time
 
 import netplan.cli.utils as utils
 from netplan.configmanager import ConfigManager, ConfigurationError
 from netplan.cli.sriov import apply_sriov_config
-from netplan.cli.ovs import apply_ovs_cleanup
+from netplan.cli.ovs import OvsDbServerNotRunning, apply_ovs_cleanup
 
-import netifaces
+
+OVS_CLEANUP_SERVICE = 'netplan-ovs-cleanup.service'
+
+IF_NAMESIZE = 16
 
 
 class NetplanApply(utils.NetplanCommand):
@@ -42,20 +47,25 @@ class NetplanApply(utils.NetplanCommand):
                          leaf=True)
         self.sriov_only = False
         self.only_ovs_cleanup = False
+        self.state = None  # to be filled by the '--state' argument
 
     def run(self):  # pragma: nocover (covered in autopkgtest)
         self.parser.add_argument('--sriov-only', action='store_true',
                                  help='Only apply SR-IOV related configuration and exit')
         self.parser.add_argument('--only-ovs-cleanup', action='store_true',
                                  help='Only clean up old OpenVSwitch interfaces and exit')
+        self.parser.add_argument('--state',
+                                 help='Directory containing previous YAML configuration')
 
         self.func = self.command_apply
 
         self.parse_args()
         self.run_command()
 
-    def command_apply(self, run_generate=True, sync=False, exit_on_error=True):  # pragma: nocover (covered in autopkgtest)
+    def command_apply(self, run_generate=True, sync=False, exit_on_error=True, state_dir=None):  # pragma: nocover
         config_manager = ConfigManager()
+        if state_dir:
+            self.state = state_dir
 
         # For certain use-cases, we might want to only apply specific configuration.
         # If we only need SR-IOV configuration, do that and exit early.
@@ -74,6 +84,11 @@ class NetplanApply(utils.NetplanCommand):
             busctl = shutil.which("busctl")
             if busctl is None:
                 raise RuntimeError("missing busctl utility")
+            # XXX: DO NOT TOUCH or change this API call, it is used by snapd to communicate
+            #      using core20 netplan binary/client/CLI on core18 base systems. Any change
+            #      must be agreed upon with the snapd team, so we don't break support for
+            #      base systems running older netplan versions.
+            #      https://github.com/snapcore/snapd/pull/5915
             res = subprocess.call([busctl, "call", "--quiet", "--system",
                                    "io.netplan.Netplan",  # the service
                                    "/io/netplan/Netplan",  # the object
@@ -87,9 +102,9 @@ class NetplanApply(utils.NetplanCommand):
                 elif res == 130:
                     raise PermissionError(
                         "failed to communicate with dbus service")
-                elif res == 1:
+                else:
                     raise RuntimeError(
-                        "failed to communicate with dbus service")
+                        "failed to communicate with dbus service: error %s" % res)
             else:
                 return
 
@@ -143,13 +158,13 @@ class NetplanApply(utils.NetplanCommand):
         if not restart_nm and old_files_nm:
             restart_nm = True
 
+        # Running 'systemctl daemon-reload' will re-run the netplan systemd generator,
+        # so let's make sure we only run it iff we're willing to run 'netplan generate'
+        if run_generate:
+            utils.systemctl_daemon_reload()
         # stop backends
         if restart_networkd:
-            logging.debug('netplan generated networkd configuration changed, restarting networkd')
-            # Running 'systemctl daemon-reload' will re-run the netplan systemd generator,
-            # so let's make sure we only run it iff we're willing to run 'netplan generate'
-            if run_generate:
-                utils.systemctl_daemon_reload()
+            logging.debug('netplan generated networkd configuration changed, reloading networkd')
             # Clean up any old netplan related OVS ports/bonds/bridges, if applicable
             NetplanApply.process_ovs_cleanup(config_manager, old_files_ovs, restart_ovs, exit_on_error)
             wpa_services = ['netplan-wpa-*.service']
@@ -157,7 +172,7 @@ class NetplanApply(utils.NetplanCommand):
             # upgraded system, we need to make sure to stop those.
             if utils.systemctl_is_active('netplan-wpa@*.service'):
                 wpa_services.insert(0, 'netplan-wpa@*.service')
-            utils.systemctl_networkd('stop', sync=sync, extra_services=wpa_services)
+            utils.systemctl('stop', wpa_services, sync=sync)
         else:
             logging.debug('no netplan generated networkd configuration exists')
 
@@ -185,6 +200,14 @@ class NetplanApply(utils.NetplanCommand):
         # for now, only applies to non-virtual (real) devices.
         config_manager.parse()
         changes = NetplanApply.process_link_changes(devices, config_manager)
+        # delete virtual interfaces that have been defined in a previous state
+        # but are not configured anymore in the current YAML
+        if self.state:
+            cm = ConfigManager(self.state)
+            cm.parse()  # get previous configuration state
+            prev_links = cm.virtual_interfaces.keys()
+            curr_links = config_manager.virtual_interfaces.keys()
+            NetplanApply.clear_virtual_links(prev_links, curr_links, devices)
 
         # if the interface is up, we can still apply some .link file changes
         # but we cannot apply the interface rename via udev, as it won't touch
@@ -200,13 +223,25 @@ class NetplanApply(utils.NetplanCommand):
                                        '/sys/class/net/' + device],
                                       stdout=subprocess.DEVNULL,
                                       stderr=subprocess.DEVNULL)
+                subprocess.check_call(['udevadm', 'test',
+                                       '/sys/class/net/' + device],
+                                      stdout=subprocess.DEVNULL,
+                                      stderr=subprocess.DEVNULL)
             except subprocess.CalledProcessError:
                 logging.debug('Ignoring device without syspath: %s', device)
 
+        devices_after_udev = netifaces.interfaces()
         # apply some more changes manually
         for iface, settings in changes.items():
             # rename non-critical network interfaces
-            if settings.get('name'):
+            new_name = settings.get('name')
+            if new_name:
+                if len(new_name) >= IF_NAMESIZE:
+                    logging.warning('Interface name {} is too long. {} will not be renamed'.format(new_name, iface))
+                    continue
+                if iface in devices and new_name in devices_after_udev:
+                    logging.debug('Interface rename {} -> {} already happened.'.format(iface, new_name))
+                    continue  # re-name already happened via 'udevadm test'
                 # bring down the interface, using its current (matched) interface name
                 subprocess.check_call(['ip', 'link', 'set', 'dev', iface, 'down'],
                                       stdout=subprocess.DEVNULL,
@@ -218,33 +253,72 @@ class NetplanApply(utils.NetplanCommand):
                                       stdout=subprocess.DEVNULL,
                                       stderr=subprocess.DEVNULL)
 
+        # Reloading of udev rules happens during 'netplan generate' already
+        # subprocess.check_call(['udevadm', 'control', '--reload-rules'])
+        subprocess.check_call(['udevadm', 'trigger', '--attr-match=subsystem=net'])
         subprocess.check_call(['udevadm', 'settle'])
 
         # apply any SR-IOV related changes, if applicable
         NetplanApply.process_sriov_config(config_manager, exit_on_error)
 
+        # (re)set global regulatory domain
+        if os.path.exists('/run/systemd/system/netplan-regdom.service'):
+            utils.systemctl('start', ['netplan-regdom.service'])
         # (re)start backends
         if restart_networkd:
             netplan_wpa = [os.path.basename(f) for f in glob.glob('/run/systemd/system/*.wants/netplan-wpa-*.service')]
-            netplan_ovs = [os.path.basename(f) for f in glob.glob('/run/systemd/system/*.wants/netplan-ovs-*.service')]
+            # exclude the special 'netplan-ovs-cleanup.service' unit
+            netplan_ovs = [os.path.basename(f) for f in glob.glob('/run/systemd/system/*.wants/netplan-ovs-*.service')
+                           if not f.endswith('/' + OVS_CLEANUP_SERVICE)]
             # Run 'systemctl start' command synchronously, to avoid race conditions
             # with 'oneshot' systemd service units, e.g. netplan-ovs-*.service.
-            utils.systemctl_networkd('start', sync=True, extra_services=netplan_wpa + netplan_ovs)
+            try:
+                utils.networkctl_reload()
+                utils.networkctl_reconfigure(utils.networkd_interfaces())
+            except subprocess.CalledProcessError:
+                # (re-)start systemd-networkd if it is not running, yet
+                logging.warning('Falling back to a hard restart of systemd-networkd.service')
+                utils.systemctl('restart', ['systemd-networkd.service'], sync=True)
+            # 1st: execute OVS cleanup, to avoid races while applying OVS config
+            utils.systemctl('start', [OVS_CLEANUP_SERVICE], sync=True)
+            # 2nd: start all other services
+            utils.systemctl('start', netplan_wpa + netplan_ovs, sync=True)
         if restart_nm:
             # Flush all IP addresses of NM managed interfaces, to avoid NM creating
             # new, non netplan-* connection profiles, using the existing IPs.
             for iface in utils.nm_interfaces(restart_nm_glob, devices):
                 utils.ip_addr_flush(iface)
+            # clear NM state, especially the [device].managed=true config, as that might have been
+            # re-set via an udev rule setting "NM_UNMANAGED=1"
+            shutil.rmtree('/run/NetworkManager/devices', ignore_errors=True)
             utils.systemctl_network_manager('start', sync=sync)
+            if sync:
+                # 'nmcli' could be /usr/bin/nmcli or
+                # /snap/bin/nmcli -> /snap/bin/network-manager.nmcli
+                cmd = ['nmcli', 'general', 'status']
+                # wait a bit for 'connected (site/local-only)' or
+                # 'connected' to appear in 'nmcli general' STATE
+                for _ in range(10):
+                    out = subprocess.run(cmd, capture_output=True, text=True)
+                    # Handle nmcli's "not running" return code (8) gracefully,
+                    # giving some more time for NetworkManager startup
+                    if out.returncode == 8:
+                        time.sleep(1)
+                        continue
+                    if '\nconnected' in str(out.stdout):
+                        break
+                    time.sleep(0.5)
 
     @staticmethod
-    def is_composite_member(composites, phy):  # pragma: nocover (covered in autopkgtest)
+    def is_composite_member(composites, phy):
         """
         Is this physical interface a member of a 'composite' virtual
         interface? (bond, bridge)
         """
         for composite in composites:
             for _, settings in composite.items():
+                if not type(settings) is dict:
+                    continue
                 members = settings.get('interfaces', [])
                 for iface in members:
                     if iface == phy:
@@ -253,45 +327,67 @@ class NetplanApply(utils.NetplanCommand):
         return False
 
     @staticmethod
-    def process_link_changes(interfaces, config_manager):  # pragma: nocover (covered in autopkgtest)
+    def clear_virtual_links(prev_links, curr_links, devices=[]):
+        """
+        Calculate the delta of virtual links. And remove the links that were
+        dropped from the YAML config, if they were not dropped by the backend
+        already.
+        We can make use of the netplan netdef ids, as those equal the interface
+        name for virtual links.
+        """
+        if not devices:
+            logging.warning('Cannot clear virtual links: no network interfaces provided.')
+            return []
+
+        dropped_interfaces = list(set(prev_links) - set(curr_links))
+        # some interfaces might have been cleaned up already, e.g. by the
+        # NetworkManager backend
+        interfaces_to_clear = list(set(dropped_interfaces).intersection(devices))
+        for link in interfaces_to_clear:
+            try:
+                cmd = ['ip', 'link', 'delete', 'dev', link]
+                subprocess.check_call(cmd)
+            except subprocess.CalledProcessError:
+                logging.warning('Could not delete interface {}'.format(link))
+
+        return dropped_interfaces
+
+    @staticmethod
+    def process_link_changes(interfaces, config_manager: ConfigManager):  # pragma: nocover (covered in autopkgtest)
         """
         Go through the pending changes and pick what needs special handling.
         Only applies to non-critical interfaces which can be safely updated.
         """
 
         changes = {}
-        phys = dict(config_manager.physical_interfaces)
         composite_interfaces = [config_manager.bridges, config_manager.bonds]
 
         # Find physical interfaces which need a rename
         # But do not rename virtual interfaces
-        for phy, settings in phys.items():
-            if not settings or not isinstance(settings, dict):
-                continue  # Skip special values, like "renderer: ..."
-            newname = settings.get('set-name')
+        for netdef in config_manager.physical_interfaces.values():
+            newname = netdef.set_name
             if not newname:
                 continue  # Skip if no new name needs to be set
-            match = settings.get('match')
-            if not match:
+            if not netdef.has_match:
                 continue  # Skip if no match for current name is given
-            if NetplanApply.is_composite_member(composite_interfaces, phy):
-                logging.debug('Skipping composite member {}'.format(phy))
+            if NetplanApply.is_composite_member(composite_interfaces, netdef.id):
+                logging.debug('Skipping composite member {}'.format(netdef.id))
                 # do not rename members of virtual devices. MAC addresses
                 # may be the same for all interface members.
                 continue
             # Find current name of the interface, according to match conditions and globs (name, mac, driver)
-            current_iface_name = utils.find_matching_iface(interfaces, match)
+            current_iface_name = utils.find_matching_iface(interfaces, netdef)
             if not current_iface_name:
-                logging.warning('Cannot find unique matching interface for {}: {}'.format(phy, match))
+                logging.warning('Cannot find unique matching interface for {}'.format(netdef.id))
                 continue
             if current_iface_name == newname:
                 # Skip interface if it already has the correct name
                 logging.debug('Skipping correctly named interface: {}'.format(newname))
                 continue
-            if settings.get('critical', False):
+            if netdef.critical:
                 # Skip interfaces defined as critical, as we should not take them down in order to rename
                 logging.warning('Cannot rename {} ({} -> {}) at runtime (needs reboot), due to being critical'
-                                .format(phy, current_iface_name, newname))
+                                .format(netdef.id, current_iface_name, newname))
                 continue
 
             # record the interface rename change
@@ -304,7 +400,7 @@ class NetplanApply(utils.NetplanCommand):
     def process_sriov_config(config_manager, exit_on_error=True):  # pragma: nocover (covered in autopkgtest)
         try:
             apply_sriov_config(config_manager)
-        except (ConfigurationError, RuntimeError) as e:
+        except utils.config_errors as e:
             logging.error(str(e))
             if exit_on_error:
                 sys.exit(1)
@@ -317,3 +413,5 @@ class NetplanApply(utils.NetplanCommand):
             logging.error(str(e))
             if exit_on_error:
                 sys.exit(1)
+        except OvsDbServerNotRunning as e:
+            logging.warning('Cannot call Open vSwitch: {}.'.format(e))
